/* global define, require */
define('WS.Data/Adapter/Sbis', [
   'WS.Data/Adapter/Abstract',
   'WS.Data/Adapter/SbisTable',
   'WS.Data/Adapter/SbisRecord',
   'WS.Data/Adapter/SbisFieldType',
   'WS.Data/Di'
], function (
   Abstract,
   SbisTable,
   SbisRecord,
   FIELD_TYPE,
   Di
) {
   'use strict';

   /**
    * Адаптер для данных в формате СБиС.
    * Работает с форматом данных, который использует БЛ СБИС.
    * Примеры можно посмотреть в модулях {@link WS.Data/Adapter/SbisRecord} и {@link WS.Data/Adapter/SbisTable}.
    * @class WS.Data/Adapter/Sbis
    * @extends WS.Data/Adapter/Abstract
    * @public
    * @author Мальцев Алексей
    */

   var Sbis = Abstract.extend(/** @lends WS.Data/Adapter/Sbis.prototype */{
      '[WS.Data/Adapter/Sbis]': true,

      forTable: function (data) {
         return new SbisTable(data);
      },

      forRecord: function (data) {
         return new SbisRecord(data);
      },

      getKeyField: function (data) {
         //TODO: имя поля с ПК может лежать в this._data.k (при этом может принимать значение -1)
         var index, s;
         if(data && data.s) {
            s = data.s;
            for (var i = 0, l = s.length; i < l; i++) {
               if (s[i].n && s[i].n[0] === '@') {
                  index = i;
                  break;
               }
            }
            if (index === undefined && s.length) {
               index = 0;
            }
         }
         return index === undefined ? undefined : s[index].n;
      }
   });

   Sbis.FIELD_TYPE = FIELD_TYPE;

   Di.register('adapter.sbis', Sbis);

   return Sbis;
});
