/* global define, require */
define('WS.Data/Chain/Reversed', [
   'WS.Data/Chain/Abstract',
   'WS.Data/Chain/IndexedEnumerator',
   'WS.Data/Di'
], function (
   Abstract,
   IndexedEnumerator,
   Di
) {
   'use strict';

   /**
    * Реверсивное звено цепочки.
    * @class WS.Data/Chain/Reversed
    * @extends WS.Data/Chain/Abstract
    * @public
    * @author Мальцев Алексей
    */

   var Reversed = Abstract.extend(/** @lends WS.Data/Chain/Reversed.prototype */{
      '[WS.Data/Chain/Reversed]': true,

      /**
       * Конструктор реверсивного звена цепочки.
       * @param {WS.Data/Chain/Abstract} previous Предыдущее звено.
       */
      constructor: function Reversed(previous) {
         this._previous = previous;
         Reversed.superclass.constructor.call(this, previous._source);
      },

      destroy: function() {
         Reversed.superclass.destroy.call(this);
      },

      //region WS.Data/Collection/IEnumerable

      getEnumerator: function () {
         return new ReversedEnumerator(
            this._previous
         );
      }

      //endregion WS.Data/Collection/IEnumerable

      //region WS.Data/Chain/Abstract

      //endregion WS.Data/Chain/Abstract

   });

   /**
    * Конструктор реверсивного энумератора.
    * @param {WS.Data/Chain/Abstract} previous Предыдущее звено.
    * @protected
    */
   var ReversedEnumerator = function(previous) {
      IndexedEnumerator.call(this, previous);
   };

   ReversedEnumerator.prototype = Object.create(IndexedEnumerator.prototype);
   ReversedEnumerator.prototype.constructor = ReversedEnumerator;

   ReversedEnumerator.prototype._getItems = function() {
      if (!this._items) {
         IndexedEnumerator.prototype._getItems.call(this);
         this._items.reverse();
         //Build natural order indices if necessary
         if (!this.previous.saveIndices) {
            this._items = this._items.map(function(item, index) {
               return [
                  index,
                  item[1]
               ];
            }, this);
         }
      }

      return this._items;
   };

   Di.register('chain.$reversed', Reversed, {instantiate: false});
   Di.register('chain.reversed', Reversed);

   return Reversed;
});
