/* global define, require */
define('WS.Data/Chain/Sorted', [
   'WS.Data/Chain/Abstract',
   'WS.Data/Chain/IndexedEnumerator',
   'WS.Data/Di'
], function (
   Abstract,
   IndexedEnumerator,
   Di
) {
   'use strict';

   var global = (0, eval)('this');

   /**
    * Сортирующее звено цепочки.
    * @class WS.Data/Chain/Sorted
    * @extends WS.Data/Chain/Abstract
    * @public
    * @author Мальцев Алексей
    */

   var Sorted = Abstract.extend(/** @lends WS.Data/Chain/Sorted.prototype */{
      '[WS.Data/Chain/Sorted]': true,

      /**
       * @member {Function(*, *): Number} Функция сравнения
       */
      _compareFunction: null,

      /**
       * Конструктор сортирующего звена цепочки.
       * @param {WS.Data/Chain/Abstract} previous Предыдущее звено.
       * @param {Function(*, Number): *} [compareFunction] Функция сравнения
       */
      constructor: function Sorted(previous, compareFunction) {
         this._previous = previous;
         this._compareFunction = compareFunction;
         Sorted.superclass.constructor.call(this, previous._source);
      },

      destroy: function() {
         this._compareFunction = null;
         Sorted.superclass.destroy.call(this);
      },

      //region WS.Data/Collection/IEnumerable

      getEnumerator: function () {
         return new SortedEnumerator(
            this._previous,
            this._compareFunction
         );
      }

      //endregion WS.Data/Collection/IEnumerable

      //region WS.Data/Chain/Abstract

      //endregion WS.Data/Chain/Abstract

   });

   /**
    * Конструктор сортирующего энумератора.
    * @param {WS.Data/Chain/Abstract} previous Предыдущее звено.
    * @param {Function(*, Number): *} [compareFunction] Функция сравнения.
    * @protected
    */
   var SortedEnumerator = function(previous, compareFunction) {
      IndexedEnumerator.call(this, previous);
      this.compareFunction = compareFunction || SortedEnumerator.defaultCompare;
   };

   SortedEnumerator.prototype = Object.create(IndexedEnumerator.prototype);
   SortedEnumerator.prototype.constructor = SortedEnumerator;

   SortedEnumerator.prototype.compareFunction = null;

   SortedEnumerator.prototype._getItems = function() {
      if (!this._items) {
         var saveIndices = this.previous.saveIndices;
         this._items = IndexedEnumerator.prototype._getItems.call(this)
            .map(function(item) {
               return new SortWrapper(item[1], item[0]);
            })
            .sort(this.compareFunction)
            .map(function(item, index) {
               var result = [
                  saveIndices ? SortWrapper.indexOf(item) : index,
                  SortWrapper.valueOf(item)
               ];
               SortWrapper.clear(item);
               return result;
            }, this);
      }

      return this._items;
   };

   SortedEnumerator.defaultCompare = function defaultCompare(a, b) {
      return a > b;
   };

   /**
    * Обертка для элемента коллекции, позволяющая сохранить информацию о его индексе в коллекции.
    * @param {*} item Элемент коллекции.
    * @param {*} index Индекс элемента коллекции.
    * @protected
    */
   var SortWrapper = function(item, index) {
      if (item instanceof Object) {
         item[SortWrapper.indexKey] = index;
         return item;
      }
      this.item = item;
      this.index = index;
   };

   SortWrapper.prototype.valueOf = function() {
      return this.item;
   };

   SortWrapper.prototype.indexOf = function() {
      return this.index;
   };

   SortWrapper.indexKey = global.Symbol ? global.Symbol('[]') : ['[]'];

   SortWrapper.valueOf = function(item) {
      return item instanceof SortWrapper ? item.valueOf() : item;
   };

   SortWrapper.indexOf = function(item) {
      return item instanceof SortWrapper ? item.indexOf() : item[SortWrapper.indexKey];
   };

   SortWrapper.clear = function(item) {
      if (!(item instanceof SortWrapper)) {
         delete item[SortWrapper.indexKey];
      }
   };

   Di.register('chain.$sorted', Sorted, {instantiate: false});
   Di.register('chain.sorted', Sorted);

   return Sorted;
});
