/* global define, require */
define('WS.Data/Display/Enum', [
   'WS.Data/Display/Collection',
   'WS.Data/Di'
], function (
   CollectionDisplay,
   Di
) {
   'use strict';

   /**
    * Проекция типа "Перечисляемое".
    * @class WS.Data/Display/Enum
    * @extends WS.Data/Display/Collection
    * @public
    * @author Ганшнин Ярослав
    */

   var EnumDisplay = CollectionDisplay.extend([], /** @lends WS.Data/Display/Enum.prototype */{
      '[WS.Data/Display/Enum]': true,

      /**
       * @member {Function} Обработчик события об изменении текущего индекса Enum
       */
      _onSourceChange: null,

      constructor: function EnumDisplay(options) {
         EnumDisplay.superclass.constructor.call(this, options);

         if (!this._$collection['[WS.Data/Type/IEnum]']) {
            throw new TypeError(this._moduleName + ': source collection should implement WS.Data/Type/IEnum');
         }

         this._getCursorEnumerator().setPosition(this.getIndexBySourceIndex(this._$collection.get()));

         if (this._$collection['[WS.Data/Entity/ObservableMixin]']) {
            //mixes WS.Data/Entity/ObservableMixin
            this._$collection.subscribe('onChange', this._onSourceChange);
         }
      },

      destroy: function () {
         if (this._$collection['[WS.Data/Entity/ObservableMixin]'] && //mixes WS.Data/Entity/ObservableMixin
            !this._$collection.isDestroyed()
         ) {
            this._$collection.unsubscribe('onChange', this._onSourceChange);
         }

         EnumDisplay.superclass.destroy.call(this);
      },

      _bindHandlers: function() {
         EnumDisplay.superclass._bindHandlers.call(this);

         this._onSourceChange = _private.onSourceChange.bind(this);
      },

      _notifyCurrentChange: function(newCurrent, oldCurrent, newPosition, oldPosition) {
         var value = null;
         if (newPosition > -1) {
            value = this.getSourceIndexByIndex(newPosition);
         }
         this._$collection.set(value);

         EnumDisplay.superclass._notifyCurrentChange.apply(this, arguments);
      },

      _getSourceIndex:function (index) {
         var enumerator = this._$collection.getEnumerator(),
            i = 0;

         if (index > -1) {
            while (enumerator.moveNext()) {
               if (i === index) {
                  return enumerator.getCurrentIndex();
               }
               i++;
            }
         }
         return -1;
      },

      _getItemIndex: function (index) {
         var enumerator = this._$collection.getEnumerator(),
            i = 0;

         while (enumerator.moveNext()) {
            if (enumerator.getCurrentIndex() == index) {
               return i;
            }
            i++;
         }
         return -1;
      }
   });

   var _private = {
      /**
       * Обрабатывает событие об изменении текущего индекса Enum
       * @param {Core/EventObject} event Дескриптор события
       * @param {Number} index Новый индекс
       * @param {String} value Новое значение
       * @private
       */
      onSourceChange: function (event, index) {
         this.setCurrentPosition(this.getIndexBySourceIndex(index));
      }
   };

   Di.register('display.enum', EnumDisplay);

   return EnumDisplay;
});
