/* global define */
define('WS.Data/Display/GroupItem', [
   'WS.Data/Display/CollectionItem',
   'WS.Data/Di'
], function (
   CollectionItem,
   Di
) {
   'use strict';

   /**
    * Группа элементов
    * @class WS.Data/Display/GroupItem
    * @extends WS.Data/Display/CollectionItem
    * @public
    * @author Мальцев Алексей
    */
   var GroupItem = CollectionItem.extend(/** @lends WS.Data/Display/GroupItem.prototype */{
      '[WS.Data/Display/GroupItem]': true,

      /**
       * @cfg {Boolean} Развернута или свернута группа. По умолчанию развернута.
       * @name WS.Data/Display/GroupItem#expanded
       */
      _$expanded: true,

      _instancePrefix: 'group-item-',

      constructor: function GroupItem(options) {
         GroupItem.superclass.constructor.call(this, options);
         this._$expanded = !!this._$expanded;
      },

      //region Public methods

      /**
       * Возвращает признак, что узел развернут
       * @return {Boolean}
       */
      isExpanded: function () {
         return this._$expanded;
      },

      /**
       * Устанавливает признак, что узел развернут или свернут
       * @param {Boolean} expanded Развернут или свернут узел
       * @param {Boolean} [silent=false] Не генерировать событие
       */
      setExpanded: function (expanded, silent) {
         if (this._$expanded === expanded) {
            return;
         }
         this._$expanded = expanded;
         if (!silent) {
            this._notifyItemChangeToOwner('expanded');
         }
      },

      /**
       * Переключает признак, что узел развернут или свернут
       */
      toggleExpanded: function () {
         this.setExpanded(!this.isExpanded());
      }

      //endregion Public methods

      //region Protected methods

      //endregion Protected methods

   });

   Di.register('display.group-item', GroupItem);

   return GroupItem;
});
