/* global define */
define('WS.Data/Display/ItemsStrategy/Direct', [
   'WS.Data/Display/ItemsStrategy/Abstract',
   'WS.Data/Display/CollectionItem'
], function (
   Abstract,
   CollectionItem
) {
   'use strict';

   /**
    * Стратегия получения элементов проекции напрямую по коллекции
    * @class WS.Data/Display/ItemsStrategy/Direct
    * @extends WS.Data/Display/ItemsStrategy/Abstract
    * @author Мальцев Алексей
    */

   var Direct = Abstract.extend(/** @lends WS.Data/Display/ItemsStrategy/Direct.prototype */{
      '[WS.Data/Display/ItemsStrategy/Direct]': true,

      /**
       * @member {WS.Data/Collection/IEnumerator} Энумератор коллекции
       */
      _collectionEnumerator: null,

      constructor: function Direct(display, options) {
         Direct.superclass.constructor.call(this, display, options);
      },

      //region Public methods

      at: function (index) {
         if (this._items[index]) {
            return this._items[index];
         }

         var source;


         if (this._collection['[WS.Data/Collection/IList]']) {
            //implements WS.Data/Collection/IList
            source = this._collection.at(index);
         } else {
            var enumerator = this._getCollectionEnumerator(),
               itemIndex = 0;
            enumerator.reset();
            while (enumerator.moveNext()) {
               if (itemIndex === index) {
                  source = enumerator.getCurrent();
                  break;
               }
               itemIndex++;
            }
         }

         if (source === undefined) {
            throw new ReferenceError('Collection index ' + index + ' is out of bounds.');
         }

         return (this._items[index] = this.convertToItem(source));
      },

      getCount: function() {
         return this._items.length;
      },

      getItems: function () {
         var count = this._items.length,
            i;
         for (i = 0; i < count; i++) {
            this.at(i);
         }
         return this._items;
      },

      splice: function (start, deleteCount, added) {
         added = added || [];
         return Array.prototype.splice.apply(
            this._items,
            [start, deleteCount].concat(added.map(function(item) {
               if (item instanceof CollectionItem) {
                  return item;
               }
               return this.convertToItem(item);
            }.bind(this)))
         );
      },

      //endregion Public methods

      //region Protected methods

      _initItems: function() {
         this._items.length = this._display.getCollectionCount();
      },

      /**
       * Возвращает энумератор коллекции
       * @return {WS.Data/Collection/IEnumerator}
       * @protected
       */
      _getCollectionEnumerator: function () {
         if (!this._collectionEnumerator) {
            this._collectionEnumerator = this._collection.getEnumerator(this._options.localize);
         }
         return this._collectionEnumerator;
      }

      //endregion Protected methods
   });

   return Direct;
});
