/* global define */
define('WS.Data/Display/ItemsStrategy/Group', [
   'WS.Data/Entity/Abstract',
   'WS.Data/Display/GroupItem'
], function (
   Abstract,
   GroupItem
) {
   'use strict';

   /**
    * Стратегия-декоратор для формирования групп элементов
    * @class WS.Data/Display/ItemsStrategy/Group
    * @extends WS.Data/Entity/Abstract
    * @author Мальцев Алексей
    */

   var Group = Abstract.extend(/** @lends WS.Data/Display/ItemsStrategy/Group.prototype */{
      '[WS.Data/Display/ItemsStrategy/Group]': true,

      /**
       * @member {WS.Data/Display/ItemsStrategy/Abstract} Декорирумая стратегия
       */
      _source: null,

      /**
       * {Array.<WS.Data/Display/GroupItem} Группы
       */
      _groups: null,

      /**
       * @member {WS.Data/Display/ItemsStrategy/CollectionItem} Кэш элементов
       */
      _items: null,

      get source() {
         return this._source;
      },

      get groups() {
         return this._groups;
      },

      /**
       * Конструктор
       * @param {WS.Data/Display/ItemsStrategy/Abstract} source Декорирумая стратегия
       */
      constructor: function Group(source) {
         this._source = source;
         this._groups = [];
      },

      //region Public methods

      addGroup: function(group, at) {
         this._items = null;
         if (at === undefined) {
            this._groups.push(group);
         } else {
            this._groups.splice(at, 0, group);
         }
      },

      removeGroup: function(group) {
         this._items = null;
         var index = this._groups.indexOf(group);
         return index === -1 ? undefined : this._groups.splice(index, 1)[0];
      },

      //endregion Public methods

      //region WS.Data/Display/ItemsStrategy/Abstract

      getOptions: function () {
         return this._source.getOptions();
      },

      at: function (index) {
         if (this._isGroup(index)) {
            return this._groups[index];
         } else {
            return this._source.at(index - this._groups.length);
         }
      },

      getCount: function() {
         return this._groups.length + this._source.getCount();
      },

      getItems: function () {
         if (!this._items) {
            this._items = this._groups.concat(this._source.getItems());
         }
         return this._items;
      },

      setItems: function (items) {
         this._items = null;
         this._groups = items.filter(function(item) {
            return item instanceof GroupItem;
         });
         this._source.setItems(items.filter(function(item) {
            return !(item instanceof GroupItem);
         }));
      },

      getItemIndex: function (index) {
         if (isNaN(parseInt(index, 10))) {
            return -1;
         }
         index = this._source.getItemIndex(index);
         return index === -1 ? index : this._groups.length + index;
      },

      getCollectionIndex: function (index) {
         return this._source.getCollectionIndex(index - this._groups.length);
      },

      splice: function (start, deleteCount, added) {
         this._items = null;
         return this._source.splice(Math.max(start - this._groups.length, 0), deleteCount, added);
      },

      reset: function () {
         this._groups = [];
         this._items = null;
         return this._source.reset();
      },

      addSorters: function (sorters) {
         return this._source.addSorters(sorters);
      },

      convertToItem: function (source, options) {
         return this._source.convertToItem(source, options);
      },

      getItemModule: function() {
         return this._source.getItemModule();
      },

      //endregion WS.Data/Display/ItemsStrategy/Abstract

      //region Protected methods

      /**
       * Проверяет, что в этом индексе находится группа
       * @param {Number} index Объект
       * @return {Boolean}
       */
      _isGroup: function(index) {
         return index < this._groups.length;
      }

      //endregion Protected methods
   });

   return Group;
});
