/* global define */
define('WS.Data/Display/ItemsStrategy/Root', [
   'WS.Data/Entity/Abstract'
], function (
   Abstract
) {
   'use strict';

   /**
    * Стратегия-декоратор для формирования корня дерева
    * @class WS.Data/Display/ItemsStrategy/Root
    * @extends WS.Data/Entity/Abstract
    * @author Мальцев Алексей
    */

   return Abstract.extend(/** @lends WS.Data/Display/ItemsStrategy/Root.prototype */{
      '[WS.Data/Display/ItemsStrategy/Root]': true,

      /**
       * @member {WS.Data/Display/ItemsStrategy/Abstract} Декорирумая стратегия
       */
      _source: null,

      /**
       * @member {Function.<WS.Data/Display/TreeItem>} Функция, возвращающая корень дерева
       */
      _root: null,

      get source() {
         return this._source;
      },

      /**
       * Конструктор
       * @param {WS.Data/Display/ItemsStrategy/Abstract} source Декорирумая стратегия
       * @param {Function.<WS.Data/Display/TreeItem>} root Функция, возвращающая корень дерева
       */
      constructor: function Root(source, root) {
         this._source = source;
         this._root = root;
      },

      //region WS.Data/Display/ItemsStrategy/Abstract

      getOptions: function () {
         return this._source.getOptions();
      },

      at: function (index) {
         if (index === 0) {
            return this._root();
         } else {
            return this._source.at(index - 1);
         }
      },

      getCount: function() {
         return this._source.getCount() + 1;
      },

      getItems: function () {
         return [this._root()].concat(this._source.getItems());
      },

      setItems: function (items) {
         return this._source.setItems(items.slice(1));
      },

      getItemIndex: function (index) {
         if (isNaN(parseInt(index, 10))) {
            return -1;
         }
         index = this._source.getItemIndex(index);
         return index === -1 ? index : 1 + index;
      },

      getCollectionIndex: function (index) {
         return this._source.getCollectionIndex(index - 1);
      },

      splice: function (start, deleteCount, added) {
         return this._source.splice(Math.max(start - 1, 0), deleteCount, added);
      },

      reset: function () {
         return this._source.reset();
      },

      addSorters: function (sorters) {
         return this._source.addSorters(sorters);
      },

      convertToItem: function (source, options) {
         return this._source.convertToItem(source, options);
      },

      getItemModule: function() {
         return this._source.getItemModule();
      }

      //endregion WS.Data/Display/ItemsStrategy/Abstract
   });
});
