/* global define */
define('WS.Data/Entity/Abstract', [
   'WS.Data/Di',
   'Core/core-extend',
   'Core/constants'
], function (
   Di,
   coreExtend,
   constants
) {
   'use strict';

   /**
    * Абстрактная сущность.
    * Это абстрактный класс, не предназначенный для создания самостоятельных экземпляров.
    * Явлется прототипом большинства сущностей.
    * Обладает аспектом состояния "экземпляр разрушен".
    * @class WS.Data/Entity/Abstract
    * @public
    * @author Мальцев Алексей
    */

   var Abstract = coreExtend.extend(/** @lends WS.Data/Entity/Abstract.prototype */{
      '[WS.Data/Entity/Abstract]': true,

      /**
       * @member {Boolean} Экземпляр был разрушен
       */
      _destroyed: false,

      /**
       * Разрушает экземпляр
       */
      destroy: function() {
         this._destroyed = true;

         if (!constants.isProduction) {
            /*jshint -W089 */
            /* eslint-disable guard-for-in */
            for (var key in this) {
               switch(key) {
                  case 'destroy':
                  case 'isDestroyed':
                     break;
                  default:
                     if (typeof this[key] === 'function') {
                        this[key] = dontTouchDeads;
                     }
               }
            }
            /* eslint-enable guard-for-in */
         }
      },

      /**
       * Возвращает признак, что экземпляр разрушен
       * @return {Boolean}
       */
      isDestroyed: function() {
         return this._destroyed;
      }
   }),

   dontTouchDeads = function() {
      throw new ReferenceError('This class instance is destroyed.');
   };

   Di.register('entity.abstract', Abstract);

   return Abstract;
});
