/* global define, Symbol, Object */
define('WS.Data/Entity/ReadWriteMixin', [
   'WS.Data/Entity/OptionsMixin',
   'WS.Data/Entity/ObservableMixin',
   'WS.Data/Entity/ManyToManyMixin'
], function (
   OptionsMixin,
   ObservableMixin,
   ManyToManyMixin
) {
   'use strict';

   var hasOwnProperty = Object.prototype.hasOwnProperty,
      global = (0, eval)('this'),
      writable = typeof Symbol === 'undefined' ? '_writable' : Symbol('writable'),
      ReadWriteMixin;

   /**
    * Миксин, позволяющий ограничивать запись и чтение.
    * Подмешивается после WS.Data/Entity/ObservableMixin и после WS.Data/Entity/ManyToManyMixin, перекрывая часть их методов
    * @mixin WS.Data/Entity/ReadWriteMixin
    * @public
    * @author Мальцев Алексей
    */
   ReadWriteMixin = /**@lends WS.Data/Entity/ReadWriteMixin.prototype  */{
      '[WS.Data/Entity/ReadWriteMixin]': true,

      //region WS.Data/Entity/ReadWriteMixin

      get writable() {
         return this[writable];
      },

      constructor: function (options) {
         if (this._options && hasOwnProperty.call(this._options, 'writable')) {
            this[writable] = this._options.writable;
         }
         if (options && hasOwnProperty.call(options, 'writable')) {
            this[writable] = options.writable;
         }
         if (this[writable]) {
            ObservableMixin.constructor.apply(this, arguments);
         }
      },

      destroy: function() {
         if (this[writable]) {
            ObservableMixin.destroy.call(this);
            ManyToManyMixin.destroy.call(this);
         }
      },

      //endregion WS.Data/Entity/ReadWriteMixin

      //region WS.Data/Entity/ManyToManyMixin

      _addChild: function(child, name) {
         if (this[writable]) {
            return ManyToManyMixin._addChild.call(this, child, name);
         }
      },

      _removeChild: function(child) {
         if (this[writable]) {
            return ManyToManyMixin._removeChild.call(this, child);
         }
      },

      _parentChanged: function(data) {
         if (this[writable]) {
            return ManyToManyMixin._parentChanged.call(this, data);
         }
      },

      _childChanged: function (data) {
         if (this[writable]) {
            return ManyToManyMixin._childChanged.call(this, data);
         }
      },

      //endregion WS.Data/Entity/ManyToManyMixin

      //region WS.Data/Entity/ObservableMixin

      subscribe: function (event, handler, ctx) {
         if (this[writable]) {
            return ObservableMixin.subscribe.call(this, event, handler, ctx);
         }
      },

      unsubscribe: function (event, handler, ctx) {
         if (this[writable]) {
            return ObservableMixin.unsubscribe.call(this, event, handler, ctx);
         }
      },

      _publish: function() {
         if (this[writable]) {
            return ObservableMixin._publish.apply(this, arguments);
         }
      },

      _notify: function () {
         if (this[writable]) {
            return ObservableMixin._notify.apply(this, arguments);
         }
      },

      //endregion WS.Data/Entity/ObservableMixin

      //region WS.Data/Entity/OptionsMixin

      _getOptions: function() {
         var options = OptionsMixin._getOptions.call(this);
         //Delete "writable" property received from _options
         delete options.writable;
         return options;
      }

      //endregion WS.Data/Entity/OptionsMixin
   };

   /**
    * @member {Boolean} Объект можно модифицировать. Запрет модификации выключит механизмы генерации событий (ObservableMixin) и распространения изменений от детей к родителям (ManyToManyMixin).
    */
   Object.defineProperty(ReadWriteMixin, writable, {
      writable: true,
      value: typeof window !== 'undefined' || global.isMochaRunned
   });

   return ReadWriteMixin;
});
