/* global define */
define('WS.Data/Format/FormatsFactory', [
   'WS.Data/Format/Format',
   'WS.Data/Format/FieldsFactory'
], function (
   Format,
   FieldsFactory
) {
   'use strict';

   /**
    * Фабрика форматов - конструирует формат по декларативному описанию
    * @class WS.Data/Format/FormatsFactory
    * @public
    * @author Мальцев Алексей
    */

   var FormatsFactory = /** @lends WS.Data/Format/FormatsFactory.prototype */{
      '[WS.Data/Format/FormatsFactory]': true,

      //region Public methods

      /**
       * Конструирует формат полей по декларативному описанию
       * @param {Array.<WS.Data/Format/FieldsFactory/FieldDeclaration.typedef>} declaration Декларативное описание
       * @return {WS.Data/Format/Format}
       * @static
       */
      create: function(declaration) {
         if (!declaration || !(declaration instanceof Array)) {
            throw new TypeError(this._moduleName + '::create(): declaration should be an instance of Array');
         }
         var format = new Format();
         for (var i = 0; i < declaration.length; i++) {
            format.add(FieldsFactory.create(declaration[i]));
         }
         return format;
      }

      //endregion Public methods

   };

   return FormatsFactory;
});
