/* global define */
define('WS.Data/Shim/Map', [
   'WS.Data/Shim/Set'
], function (
   Set
) {
   'use strict';

   /**
    * Limited emulation of standard built-in object "Map" if it's not supported.
    * Follow {@link https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Global_Objects/Map} for details.
    * @class WS.Data/Shim/Map
    * @author Мальцев Алексей
    */

   //Return native implementation if supported
   if (typeof Map !== 'undefined') {
      return Map;
   }

   return (function() {
      var Map = function() {
         this.clear();
      };

      Map.prototype._hash = null;
      Map.prototype._objectPrefix = Set.prototype._objectPrefix;
      Map.prototype._objects = null;

      Object.defineProperty(Map.prototype, 'size', {
         get: function() {
            return Object.keys(this._hash).length;
         },
         enumerable: true,
         configurable: false
      });

      Map.prototype.clear = function() {
         this._hash = {};
         this._objects = [];
      };

      Map.prototype.delete = function(key) {
         if (this._isObject(key)) {
            key = this._addObject(key);
            if (!key) {
               return;
            }
         }
         this._hash[key] = undefined;
      };

      Map.prototype.entries = function() {
         throw new Error('Method is not supported');
      };

      Map.prototype.forEach = function(callbackFn, thisArg) {
         //FIXME: now not in insertion order
         var hash = this._hash,
            key,
            value;
         for (key in hash) {
            if (hash.hasOwnProperty(key) && hash[key] !== undefined) {
               value = hash[key];
               if (this._isObjectKey(key)) {
                  key = this._getObject(key);
               }
               callbackFn.call(thisArg, value, key, this);
            }
         }
      };

      Map.prototype.get = function(key) {
         if (this._isObject(key)) {
            key = this._getObjectKey(key);
            if (!key) {
               return;
            }
         }
         return this._hash[key];
      };

      Map.prototype.has = function(key) {
         if (this._isObject(key)) {
            key = this._getObjectKey(key);
            if (!key) {
               return false;
            }
         }
         return this._hash.hasOwnProperty(key) && this._hash[key] !== undefined;
      };

      Map.prototype.keys = function() {
         throw new Error('Method is not supported');
      };

      Map.prototype.set = function(key, value) {
         if (this._isObject(key)) {
            key = this._addObject(key);
         }
         this._hash[key] = value;
         return this;
      };


      Map.prototype._isObjectKey = function(key) {
         return String(key).substr(0, this._objectPrefix.length) === this._objectPrefix;
      };

      Map.prototype._getObject = function(key) {
         var index = parseInt(key.substr(this._objectPrefix.length), 10);
         return this._objects[index];
      };

      Map.prototype._isObject = Set.prototype._isObject;
      Map.prototype._addObject = Set.prototype._addObject;
      Map.prototype._deleteObject = Set.prototype._deleteObject;
      Map.prototype._getObjectKey = Set.prototype._getObjectKey;

      return Map;
   })();
});
