/* global define */
define('WS.Data/Source/Provider/INotify', function () {
   'use strict';

   /**
    * Интерфейс провайдера c доступом к серверным событиям
    * @interface WS.Data/Source/Provider/INotify
    * @public
    * @author Мальцев Алексей
    * @example
    * <pre>
    *    require(['WS.Data/Source/Remote', 'Core/core-instance', 'Core/core-functions'], function(RemoteSource, coreInstance, CoreFunctions) {
    *       //...
    *       if (dataSource instanceof RemoteSource) {
    *          var provider = dataSource.getProvider();
    *          if (coreInstance.instanceOfMixin(provider, 'WS.Data/Source/Provider/INotify') {
    *             provider.getEventsChannel().subscribe('onMessage', function(event, message) {
    *                CoreFunctions.alert('A message from the server: ' + message);
    *             });
    *          }
    *       }
    *    });
    * </pre>
    * @example
    * <pre>
    *    require(['Core/helpers/fast-control-helpers'], function(helpers) {
    *       //...
    *       dataSource.getProvider().getEventsChannel('ErrorLog').subscribe('onMessage', function(event, message) {
    *          helpers.alert('Something went wrong: ' + message);
    *       });
    *    });
    * </pre>
    */

   return /** @lends WS.Data/Source/Provider/INotify.prototype */{
      '[WS.Data/Source/Provider/INotify]': true,

      /**
       * Возвращает канал серверных событий
       * @param {String} [name] Имя канала событий
       * @return {WS.Data/Source/Provider/IChannel}
       */
      getEventsChannel: function (name) {
         throw new Error('Method must be implemented');
      }
   };
});
