define('WS.Data/Source/SbisFile', [
   'WS.Data/Source/SbisService',
   'WS.Data/Entity/Record',
   'WS.Data/Source/Provider/SbisFile'
], function (SbisService, Record) {
   'use strict';

   function getRPCFile(href) {
       return {
           'Данные': {
               'href': href
           }
       };
   }
   function injectField(obj, name, value) {
      if (obj instanceof Record) {
         obj.set(name, value);
      } else {
         obj[name] = value;
      }
   }
   /**
    * Класс источника данных,
    * позволяющий загружать сущность файлов {@link Lib/File/LocalFile} на бизнес-логику СБИС
    * @class
    * @name WS.Data/Source/SbisFile
    * @public
    */
   var SbisFile = SbisService.extend(/** @lends WS.Data/Source/SbisFile.prototype */ {
      '[WS.Data/Source/SbisFile]': true,

       /**
        * @cfg {String} Имя поля, в котором будет лежать тело запроса
        */
       _$requestField: 'Запрос',
      /**
       * @cfg {String} Имя параметра на бизнес-логике, который будет содержать файл
       */
       _$fileProperty: 'Файл',

       _$provider: 'source.provider.sbis-file',

      /**
       * Создает пустую модель через источник данных (при этом она не сохраняется в хранилище)
       *
       * @param {Object | WS.Data/Entity/Record} meta Дополнительные мета данные, которые могут понадобиться для создания модели
       * @param {Lib/File/LocalFile} file Загружаемый файл на бизнес-логику
       * @return {Core/Deferred<WS.Data/Entity/Model>} Асинхронный результат выполнения. В колбэке придет {@link WS.Data/Entity/Model}.
       * @see WS.Data/Entity/Model
       * @see Lib/File/LocalFile
       * @override
       */
       create: function(meta, file) {
          var self = this;
           if (!file) {
              return SbisFile.superclass.create.call(self, meta);
           }

           this._notify('onBeforeProviderCall', self._$binding.create, meta, file);

           var name = '_file';
           var filesObj = Object.create(null);
           filesObj[name] = file;
           injectField(meta, self._$fileProperty, getRPCFile(name));

           return self.getProvider().upload({
               meta: self._prepareArgumentsForCall(meta),
               files: filesObj,
               uploadParam: {
                  method: self._$binding.create,
                  requestField: self._$requestField
               }
           }).addCallback(function(data) {
              return self._prepareCreateResult(data);
           });
       }
   });

   return SbisFile;
});
