/* global define */
define('WS.Data/Type/Identity', [
   'WS.Data/Di',
   'Core/core-extend'
], function (
   Di,
   coreExtend
) {
   'use strict';

   /**
    * Тип данных "Идентификатор".
    * @class WS.Data/Type/Identity
    * @public
    * @author Мальцев Алексей
    * @example
    * <pre>
    *    require(['WS.Data/Type/Identity'], function (Identity) {
    *       var id = new Identity([1, 'Employees']);
    *       console.log(id.getValue());//1
    *       console.log(id.getName());//'Employees'
    *       console.log(String(id));//'1,Employees'
    *    });
    * </pre>
    */

   var Identity = coreExtend.extend(Object, /** @lends WS.Data/Type/Identity.prototype */{
      '[WS.Data/Type/Identity]': true,

      /**
       * @member {String} Разделитель значений
       */
      _separator: ',',

      /**
       * @member {Array} Значение идентификатора
       */
      _value: null,

      /**
       * Конструктор типа "Идентификатор".
       * @param {Array|String|Number} value Значение идентификатора
       */
      constructor: function Identity(value) {
         if (!(value instanceof Array)) {
            if (typeof value === 'string') {
               value = value.split(this._separator);
            } else {
               value = [value];
            }
         }
         this._value = value;
      },

      //region Public methods

      /**
       * Возвращает значение поля таблицы.
       * @return {Number|Null}
       */
      getValue: function() {
         return this._value[0];
      },

      /**
       * Возвращает название таблицы.
       * @return {String|undefined}
       */
      getName: function() {
         return this._value[1];
      },

      valueOf: function() {
         return this._value;
      },

      toString: function() {
         return this._value[0] === null ? null : this._value.join(',');
      }

      //endregion Public methods

      //region Protected methods

      //endregion Protected methods
   });

   Di.register('types.$identity', Identity, {instantiate: false});
   Di.register('types.identity', Identity);

   return Identity;
});
