/* global define */
define('WS.Data/Type/descriptor', [
], function (
) {
   'use strict';

   /**
    * Модуль описания типа.
    * @class WS.Data/Type/descriptor
    * @public
    * @author Мальцев Алексей
    */
   var validate = function(type) {
         if (typeof type === 'function') {
            switch (type) {
               case Boolean:
                  type = 'boolean';
                  break;
               case Number:
                  type = 'number';
                  break;
               case String:
                  type = 'string';
                  break;
            }
         }

         var typeName = typeof type;
         switch (typeName) {
            case 'string':
               return function validateTypeName(value) {
                  if (value === undefined || typeof value === type) {
                     return value;
                  }
                  return new TypeError('Value ' + value + ' should be type of ' + type);
               };
            case 'function':
               return function validateTypeIntance(value) {
                  if (value === undefined || value instanceof type) {
                     return value;
                  }
                  return new TypeError('Value ' + value + ' should be instance of ' + type);
               };
            case 'object':
               return function validateTypeInterface(value) {
                  if (value === undefined) {
                     return value;
                  }

                  var mixins = value && value._mixins;
                  if (mixins instanceof Array && mixins.indexOf(type) !== -1) {
                     return value;
                  }
                  return new TypeError('Value ' + value + ' should implement ' + type);
               };
         }

         throw new TypeError('Argument "type" should be one of following types: string, function or object. But "' + typeName + '" given.');
      },

      /**
       * Проверяет обязательность значения.
       * @name WS.Data/Type/descriptor#required
       * @returns {Function} Следующий обработчик.
       */
      required = function() {
         var prev = this,
            result = function(value) {
               if (value === undefined) {
                  return new TypeError('Value is required');
               }
               return prev(value);
            };

         result.oneOf = oneOf;

         return result;
      },

      /**
       * Проверяет вхождение значения в список разрешенных.
       * @name WS.Data/Type/descriptor#oneOf
       * @param {Array} values Список разрешенных значений.
       * @returns {Function} Следующий обработчик.
       */
      oneOf = function(values) {
         if (!(values instanceof Array)) {
            throw new TypeError('Argument values should be an instance of Array');
         }

         var prev = this,
            result = function(value) {
               if (value !== undefined && values.indexOf(value) === -1) {
                  return new TypeError('Invalid value ' + value);
               }
               return prev(value);
            };

         result.required = required;

         return result;
      };

   return function(type) {
      var result = validate(type);
      result.required = required;
      result.oneOf = oneOf;

      return result;
   };
});
