define('Core/ConsoleLogger', [
   'Core/core-classicExtend',
   'Core/ILogger',
   'Core/IoC',
   'Core/constants'
], function(
   classicExtend,
   ILogger,
   IoC,
   constants
) {
   /**
    * Этот класс задаёт реализацию интерфейса ILogger по умолчанию в ws, обеспечивает работу механизма {@link https://ru.wikipedia.org/wiki/Инверсия_управления ioc}.
    * Доступ к реализации осуществляется с помощью вызова конструкции Core/Ioc.resolve('ILogger').
    * @class Core/ConsoleLogger
    * @extends Core/ILogger
    * @public
    * @author Бегунов А.В.
    */
   var ConsoleLogger = function () {
      var global = (function(){ return this || (0,eval)('this'); }());
      if ('jstestdriver' in global) {
         this._con = global['jstestdriver']['console'];
      } else if ('console' in global) {
         this._con = global['console'];
      }

      this.log = function(tag, message) {
         if (this._con && typeof(this._con) === 'object' && 'log' in this._con && typeof(this._con.log) == 'function') {
            this._con.log(tag + ": " + message + "\n");
         } else {
            try {
               this._con.log(tag + ": " + message + "\n");
            } catch (e) {
            }
         }
      };

      this.warn = function(tag, message) {
         if (!constants.isProduction) {
            if (this._con && typeof(this._con) === 'object' && 'warn' in this._con && typeof(this._con.warn) == 'function') {
               this._con.warn(tag + ": " + message + "\n");
            } else {
               this.log(tag, message);
            }
         }
      };

      this.error = function(tag, message, exception) {
         message = message + (exception && exception.stack ? '\nStack: ' + exception.stack : '');
         if (this._con && typeof(this._con) === 'object' && 'error' in this._con && typeof(this._con.error) === 'function') {
            this._con.error(tag + ": " + message);
         } else {
            try {
               this._con.error(tag + ": " + message + "\n");
            } catch (e) {
               this.log(tag, message);
            }
         }
      };

      this.info = function() {
         if (!constants.isProduction) {
            if (this._con && typeof(this._con) === 'object' && 'info' in this._con && typeof(this._con.info) === 'function') {
               this._con.info.apply(this._con, arguments);
            } else {
               try {
                  this._con.info.apply(this._con, arguments);
               } catch (e) {
               }
            }
         }
      };

      this.methodExecutionStart = function(func) {
         this.log('methodExecutionStart', func.name);
      };

      this.methodExecutionFinish = function(func, executionTime) {
         this.log('methodExecutionFinish', func.name + ' execution time = ' + executionTime + 'ms');
      };
   };

   classicExtend(ConsoleLogger, ILogger);

   if (!IoC.has('ILogger')) {
      IoC.bindSingle('ILogger', new ConsoleLogger());
   }

   return ConsoleLogger;

});
