/**
 * Created by dv.zuev on 02.06.2017.
 */
define('Core/Control',
   [  'require',
      'tmpl!Core/Control',
      'Core/core-extend',
      'Core/helpers/random-helpers',
      'Core/EventBus',
      'Core/vdom/Synchronizer/Synchronizer',
      'WS.Data/Entity/InstantiableMixin',
      'Core/Abstract.compatible',
      'Core/IoC',
      "Core/constants",
      'Core/helpers/Hcontrol/doAutofocus',
      'Core/vdom/Synchronizer/resources/TabIndex',
      'Core/vdom/Utils/FocusHelper',
      'View/Runner/expressions/contextResolver',
      'View/Runner/Vdom/utils/optionsResolver',
      'View/Runner/expressions/attr',
      'Core/vdom/Synchronizer/resources/DOMEnvironment'
   ],

   function (require,
             tmplControl,
             extend,
             randomHelpers,
             EventBus,
             Sync,
             InstantiableMixin,
             AbstractCompatible,
             IoC,
             Constants,
             doAutofocus,
             TabIndex,
             FocusHelper,
             contextResolver,
             optionsResolver,
             attrExpressions) {

      'use strict';

      var ControlCompatible, AreaAbstractCompatible,
         BaseCompatible;
      //На сервере всегда надо подтянуть слой, потому что контролы могут строиться для разных клиентов
      //и для разных страниц
      if (Constants.compat && (typeof process === 'undefined' || !process.domain ||
         !process.domain.req || process.domain.req.compatible!==false)){
         ControlCompatible = require.defined('Lib/Control/Control.compatible') && require('Lib/Control/Control.compatible');
         AreaAbstractCompatible = require.defined('Lib/Control/AreaAbstract/AreaAbstract.compatible') && require('Lib/Control/AreaAbstract/AreaAbstract.compatible');
         BaseCompatible = require.defined('Lib/Control/BaseCompatible/BaseCompatible') && require('Lib/Control/BaseCompatible/BaseCompatible');
      }

      var _private = {
         _forceUpdate: function(self, environment, controlNode){
            environment && environment.forceRebuild(controlNode.id);
         },

         /** Функция инициирует событие eventName
          *  Возвращает результат последнего выполненого обработчика
          *  @private
          */
         _notify: function(self, environment, controlNode, args){
            return environment && environment.startEvent(controlNode, args);
         }
      };
      /**
       *
       * @class Core/Control
       * @author Зуев Д.В.
       * @public
       *
       * @mixes Core/Abstract.compatible
       * @mixes Lib/Control/Control.compatible
       * @mixes Lib/Control/AreaAbstract/AreaAbstract.compatible
       * @mixes Lib/Control/BaseCompatible/BaseCompatible
       * @mixes WS.Data/Entity/InstantiableMixin
       */
      var Base = extend.extend([AbstractCompatible || {},
            ControlCompatible || {},
            AreaAbstractCompatible||{},
            BaseCompatible||{},
            InstantiableMixin],
         /** @lends Core/Control.prototype */{
            _controlName: 'Core/Control',
            _context: null,
            _contextObj: null,
            /**
             * Состояния режима совместимости
             */
            VDOMReady: false, //состояние, которое используется в bootup для принятия решения маунтить контрол или он уже привязан к дому
            /**
             * Состояния с которыми не докнца ясно, что делать.
             * НЕ АПИ.
             * _logicParent хранит ссылку на логического родителя для регистрации в нем
             * _decOptions - набор атрибутов, которые были на теге при создании
             * (контрол могли создать через new и положить на контейнер, у которого есть класс или какие-нибудь другие атрибуты)
             */
            _logicParent: null,
            _mounted: false,
            _template: tmplControl,
            _decOptions: null,
            /**
             * @cfg {String} Устанавливает текст простой всплывающей подсказки, которая отображается при наведении на контрол курсора.
             * @name Core/Control#tooltip
             * @translatable
             */
            _$tooltip: '',
            /**
             * Логика вынесена в функцию для переопределения поведения легкого инстанса
             * @returns {null}
             * @private
             * @deprecated
             */
            _getDecOptions: function(){
               return this._decOptions;
            },

            hasCompatible: function(){
               if (this.fixBaseCompatible) {
                  return false;
               }

               return !!BaseCompatible;
            },

            /**
             * Создание объекта с опциями для декорирования
             * @param cfg
             * @private
             * @deprecated
             */
            _parseDecOptions: function(cfg){
               this._decOptions = {};
               /**
                * Опциями для декорирования могут быть лишь фиксированные опции
                */
               if(!attrExpressions.checkAttr(cfg)) {
                  if (cfg['class'] || cfg['className']) {
                     this._decOptions['class'] = (cfg['class']?cfg['class']+' ':'') + (cfg['className']?cfg['className']:'');
                  }
                  if (cfg['style']) {
                     this._decOptions['style'] = cfg['style'];
                  }
                  if (cfg['data-component']) {
                     this._decOptions['data-component'] = cfg['data-component'];
                  }

                  if (cfg['__$config']) {
                     this._decOptions['config'] = cfg['__$config'];
                  }
               }
            },
            /**
             * Метод для bootup, который показвает на то какого типа будет результат функции шаблонизации компонента
             * @returns {boolean}
             */
            isBuildVDom: function(){
               return this.hasCompatible()?BaseCompatible.isBuildVDom.call(this):true;
            },


            mountToDom: function(element, cfg){
               if (!this.VDOMReady) {
                  this.VDOMReady = true;
                  this._container = element;
                  Sync.mountControlToDOM(this, cfg, this._container);
               }
               if (cfg) {
                  this.saveOptions(cfg);
               }
            },

            //Функция которая перебивает ссылку на _options
            //Вызывается перед отрисовкой, но после _beforeMount
            saveOptions: function (options, controlNode) {
               this._options = options;
               if (controlNode) {
                  this._container = controlNode.element;
               }
               return this.hasCompatible()?BaseCompatible.saveOptions.call(this, options, controlNode):true;
            },

            /**
             * Метод, возвращающий значение служебной опции
             * @param {String} name Имя служебной опции
             * @returns {*} Значение опции
             * @private
             */
            _getInternalOption: function(name) {
               var result;
               if (this._internalOptions[name] !== undefined) {
                  result = this._internalOptions[name];
               } else if (this.hasCompatible()) {
                  result = BaseCompatible._getInternalOption.call(this, name);
               }
               return result;
            },

            /**
             * Метод задания значения служебной опции
             * @param {String} name Имя служебной опции
             * @param {*} value Значение опции
             * @private
             */
            _setInternalOption: function (name, value) {
               if (!this._internalOptions){
                  this._internalOptions = {};
                  IoC.resolve('ILogger').error('Component with '+(this._options?('name '+this._options.name+' config '+this._options.__$config):('maybe id '+this._$id)), 'Control.constructor wasn\'t called');
               }
               this._internalOptions[name] = value;

               if (this.hasCompatible()) {
                  BaseCompatible._setInternalOption.call(this, name, value);
               }
            },

            /**
             * Метод задания служебных опций
             * @param {Object} internal Объект, содержащий ключи и значения устанавливаемых служебных опций
             * @private
             */
            _setInternalOptions: function (internal) {
               for (var name in internal) {
                  if (internal.hasOwnProperty(name)) {
                     this._setInternalOption(name, internal[name]);
                  }
               }

               if (this.hasCompatible()) {
                  BaseCompatible._setInternalOptions.call(this, internal);
               }
            },

            //Определение слоя совместимости.
            //Логика такая, если создается контрол НЕ внутри CompoundControl
            //Пытаемся считать, что слоя совместимости нет, для того, чтобы не определять все
            //свойства, которые нужны CompoundControl
            //А также чтобы не пытаться зарегистрироваться в списке детей родителя и т.д.
            isCompatibleLayout: function(cfg) {
               if (!this.$constructor) {
                  return false;
               }
               var parCompatible = cfg && cfg.parent && cfg.parent.isCompatibleLayout && cfg.parent.isCompatibleLayout(cfg.parent._options);
               return parCompatible===undefined?!!this.$constructor:parCompatible;
            },

            saveFullContext: null,
            _saveEnvironment: null,

            constructor: function (cfg) {
               if (!cfg){
                  cfg = {};
               }
               var fullContext = null,
                  controlNode = null,
                  environment = null;
               this.saveFullContext = function saveFullContext(ctx){
                  fullContext = ctx;
               };

               this._saveEnvironment = function(env, cntNode){
                  controlNode = cntNode;
                  environment = env;
               };

               this._notify = function(){
                  return _private._notify(this, environment, controlNode, arguments);
               };

               this._notify._isVdomNotify = true;

               this._forceUpdate = function(){
                  _private._forceUpdate(this, environment, controlNode);
               };

               /**
                * Метод, который возвращает разметку для компонента
                * @param rootKey
                * @returns {*}
                * @private
                */
               this._getMarkup = function _getMarkup(rootKey, isRoot, attributes) {
                  var res;
                  if (this.hasCompatible()) {
                     res = BaseCompatible._getMarkup.call(this, rootKey, isRoot, attributes);
                  } else {
                     var decOpts = this._getDecOptions();
                     decOpts = attributes || (attrExpressions.checkAttr(decOpts) ? {} : decOpts);
                     /*Нет слоя совместимости, а значит name и sbisname нам не нужны как атрибуты
                     * TODO:: удалить как будет удален CompoundControl*/
                     delete decOpts.name;
                     delete decOpts.sbisname;
                     res = this._template(this, decOpts, rootKey, true, {
                        isSetts: true,
                        fullContext: fullContext
                     })[0];
                  }
                  return res;
               };



               this.render = function (empty, attributes) {
                  if (this.hasCompatible()) {
                     return BaseCompatible.render.call(this, arguments[0], arguments[1], {
                        isSetts: true,
                        fullContext: fullContext
                     });
                  }
                  var decOpts = this._getDecOptions();
                  decOpts = attrExpressions.checkAttr(attributes) ? attributes : decOpts;
                  var markup = this._template(this, decOpts, this._context, false, {
                     isSetts: true,
                     fullContext: fullContext
                  });
                  return markup;
               };

               this._logicParent = cfg._logicParent;
               this._options = {};
               this.context = {
                  get: function(field){
                     if (this._contextObj && this._contextObj.hasOwnProperty(field)){
                        return this._contextObj[field];
                     } else {
                        throw new Error('You trying to get unrequired field ' + field)
                     }
                  }.bind(this),
                  set: function(){
                     throw new Error("Can't set data to context. Context is readonly!");
                  },
                  has: function(){
                     return true;
                  }

               };
               this._internalOptions = {};
               this._children = {};
               this._subscriptions = this._subscriptions || [];
               this._subDestroyControls = this._subDestroyControls || [];

               if (cfg.fixBaseCompatible) {
                  this.fixBaseCompatible = true;
               }


               //Необходимо для сериализации состояния с сервера
               if (!cfg.__$config){
                  cfg.__$config = randomHelpers.randomId("cfg-");
               }

               if (!this.hasCompatible()) {
                  this._parseDecOptions(cfg);
                  this._handlers = {};
                  if (typeof window !== 'undefined') {
                     this._$eventBusId = randomHelpers.randomId();
                     EventBus.channel(this._$eventBusId, {
                        waitForPermit: true
                     });
                  }
               } else {
                  this._parseDecOptions(cfg);
                  this.deprecatedContr(cfg);
               }

               if (!this.isCompatibleLayout() && this.iWantVDOM!==false) {
                  this._savedOptions = this._options;
                  this._options = {};
               }
            },

            /**
             * Точка разрушения компонента
             */
            destroy: function(fromDirtyChecking) {
               this._beforeUnmount();
               if (!fromDirtyChecking && this.hasCompatible()) {
                  BaseCompatible.destroy.call(this);
               }
            },

            //<editor-fold desc="API">

            /**
             * Метод возвращающий идентификатор данных для сохранения при асинхронном построении
             */
            getDataId: function(){
               return randomHelpers.randomId('dataid');
            },

            /**
             * Рассчетное свойство, если свое не определено - берем родительское свойство
             * @returns {parentEnabled|*}
             */
            isEnabled: function(){
               if (this._options.enabled === undefined) {
                  var parentEnabled = this._getInternalOption('parentEnabled');
                  if (parentEnabled === undefined) {
                     parentEnabled = true;
                  }
                  return parentEnabled;
               }
               return this._options.enabled;
            },

            /**
             * Рассчетное свойство, если свое не определено - берем родительское свойство
             * @returns {parentVisible|*}
             */
            isVisible: function(){
               /**
                * В новых контролах нет метода isVisible
                * но сейчас он должен отрабатывать т.к. есть слой совместимости
                * иногда опций у новых контролов нет вообще. А согласно новой идеологии - если контрол есть,
                * значит он видимый.
                * Получается, что если опций нет - контрол видимый
                */
               if (this._options && this._options.visible === undefined) {
                  var
                     parent = this._getInternalOption('parent'),
                     parentVisible = this._getInternalOption('parentVisible');

                  // Если parent - compound контрол, узнаем его видимость через геттер
                  if (parent && parent.$constructor && parent.isVisible) {
                     parentVisible = parent.isVisible();
                  }

                  if (parentVisible === undefined) {
                     parentVisible = true;
                  }
                  return parentVisible;
               }
               return this._options?this._options.visible:true;
            },

            /**
             * Находится ли фокус внутри компонента (в том числе на самом контейнере компонента)
             * @returns {boolean}
             */
            hasFocus: function() {
               return !!this._active;
            },
            /**
             * Находится ли фокус на контейнере компонента
             * @returns {boolean}
             */
            isFocused: function () {
               var container = this._container[0] ? this._container[0] : this._container;
               return container === document.activeElement;
            },
            /**
             * Убрать фокус на body, если он находится внутри компонента.
             */
            blur: function () {
               var container = this._container[0] ? this._container[0] : this._container,
                  activeElement = document.activeElement,
                  tmpTabindex;

               if (!FocusHelper.closest(document.activeElement, container)) {
                  return;
               }

               // задача - убрать фокус с текущего элемента. куда? ну, например на body
               // чтобы можно было перевести фокус на body, сначала выставим табиндекс, а потом уберем
               if (document.body.tabIndex === -1) {
                  tmpTabindex = document.body.tabIndex;
                  document.body.tabIndex = 1;
               }
               document.body.focus();
               if (this.hasFocus()) {
                  var env = container.controlNodes[0].environment;
                  // если DOMEnvironment не перехватил переход фокуса, вызовем обработчик ухода фокуса вручную
                  env._handleBlurEvent({target: activeElement, relatedTarget: document.body});
               }

               if (tmpTabindex !== undefined) {
                  document.body.tabIndex = tmpTabindex;
               }

               // совместимость, дизактивируем старый компонент
               if (this.hasCompatible()) {
                  var $activeElement = $(activeElement),
                     oldControl = $activeElement.wsControl();
                  if (oldControl && oldControl.isActive() && !oldControl.iWantVDOM) {
                     oldControl.setActive(false);
                  }
               }
            },
            /**
             * Поместить фокус внутрь компонента.
             * По алгоритму поиска сначала ищем внутри компонент с директивой ws-autofocus, если не находим - ищем по табиндексам.
             * @returns {boolean} Получилось ли установить фокус внутрь
             */
            focus: function () {
               function needFocusOnActivated() {
                  return !(Constants.browser.isMobilePlatform &&
                     self._hasOption('focusOnActivatedOnMobiles') &&
                     self._getOption('focusOnActivatedOnMobiles') === false);
               }
               function findEnvironment(component) {
                  while (component) {
                     if (component.environment) {
                        return component.environment;
                     }
                     component = component._logicParent;
                  }
                  throw new Error('environment не был найден среди предков');
               }
               function doFocus(container) {
                  var res = false,
                     activeElement = document.activeElement;
                  if (container.wsControl && container.wsControl.setActive) {
                     // если нашли контейнер старого контрола, активируем его старым способом (для совместимости)
                     if (container.wsControl.canAcceptFocus()) {
                        container.wsControl.setActive(true);
                        res = container.wsControl.isActive();
                     } else {
                        // todo попробовать поискать следующий элемент?
                        res = false;
                     }
                  } else {
                     // нашли контейнер, проверим что можем его сфокусировать. Если можем - фокусируем.
                     if (needFocusOnActivated()) {
                        container.focus();
                        res = container === document.activeElement;
                     } else {
                        res = false;
                     }

                     container = this._container[0] ? this._container[0] : this._container
                     // может случиться так, что на focus() сработает обработчик в DOMEnvironment, и тогда тут ничего не надо делать
                     if (res && !this.hasFocus()) {
                        var env = container.controlNodes[0].environment;
                        env._handleBlurEvent({target: activeElement, relatedTarget: container});
                        env._handleFocusEvent({target: container, relatedTarget: activeElement});
                     }
                  }
                  return res;
               }

               var res = false,
                  self = this,
                  container = this._container[0] ? this._container[0] : this._container;

               // сначала попробуем поискать по ws-autofocus, если найдем - позовем focus рекурсивно для найденного компонента
               var autofocusElems = doAutofocus.findAutofocusForVDOM(container),
                  autofocusElem,
                  found;

               for (var i = 0; i < autofocusElems.length; i++) {
                  autofocusElem = autofocusElems[i];
                  // если что-то зафокусировали, перестаем поиск
                  if (!found) {
                     // фокусируем только найденный компонент, ws-autofocus можно повесить только на контейнер компонента
                     if (autofocusElem && autofocusElem.controlNodes && autofocusElem.controlNodes.length) {
                        res = autofocusElem.controlNodes[0].control.focus();
                        found = res;
                     }
                  }
               }

               // если не получилось найти по автофокусу, поищем первый элемент по табиндексам и сфокусируем его.
               // причем если это будет конейнер старого компонента, активируем его по старому тоже
               if (!found) {
                  // так ищем DOMEnvironment для текущего компонента. В нем сосредоточен код по работе с фокусами.
                  var environment = findEnvironment(this),
                     getElementProps = environment.getElementProps;

                  var next = TabIndex.findFirstInContext(container, false, getElementProps);
                  if (next) {
                     res = doFocus.call(this, next);
                  } else {
                     res = doFocus.call(this, container);
                  }
               }

               return res;
            },

            /**
             * Перед отрисовкой контрола, перед mount контрола в DOM
             * Выполняется как на клиенте, так и на сервере. Здесь мы можем скорректировать наше состояние
             * в зависимости от параметров конструктора, которые были сохранены в _options
             * Вызывается один раз в течение жизненного цикла
             * На этом методе заканчивается управление жизненным циклом компонента на сервере.
             * После выполнения шаблонизации контрол будет разрушен и будет вызван _beforeDestroy
             * @private
             */
            _beforeMount: function(options, context, receivedState){
            },

            /**
             * После отрисовки контрола. Выполняется на клиенте после синхронизации VDom с реальным Dom
             * Здесь мы можем впервые обратиться к DOMу, сделать какие-то асинхронные операции,
             * и при необходимости запланировать перерисовку
             * Вызывается один раз в течение жизненного цикла
             * Вызывается только на клиенте
             * @private
             */
            _afterMount: function (options, context) {
            },

            /**
             * Точка входа перед шаблонизацией. _beforeUpdate точка применения новых
             * опций для компонента. Здесь мы можем понять измененные опции и как-то повлиять на состяние
             * Вызывается множество раз за жизненный цикл
             * Вызывается только на клиенте
             * @param newOptions - предыдущие опции компонента
             * @private
             */
            _beforeUpdate: function(newOptions, newContext){
            },

            /**
             * Если возвращает false, то рендеринга не происходит
             * @param newOptions
             * @private
             */

            _shouldUpdate: function(newOptions, newContext) {
               return true;
            },

            /**
             * Точка завершения шаблонизации и синхронизации. Здесь доступен DOM и
             * объект this._children
             * Здесь мы можем выполнить асинхронные операции, потрогать DOM
             * Вызывается каждый раз после шаблонизации после _beforeUpdate
             * Вызывается только на клиенте
             * @private
             */

            _afterUpdate: function(oldOptions, oldContext){
            },

            /**
             * Перед разрушением. Точка, когда компонент жив.
             * Здесь нужно разрушить объекты, которые были созданы в _applyOptions
             * Вызывается и на клиенте и на сервере
             * @private
             */
            _beforeUnmount: function(){
            }
            //</editor-fold>
         });



      Base.createControl = function(ctor, cfg, domElement){
         if (optionsResolver.resolveOptions(ctor, cfg)) {
            var ctr = new ctor(cfg);
            ctr._container = domElement;
            FocusHelper.patchDom(domElement, cfg);
            ctr.saveFullContext(contextResolver.wrapContext(ctr, {}));
            ctr.mountToDom(ctr._container, cfg);
            return ctr;
         }
      };

      return Base;

   });