define('Core/Tickers', ['Core/EventBus'], function(EventBus) {
   if (typeof window !== 'undefined') {
        /**
         * TODO разобраться чем обусловлены эти константы
         * TICKER_INTERVAL = TICKER_INTERVAL_FAST * 20
         * TICKER_INTERVAL_SLOW = TICKER_INTERVAL * 3
         * Рассмотреть использование requestAnimationFrame, чтобы не плодить кучу интервалов
         */
      var
         TICKER_INTERVAL_SLOW = 1260,
         TICKER_INTERVAL = 420;

      var ticker = EventBus.channel('ticker');
      // timestamp последнего onTickSlow
      var lastTick = Date.now();
      /*
        * Время, по прошествии которого считаем что машина клиента была в спящем режиме или гибернации
        * Здесь надо учитывать, что последние браузеры замедляют работу вкладок не в фокусе
        * Поэтому пока эксперементально считаем, что двух минут достаточно чтобы отработала функция в условиях подвисания или потери фокуса
       */
        var SLEEP_TIMEOUT = 2 * 60 * 1000;


      setInterval(function() {
         ticker.notify('onTickSlow');
         var timestamp = Date.now();
         if (timestamp - lastTick >= SLEEP_TIMEOUT) {
             EventBus.globalChannel().notify("onWakeUp", lastTick);
         }
         lastTick = timestamp;
      }, TICKER_INTERVAL_SLOW);

      setInterval(function() {
         ticker.notify('onTick');
      }, TICKER_INTERVAL);
   }
});
/**
 * @event         Core/EventBusGlobalChannel#onWakeUp
 * @description   Событие происходит при выходе машины клиента из спящего режима(гибернации)
 * @remark        Считается, что спящий режим(гибернации) наступает через 2 минуты
 * @param {Core/EventObject} event Дескриптор события
 * @param {Number} lastTick Таймстамп активности окна перед спящим режимом
 * @example
 * <pre>
 *    EventBus.globalChannel().subscribe('onWakeUp', function(event, lastTick){
 *       console.log("Wake up, Neo...");
 *       var sleepTime = Date.now() - lastTick;
 *       if (sleepTime > 10 * MIN) {
 *          console.log("The Matrix has you...");
 *       }
 *       console.log("Sleep time: ", sleepTime);
 *    });
 * </pre>
 */