define('Core/constants', ['Core/compatibility', 'Core/detection'], function(compatibility, detection) {
   var now = +new Date(), timeShift2014 = +new Date(2014, 9, 26);
    /**
     * Список констант WS.
     * Ряд констант не описаны, однако их можно найти в исходном файле классе.
     * @class Core/constants
     * @public
     */
   var constants = /** @lends Core/constants.prototype */{
      isNewNav: true, // флаг для data-provider-а навигации, о том, что нужно использовать новый компонент.
      JSONRPC_PROOTOCOL_VERSION: 5,
      fasttemplate: false,
      nostyle: false,
      compat: (typeof window === 'undefined' || window && window.location.href.indexOf("withoutLayout")===-1) &&
               (typeof window === 'undefined' || window.wsConfig.compatible!==false),
      moscowTimeOffset: now >= timeShift2014 ? 3 * 60 : 4 * 60, // UTC+04:00 ... 26/10/2014 UTC+03:00
      javaStartTimeout: 15000,
      styleLoadTimeout: 4000, // Поднял до 4 секунд, так как на медленном соединении не успевало отработать
      IDENTITY_SPLITTER: ',',
      userConfigSupport: false,
      globalConfigSupport: true,
      theme: 'wi_scheme',
      appRoot: '/',
      resourceRoot: '/resources/',
      wsRoot: '/ws/',
      cdnRoot: '/cdn/',
      defaultServiceUrl: '/service/',
      debug: false,
      saveLastState: true,
      checkSessionCookie: true,
      /**
       * @cfg {String} Адрес для автоматической подгрузки шрифтов семейства TensorFont.
       * По умолчанию они лежат в теме-скрепке, но для особых случаев можно задать особый адрес.
       * Если задана пустая строка, то шрифты подгружаться не будут.
       */
      tensorFontsUrl: ['SBIS3.CONTROLS/themes/fonts/tensorFont.css', 'SBIS3.CONTROLS/themes/fonts/cbus-icons.css', 'SBIS3.CONTROLS/themes/general.css'],
      i18n: false,
      defaultLanguage: '',

      layoutConfig: {
         contentArea: ['#contentArea', '#contentAreaEdit', '#ctr']
      },
      /***
       * Флаг показывает, что код исполняется на клиентской стороне, на браузере
       */
      isBrowserPlatform: typeof window !== 'undefined',
      /**
       * Флаг показывает, что код исполняется на серверной стороне, на nodejs
       */
      isNodePlatform: typeof process !== 'undefined',
      isServerScript: typeof window === 'undefined' && typeof process === 'undefined',

      /**
       * Флаг показывает идет ли верстка на сервере, при этом он false в юнит тестах под нодой
       */
      isBuildOnServer: typeof process !== 'undefined' && process.application,

      /**
       * Константы с поддержкой тех или иных фич
       * @namespace
       */
      compatibility: compatibility,
      /**
       * Константы, которые можно использовать для проверки браузера
       * @namespace
       */
      browser: detection,
      buildnumber: "", // jshint ignore:line
      resizeDelay: 40,
      /**
       * Пакеты XML-файлов
       * Формат:
       *    ИмяШаблона: ФайлПакета, ...
       */
      xmlPackages: {},
      /**
       * Пакеты JS-файлов
       * Формат:
       *    ОригинальноеИмяФайла: ИмяФайлаПакета, ...
       */
      jsPackages: {},
      /**
       * Оглавление. Сопоставление коротких имен полным путям
       * Формат:
       *    ИмяШаблона: ФайлШаблона, ...
       */
      xmlContents: {},
      /**
       * Альтернативные имена входного файла
       */
      htmlNames: {},
      /**
       * Декларативная привязка обработчиков на ресурс
       * Формат:
       *    ИмяШаблона: [ ФайлОбработчиков, ФайлОбработчиков, ... ], ...
       */
      hdlBindings: {},
      /**
       * Хосты, с которых запрашиваются файлы
       * Формат:
       *    Хост1, Хост2, Хост3, ...
       */
      hosts: [],
      services: {},
      modules: {},
      jsCoreModules: {},
      jsModules: {},
      dictionary: {},
      classLoaderMappings: {},
      /**
       * Расстояние, утянув объект на которое, начинается drag&drop
       */
      startDragDistance: 4,
      availableLanguage: {},
      /**
       * Классы контролов-окон, которые могут быть родителями других окон и всплывающих панелей.
       */
      WINDOW_CLASSES: ['Lib/Control/FloatArea/FloatArea', 'Lib/Control/Window/Window', 'Deprecated/Controls/FieldEditAtPlace/FieldEditAtPlace'],
      key: {
         left: 37,
         up: 38,
         right: 39,
         down: 40,
         insert: 45,
         del: 46,
         space: 32,
         backspace: 8,
         minus: 109,
         plus: 107,
         enter: 13,
         esc: 27,
         f1: 112,
         f3: 114,
         f4: 115,
         f5: 116,
         f7: 118,
         f12: 123,
         meta: 157,
         underscore: 95,
         pageUp: 33,
         pageDown: 34,
         end: 35,
         home: 36,
         tab: 9,
         ctrl: 17,
         b: 66,
         h: 72,
         v: 86,
         y: 89,
         q: 81,
         p: 80,
         m: 77,
         n: 78,
         o: 79,
         androidAnyKey: 229
      },
      modifiers: {
         nothing: 0,
         shift: 2,
         control: 4,
         alt: 8
      },
      operaKeys: {
         43: 'plus',
         45: 'minus'
      },
      NON_CTRL: '.ws-non-control',
      IE_ACTIVEOBJECT_XML_PRINT_TYPE: 'Microsoft.XmlDom',
      /**
       * Используется что бы не показвать сообщения в консоль на продакшене, возможно надо придумать что то по лучше
       */
      isProduction: (['inside.tensor.ru', 'online.sbis.ru'].indexOf(typeof location !== 'undefined' ? location.host : '') > -1)
   };

   var global = (function() { return this || (0,eval)('this') })();
   global.IS_PRODUCTION = constants.isProduction; //для nativeExtensions

   return constants;
});