define('Core/core-extensions', [
   'Core/constants',
   'Core/Deferred',
   'Core/ParallelDeferred',
   'Core/SessionStorage',
   'Core/Context',
   'Core/IoC',
   'Core/UserConfig',
   'Core/ClientsGlobalConfig',
   'Core/RightsManager',
   'Core/ExtensionsManager',
   'text!View/Builder/Tmpl/modules/templates/objectFunctionHeaderTemplate.jstpl',
   'is!browser?WS.Data/ContextField/Flags',
   'is!browser?WS.Data/ContextField/Record',
   'is!browser?WS.Data/ContextField/Enum',
   'is!browser?WS.Data/ContextField/List'
], function(
   _const,
   Deferred,
   ParallelDeferred,
   SessionStorage,
   Context,
   IoC,
   UserConfig,
   ClientsGlobalConfig,
   RightsManager,
   ExtensionsManager
) {
   var
      pd,
      configDefined = SessionStorage.get("globalConfigDefined"),
      white_list = ['sid', 'ИдентификаторНашейОрганизации', 'ИдентификаторНашейОрганизацииОпции', 'IdCOOAcc', 'IdCOOAccOpts', 'ФункционалНавигация'], // [всем, Толик Антонов, Толик Антонов, Толик Антонов, Толик Антонов, Овчинников]
      _getValue = Context.global.getValue;

   function push(def) {
      if (!pd) {
         pd = new ParallelDeferred({stopOnFirstError: false, maxRunningCount: 1});
      }
      pd.push(def);
   }

   function pushToGlobalContext() {
      white_list.forEach(function (fieldName) {
         // sid не пишем
         if (fieldName !== 'sid' && fieldName) {
            Context.global.setValue(fieldName, SessionStorage.get(fieldName));
         }
      });
      return true;
   }

   Context.global.getValue = function (fieldName) {
      if (white_list.indexOf(fieldName) < 0 && SessionStorage.get(fieldName) !== undefined) {
         IoC.resolve('ILogger').error('Global Context', 'Попытка получить параметр "' + fieldName + '" из глобального контекста. ' +
            'Используйте $ws.single.ClientsGlobalConfig.getParam или $ws.single.UserConfig.getParam в зависимости от вида параметра.');
      } else {
         return _getValue.apply(Context.global, arguments);
      }
   };

   function checkWhiteListParams() {
      var paramsToGet = white_list.filter(function (p) {
         return !SessionStorage.get(p);
      });

      //Если все параметры пользователя есть, то не будем дергать бд
      if (paramsToGet.length) {
         return UserConfig.getParams(paramsToGet);
      }
   }

   if (!ExtensionsManager.extensionsLoaded()) {
         push(ExtensionsManager.loadExtensions());
      }

   if (_const.rights) {
      var rights = RightsManager.getRights();
      if (rights instanceof Deferred) {
         push(rights.addCallback(function(rights) {
            window.rights = rights;
         }));
      }
   }

   if (_const.globalConfigSupport && !configDefined) {
      push(function() {
         var def = new Deferred();
         def.dependOn(ClientsGlobalConfig.getParams());
         SessionStorage.set('globalConfigDefined', true, true);
         return def;
      })
   }

   var extensonDeferred = (pd ? pd.done().getResult() : Deferred.success()).addCallback(function () {
      if (_const.userConfigSupport) {
         if (!configDefined) {
            return UserConfig.getParams();
         } else {
            //Гарантируем наличие в глобальном контексте параметров из белого списка.
            return checkWhiteListParams();
         }
      }
   });

   return extensonDeferred.addBoth(function() {
      return pushToGlobalContext();
   });
});