define('Core/core-instance', [
   'require',
   'text!Core/resources/deprecatedModules.json',
   'Core/detection'
], function(
   require,
   deprecatedModules
) {
   var _requireCache = {},
      _definedCache = {},
      _isNode = typeof window === 'undefined',
      deprecatedModules = JSON.parse(deprecatedModules),
      _requireFromCache = function(name) {
         if (_isNode) {
            return require(name);
         }
         return _requireCache[name] || (_requireCache[name] = require(name));
      },
      _definedFromCache = function(name) {
         if (_isNode) {
            return require.defined(name);
         }
         return _definedCache[name] || (_definedCache[name] = require.defined(name));
      },
      _isInheritedFrom = function(inst, moduleName) {
         var proto = Object.getPrototypeOf(inst);
         if (proto) {
            return proto._moduleName === moduleName || _isInheritedFrom(proto, moduleName);
         } else {
            return false;
         }
      },

      /**
       * @class Core/core-instance
       * @public
       * @author Мальцев А.А.
       */
      coreInstance = /** @lends Core/core-instance.prototype */{
          /**
           * Возвращает признак: является ли объект экземпляром класса, с которым производят сравнение.
           * @param {*} inst Проверяемый объект.
           * @param {String} moduleName Имя модуля, в котором определён класс. Например, SBIS3.CONTROLS.Button.
           * @returns {Boolean} false - модуль не определен.
           * @example
           * <pre>
           *     if (helpers.instanceOfModule(this, 'Lib/Control/FloatArea/FloatArea')) {
           *        console.log('Это FloatArea');
           *     }
           * </pre>
           * @function
           */
         instanceOfModule: function(inst, moduleName) {
            //на время действия проекта отказ от оглавлений заменяем старое название на новое, для проверки инстанса, потом надо убрать
            moduleName = deprecatedModules.hasOwnProperty(moduleName) ? deprecatedModules[moduleName] : moduleName;

            //Support fast check by marker property
            var moduleMarker = '[' + moduleName + ']';
            if (inst && inst[moduleMarker]) {
               return true;
            }

            if (
               inst &&
               inst._moduleName !== undefined
            ) {
               return _isInheritedFrom(inst, moduleName);
            }
            var Module;
            if (_definedFromCache('js!' + moduleName)) {
                Module = _requireFromCache('js!' + moduleName);
                if (Module && inst instanceof Module) {
                    return true;
                }
            }
            if (_definedFromCache(moduleName)) {
                Module = _requireFromCache(moduleName);
                if (Module && inst instanceof Module) {
                    return true;
                }
            }

            return false;
         },
          /**
           * Возвращает признак: является ли объект миксином, с которым производят сравнение.
           * @param {*} inst Проверяемый объект.
           * @param {String} moduleName Имя модуля, в котором определён миксин.
           * @returns {Boolean} false - модуль не определен.
           * @function
           */
         instanceOfMixin: function(inst, mixin) {
             //на время действия проекта отказ от оглавлений заменяем старое название на новое, для проверки инстанса, потом надо убрать
            mixin = deprecatedModules.hasOwnProperty(mixin) ? deprecatedModules[mixin] : mixin;

            if (!inst || !inst._mixins) {
               return false;
            }

            //Support fast check by marker property
            var mixinMarker = '[' + mixin + ']';
            if (inst[mixinMarker]) {
               return true;
            }

            if (_definedFromCache('js!' + mixin) && inst._mixins.indexOf(_requireFromCache('js!' + mixin)) !== -1) {
               return true;
            }

            return _definedFromCache(mixin) ?
               inst._mixins.indexOf(_requireFromCache(mixin)) !== -1 :
               false;
         },

         classOfMixin: function(classFn, mixin) {
            var proto = classFn.prototype;
            return coreInstance.instanceOfMixin(proto, mixin);
         }
      };

   return coreInstance;
});