define('Core/core-merge', [], function() {
   /**
    * Объединение двух hash.
    * @param {Object} hash Исходный хэш.
    * @param {Object} hashExtender хэш-расширение.
    * @param {Object} [cfg] Параметры работы.
    * @param {Boolean} [cfg.preferSource=false] Сохранять или нет исходное значение. По умолчанию исходное значение не сохраняется.
    * @param {Boolean} [cfg.rec=true] Рекурсивное объединение (по умолчанию - рекурсивное объединение включено).
    * @param {Boolean} [cfg.clone=false] Клонировать элементы или передавать по ссылке (по умолчанию - по ссылке).
    * @param {Boolean} [cfg.create=true] Создавать элементы, отсутствующие в исходном объекте. По умолчанию включено.
    * @param {Boolean} [cfg.noOverrideByNull=false] Запретить заменять исходные значения на null. По умолчанию выключено.
    * @param {Boolean} [cfg.ignoreRegExp=''] Регулярка для игнорирования части свойств
    */
   function wsCoreMerge(hash, hashExtender, cfg, currentKey, path) {
      if (cfg === undefined) {
         cfg = {};
      }
      cfg.preferSource = cfg.preferSource !== undefined ? cfg.preferSource : false; // не сохранять исходное значение
      cfg.rec = cfg.rec !== undefined ? cfg.rec : true;  // объединять рекурсивно
      cfg.clone = cfg.clone !== undefined ? cfg.clone : false; // не клонировать элементы (передаем по ссылке)
      cfg.create = cfg.create !== undefined ? cfg.create : true; // создавать элементы, которых нет в исходном хэше
      cfg.noOverrideByNull = cfg.noOverrideByNull !== undefined ? cfg.noOverrideByNull : false; // не заменять значения null'овыми

      if (hashExtender instanceof Date) {
         if (cfg.clone) {
            return new Date(hashExtender);
         } else {
            hash = hashExtender;
            return hash;
         }
      }

      function isMergeableObject(o) {
         return o && ((o.constructor === Object && !('$constructor' in o)) || o.constructor === Array);
      }

      function cloneOrCopy(i) {
         /**
          * Если к нам пришел объект и можно клонировать
          * Запускаем мерж того, что пришло с пустым объектом (клонируем ссылочные типы).
          */
         if ((typeof(hashExtender[i]) === 'object' && hashExtender[i] !== null) && cfg.clone) {
            if (isMergeableObject(hashExtender[i])) {
               hash[i] = wsCoreMerge(hashExtender[i] instanceof Array ? [] : {}, hashExtender[i], cfg, i, path);
            }
            else {
               hash[i] = hashExtender[i];
            }
         }
         /**
          * Иначе (т.е. это
          *  ... не объект (простой тип) или
          *  ... запрещено клонирование)
          *
          * Если к нам пришел null и запрещено им заменять - не копируем.
          */
         else if (!(hashExtender[i] === null && cfg.noOverrideByNull)) {
            hash[i] = hashExtender[i];
         }
      }

      if (!path) {
         path = path || {keys: [], objects: []};
      }

      if (typeof(hash) === 'object' && hash !== null && typeof(hashExtender) === 'object' && hashExtender !== null) {
         path.keys.push(currentKey === undefined ? '.' : currentKey);
         if (path.objects.indexOf(hashExtender) > -1) {
            throw new Error('Recursive traversal detected for path "' + path.keys.join(' -> ') + '" with ' + hashExtender);
         }
         path.objects.push(hashExtender);

         for (var i in hashExtender) {
            if (!hashExtender.hasOwnProperty(i)) {
               continue;
            }

            if (cfg.ignoreRegExp && cfg.ignoreRegExp.test(i)){
               continue;
            }
            // Если индекса в исходном хэше нет и можно создавать
            if (hash[i] === undefined) {
               if (cfg.create) {
                  if (hashExtender[i] === null) {
                     hash[i] = null;
                  } else {
                     cloneOrCopy(i);
                  }
               }
            } else if (!cfg.preferSource) { // Индекс есть, исходное значение можно перебивать
               if (hash[i] && typeof hash[i] === 'object' && typeof hashExtender[i] === 'object') {
                  // Объект в объект
                  if (hash[i] instanceof Date) {
                     if (hashExtender[i] instanceof Date) {
                        if (cfg.clone) {
                           hash[i] = new Date(+hashExtender[i]);
                        } else {
                           hash[i] = hashExtender[i];
                        }
                        continue;
                     } else {
                        // Исходный - дата, расщирение - нет. Сделаем пустышку в которую потом замержим новые данные
                        hash[i] = hashExtender[i] instanceof Array ? [] : {};
                     }
                  } else {
                     if (hashExtender[i] instanceof Date) {
                        if (cfg.clone) {
                           hash[i] = new Date(+hashExtender[i]);
                        } else {
                           hash[i] = hashExtender[i];
                        }
                        continue;
                     }
                  }

                  if (cfg.rec && (isMergeableObject(hashExtender[i]) || hashExtender[i] === null) && Object.keys(hash[i]).length > 0) {
                     hash[i] = wsCoreMerge(hash[i], hashExtender[i], cfg, i, path);
                  } else {
                     hash[i] = hashExtender[i];
                  }
               } else { // Перебиваем что-то в что-то другое...
                  cloneOrCopy(i);
               }
            }
            /**
             * Исходное значение имеет приоритет, но разрешена рекурсия
             * Идем глубже.
             */
            else if (typeof hash[i] === 'object' && typeof hashExtender[i] === 'object' && cfg.rec) {
               if (isMergeableObject(hashExtender[i]) || hashExtender[i] === null) {
                  hash[i] = wsCoreMerge(hash[i], hashExtender[i], cfg, i, path);
               } else {
                  // Если это сложный объект мы ничего не делаем, т.к. сюда мы попадем только в том случае, если preferSource === true
                  // А значит нам нельзя здесь ничего перетирать
               }
            }
         }

         path.keys.pop();
         path.objects.pop();
      } else if (!(hashExtender === null && cfg.noOverrideByNull) && !cfg.preferSource) {
         hash = hashExtender;
      }

      return hash;
   }

   var global = (function() { return this || (0,eval)('this') })();
   global.wsCoreMerge = global.wsCoreMerge || wsCoreMerge;

   return wsCoreMerge;
});
