define('Core/helpers/Array/clone', [
   'Core/helpers/Object/isPlainObject',
   'Core/core-merge'
], function(isPlainObject, coreMerge) {
   /**
    * Модуль, в котором описана функция <b>clone(array)</b>.
    *
    * Создает копию не только основного массива, как делает slice, но также создает копию внутренних массивов и объектов.
    *
    * <h2>Параметры функции</h2>
    * <ul>
    *     <li><b>array</b> {Array} - массив.</li>
    * </ul>
    *
    * <h2>Возвращает</h2>
    * Возвращает копию массива.
    *
    * @class Core/helpers/Array/clone
    * @public
    * @author Мальцев А.А.
    */
   //ws: ws/lib/Control/DataViewAbstract/plugins/Print-plugin.js, рекорд, рекордсет
   //контролы: SBIS3.CONTROLS.MultiSelectable, components/OperationsPanel/OperationsPanel-components/OperationMerge/resources/MergeDialogTemplate/MergeDialogTemplate.module.js
   return function clone(array) {
      var copy = array.slice(),
         obj = {};
      for (var i = 0, l = copy.length; i < l; i++) {
         if (copy[i]) {
            if (Array.isArray(copy[i])) {
               copy[i] = clone(copy[i]);
            } else if (isPlainObject(copy[i])) {
               obj = copy[i] = coreMerge({}, copy[i]);
               for (var j in obj) {
                  if (obj.hasOwnProperty(j) && Array.isArray(obj[j])) {
                     obj[j] = clone(obj[j]);
                  }
               }
            }
         }
      }
      return copy;
   };
});
