define('Core/helpers/Object/isEmpty', function() {
   /**
    *
    * Модуль, в котором описана функция <b>isEmpty(obj)</b>, которая проверяет, что переданный объект является пустым (не содержит собственных ключей).
    * Функция возвращает признак {Boolean}, где true - объект является пустым.
    *
    * <h2>Пример использования</h2>
    * <pre>
    *    require(['Core/helpers/Object/isEmpty'], function(isEmpty) {
    *       console.log(isEmpty({}));//true
    *       console.log(isEmpty({foo: bar}));//false
    *    });
    * </pre>
    *
    * @class Core/helpers/Object/isEmpty
    * @public
    * @author Мальцев А.А.
    */
   return function isEmpty(obj) {
      if (obj === null || typeof(obj) !== 'object') {
         return false;
      }

      if (obj instanceof Object) {
         for (var i in obj)
            return false;
      } else if (obj instanceof Array) {
         return obj.length === 0;
      }

      return true;
   };
});
