define('Core/helpers/fast-control-helpers', [
   'require',
   'Core/PrintDialogHTMLView',
   'Core/deprecated',
   'Core/helpers/Hcontrol/toggleLocalIndicator',
   'Core/helpers/Hcontrol/showFloatArea',
   'Deprecated/helpers/fast-control-helpers',
   'Core/helpers/Hcontrol/openErrorsReportDialog',
   'Core/IoC',
   'Core/constants',
   'Core/Deferred',
   'Core/Context'
], function(require, PrintDialogHTMLView, cDeprecated, toggleLocalIndicator, showFloatArea, deprecatedHelpers, openErrorsReportDialog) {

   var helperDeprecator = cDeprecated.callNextWrapper;
   var getHelpersDeprecatorWrapper = function (helperName) {
      return function () {
         cDeprecated.showInfoLog('Хелпер Core/helpers/fast-control-helpers::' + helperName + ' помечен как deprecated и будет удален в 3.18.');
      }
   };
    /**
     * @class Core/helpers/fast-control-helpers
     * @public
     * @author Мальцев А.А.
     * @deprecated
     */
   var fastHelpers = /** @lends  Core/helpers/fast-control-helpers.prototype */{
        /**
         * Производит создание календаря на основе контрола {@link Deprecated/Controls/FieldDate/FieldDate}.
         * @param {String|jQuery} id Идентификатор элемента, либо jQuery элемент, в который вставляем календарь.
         * @param {Function} func Устанавливает функцию для изменения отображения дней в календаре.
         * @param {Date} [firstDayToShow] Дата дня, на котором нужно сфокусировать календарь. Вызов showDatePicker("datepicker", dayFormat);
         * @returns {Core/Deferred} Нужно для того, чтобы знать, что календарик уже нарисовался.
         * @example
         * Пример функции func:
         * <pre>
         * // <div id="test"></div> Должен существовать блок с идентификатором 'test'
         * helpers.showDatePicker('test', function dayFormat(date) {
         *    // Если отображаемый день месяца меньше 15
         *    if (date.getDate() < 15) {
         *       // [может ли быть выбрано, 'css-класс для добавления к ячейке дня', "текст всплывающей подсказки"]
         *       return [true, 'ui-state-highlight', "tooltip"];
         *    }
         *    return [true, ''];
         * });
         * </pre>
         */
      showDatePicker: helperDeprecator(deprecatedHelpers.showDatePicker, getHelpersDeprecatorWrapper('showDatePicker')),
      question: helperDeprecator(deprecatedHelpers.question, getHelpersDeprecatorWrapper('question')),
        /**
         * Показывает сообщение.
         * @param {String} text Текст.
         * @param {Object} cfg Параметры, которые могут переопределять/дополнять стандартную конфигурацию окна.
         * @param {String} type Тип: 'info', 'error' или 'complete'.
         * @param {Object} opener контрол, который вызвал функцию (нужен в случае вызова из floatArea)
         * @returns {Core/Deferred} Возвращает Deferred, который сработает при закрытии окна.
         * @private
         */
      _messageBox: helperDeprecator(deprecatedHelpers._messageBox, getHelpersDeprecatorWrapper('_messageBox')),
      alert: helperDeprecator(deprecatedHelpers.alert, getHelpersDeprecatorWrapper('alert')),
      message: helperDeprecator(deprecatedHelpers.message, getHelpersDeprecatorWrapper('message')),
        /**
         * Создает контекстное меню по упрощенным параметрам - список пунктов и м.б. функция активации.
         * @param {Array} items Пункты меню.
         * @param {Function} [onActivate] Функция, которая будет выполняться при выборе пункта меню.
         * @returns {Core/Deferred} Возвращает созданным контекстным меню.
         */
      newContextMenu: helperDeprecator(deprecatedHelpers.newContextMenu, getHelpersDeprecatorWrapper('newContextMenu')),
      showHTMLForPrint: PrintDialogHTMLView,
       /**
        * Функция без описания.
        */
      toggleIndicator: deprecatedHelpers.toggleIndicator,
      toggleLocalIndicator: toggleLocalIndicator,
      showFloatArea: showFloatArea,
      showHoverFloatArea: function (element, hovered, config) {
         var
            hover = hovered || false,
            block = $(element).bind('mouseenter.wsShowHoverFloatArea', function () {
               hover = true;
            }).bind('mouseleave.wsShowHoverFloatArea', function () {
               hover = false;
            });

         config.autoShow = false;
         config.hoverTarget = element;

         return fastHelpers.showFloatArea(config).addCallback(function (area) {
            block.unbind('.wsShowHoverFloatArea');
            if (hover) {
               area.show();
            }
            return area;
         });
      },
        /**
         * Вызываем DialogSelector, выполняет функцию для каждой выбранной записи, если функция возвращает Deferred, то они объединяются в ParallelDeferred;
         * и ParallelDeferred дождется их завершения с любым результатом, в этом случае будет показан индикатор.
         * @param {Object} cfg Конфиг класса {@link Deprecated/Controls/DialogSelector/DialogSelector}, не стоит использовать handlers.
         * @param {Function} func Функция вида func(record, номер_среди_выбранных, массив_выбранных_записей).
         * @param {Core/Context} [ctx] Контекст, чтобы можно было обращаться из фильтра браузера к полям контекста.
         * @return {Core/Deferred} стрельнет, когда будет произведен выбор, и все результаты будут обработаны.
         */
      showDialogSelector: helperDeprecator(deprecatedHelpers.showDialogSelector, getHelpersDeprecatorWrapper('showDialogSelector')),
        /**
         * Вызываем flowAreaSelector, выполняет функцию для каждой выбранной записи, если функция возвращает Deferred, то они объединяются в ParallelDeferred;
         * и ParallelDeferred дождется их завершения с любым результатом, в этом случае будет показан индикатор.
         * @param {Object} cfg Конфиг класса {@link Deprecated/Controls/DialogSelector/FloatAreaSelector}.
         * @param {Function} func Функция вида func(record, номер_среди_выбранных, массив_выбранных_записей).
         * @param {Core/Context} [ctx] Контекст, чтобы можно было обращаться из фильтра браузера к полям контекста.
         * @return {Core/Deferred} стрельнет, когда будет произведен выбор, и все результаты будут обработаны.
         */
      showFloatAreaSelector: helperDeprecator(deprecatedHelpers.showFloatAreaSelector, getHelpersDeprecatorWrapper('showFloatAreaSelector')),
        /**
         * Открывает диалог с результатами выполнения массовых операций.
         * @param {Object} cfg Конфигурация.
         * @example
         * <pre>
         *    helpers.openErrorsReportDialog({
         *       'numSelected': cntSelected, // Количество обработанных\выделенных записей
         *       'numSuccess': cntSuccess,   // Количество успешно выполненных операций
         *       'errors': errors,        // Array, Список всех ошибок.
         *       'title': 'Итоги операции: "Отправить"' // Заголовок - опция для вывода названия операции, которая выполнялась
         *    });})
         * </pre>
         * @see showDatePicker
         * @see question
         * @see alert
         * @see message
         * @see newContextMenu
         * @see showHTMLForPrint
         * @see toggleLocalIndicator
         * @see showFloatArea
         * @see showHoverFloatArea
         * @see showDialogSelector
         * @see showFloatAreaSelector
         */
      openErrorsReportDialog: openErrorsReportDialog
    };

    return fastHelpers;
});