define('Core/helpers/generate-helpers', [
   'Core/constants',
   'Core/IoC',
   'Core/helpers/Object/isEmpty',
   'Core/helpers/string-helpers-min',
   'Core/helpers/random-helpers'
], function(constants, IoC, isEmptyObject, strHelpersMin, randomHelpers){
    /**
     * @class Core/helpers/generate-helpers
     * @public
     * @author Мальцев А.А.
     * @deprecated
     */
   var genHelpers = /** @lends Core/helpers/generate-helpers.prototype */ {
      /**
       * Создаёт URL для открытия страницы редактирования/создания записи.
       * @param {String} recordId Идентификатор записи.
       * @param {Boolean} isBranch Признак: узел или лист.
       * @param {Number} parentId Идентификатор родителя отображаемой записи.
       * @param {Boolean} isCopy Производить ли копирование записи.
       * @param {String} editDialogTemplate Имя шаблона диалога редактирования элемента.
       * @param {String} id Идентификатор браузера.
       * @param {Boolean} readOnly Будет ли невозможно менять содержимое.
       * @param {Object} dataSource Параметры получения данных.
       * @param {Object} filter Текущий фильтр.
       * @param {Object} [reports] Список отчетов.
       * @param {Object} [handlers] Обработчики событий.
       * @param {String} [columnParentId] Поле иерархии.
       * @return {String|Boolean}
       * @deprecated Используйте функцию {@link $Core/helpers/generate-helpers#generatePageURL}.
       * @see generatePageURL
       * @see randomId
       * @see createGUID
       */
      generateURL: function (recordId, isBranch, parentId, isCopy, editDialogTemplate, id, readOnly, dataSource, filter, reports, handlers, columnParentId) {
         var params = {
            recordId: recordId,
            isBranch: isBranch,
            parentId: parentId,
            isCopy: isCopy,
            editDialogTemplate: editDialogTemplate,
            id: id,
            readOnly: readOnly,
            dataSource: dataSource,
            filter: filter,
            reports: reports,
            handlers: handlers,
            columnParentId: columnParentId
         };
         return genHelpers.generatePageURL(params, false);
      },

      /**
       * Создаёт URL для открытия страницы редактирования/создания записи.
       * @param {Object} cfg Объект параметров функции.
       * @param {String} cfg.recordId Идентификатор записи.
       * @param {Boolean} cfg.isBranch Узел или лист.
       * @param {Number} cfg.parentId Идентификатор родителя отображаемой записи.
       * @param {Boolean} cfg.isCopy Копирование.
       * @param {String} cfg.editDialogTemplate Имя шаблона диалога редактирования элемента.
       * @param {String} cfg.id Идентификатор браузера.
       * @param {Boolean} cfg.readOnly Будет ли невозможно менять содержимое.
       * @param {Object} cfg.dataSource Параметры получения данных.
       * @param {Object} cfg.filter Текущий фильтр.
       * @param {Object} cfg.reports Список отчетов.
       * @param {Object} cfg.handlers Обработчики событий. Ключ - имя события, значение - путь к обработчику
       * @param {String} cfg.columnParentId Поле иерархии.
       * @param {String} cfg.changedRecordValues Хэш-меп значений, которые уже изменены в записи и которые нужно перенести на страницу редактирования.
       * @param {Boolean} cfg.history
       * @param {Boolean} [retParams] Признак того, как возвращать результат: в виде объекта (true) или строки (false).
       * @param {String} [url]
       * @return {String|Boolean}
       * @example
       * <pre>
       *  var params = {
       *      recordId : "1",
       *      editDialogTemplate : "Edit",
       *      id: view.getid(),
       *      readOnly: false,
       *      dataSource: dataSource,
       *      reports : reports,
       *      handlers : {
       *       'onBeforeCreate': ['/resources/Module/handlerFile/handlerFunction']
       *      }
       *  },
       *  paramsObj = helpers.generatePageURL(params, true );
       * </pre>
       * @see generateURL
       * @see randomId
       * @see createGUID
       */
      generatePageURL: function (cfg, retParams, url) {
         if ((cfg.editDialogTemplate && (constants.htmlNames[cfg.editDialogTemplate] || constants.xmlContents[cfg.editDialogTemplate]))) {
            var isHierarchyMode = !!(cfg.isBranch),
               hdlIsObject = cfg.handlers && Object.prototype.toString.call(cfg.handlers) == "[object Object]",
               params = {
                  id: cfg.id,
                  hierMode: isHierarchyMode,
                  pk: cfg.recordId,
                  copy: cfg.isCopy || false,
                  readOnly: cfg.readOnly || false,
                  obj: cfg.dataSource.readerParams.linkedObject,
                  _events: {}
               },
               editTemplate,
               pageURL;
            if (url)
               pageURL = url;
            else if (constants.htmlNames[cfg.editDialogTemplate]) {
               var arr = constants.htmlNames[cfg.editDialogTemplate].split('/');
               pageURL = arr[arr.length - 1];
            }
            else {
               editTemplate = constants.xmlContents[cfg.editDialogTemplate].split('/');
               pageURL = constants.appRoot + editTemplate[editTemplate.length - 1] + ".html";
            }
            params["changedRecordValues"] = cfg.changedRecordValues;
            params["history"] = cfg.history;
            params["format"] = cfg.dataSource.readerParams.format;
            params["type"] = cfg.dataSource.readerType;
            if (cfg.dataSource.readerParams.otherURL !== constants.defaultServiceUrl)
               params["url"] = cfg.dataSource.readerParams.otherURL;
            params["db"] = cfg.dataSource.readerParams.dbScheme;
            params["method"] = cfg.dataSource.readerParams.queryName;
            params["readMethod"] = cfg.dataSource.readerParams.readMethodName;
            params["createMethod"] = cfg.dataSource.readerParams.createMethodName;
            params["updateMethod"] = cfg.dataSource.readerParams.updateMethodName;
            params["destroyMethod"] = cfg.dataSource.readerParams.destroyMethodName;
            if (isHierarchyMode) {
               params["branch"] = cfg.isBranch;
               params["pId"] = cfg.parentId;
               if (cfg.columnParentId)
                  params["pIdCol"] = cfg.columnParentId;
            }
            if (cfg.recordId === undefined) {
               params["filter"] = cfg.filter;
               params._events["onBeforeCreate"] = hdlIsObject && cfg.handlers.onBeforeCreate || [];
               params._events["onBeforeInsert"] = hdlIsObject && cfg.handlers.onBeforeInsert || [];
            }
            params._events["onBeforeRead"] = hdlIsObject && cfg.handlers.onBeforeRead || [];
            params._events["onBeforeUpdate"] = hdlIsObject && cfg.handlers.onBeforeUpdate || [];
            params._events["onBeforeShowRecord"] = hdlIsObject && cfg.handlers.onBeforeShowRecord || [];
            params._events["onLoadError"] = hdlIsObject && cfg.handlers.onLoadError || [];
            if (cfg.reports && !(isEmptyObject(cfg.reports))) {
               params["reports"] = cfg.reports;
               params._events["onBeforeShowPrintReports"] = hdlIsObject && cfg.handlers.onBeforeShowPrintReports || [];
               params._events["onPrepareReportData"] = hdlIsObject && cfg.handlers.onPrepareReportData || [];
               params._events["onSelectReportTransform"] = hdlIsObject && cfg.handlers.onSelectReportTransform || [];
            }
            if (retParams)
               return params;
            else {
               pageURL += "?editParams=" + encodeURIComponent(strHelpersMin.serializeURLData(params));
               return pageURL;
            }
         } else {
            if (!constants.htmlNames[cfg.editDialogTemplate] && !constants.xmlContents[cfg.editDialogTemplate]) {
               IoC.resolve('ILogger').log('$ws.helpers.generatePageURL', 'ВНИМАНИЕ! Диалог "' + cfg.editDialogTemplate + '" отсутствует в оглавлении!');
            }
            return false;
         }
      },
      randomId: randomHelpers.randomId,
      createGUID: randomHelpers.createGUID
   };

   return genHelpers;
});