define('Core/helpers/random-helpers', [], function () {
    /**
     * @class Core/helpers/random-helpers
     * @public
     * @author Мальцев А.А.
     */
   return /** @lends  Core/helpers/random-helpers.prototype */ {
      /**
       * Возвращает случайный идентификатор в виде "префикс"-123...
       * @remark
       * Идентификатор без префикса составляет 16 знаков.
       * @param {String} [prefix='ws-'] Префикс идентификатора,
       * @returns {string} Случайный идентификатор с префиксом или без него в зависимости от переданного параметра.
       * @example
       * <pre>
       *     helpers.randomId(); // выведет идентификатор вида "ws-5543191684409976"
       * </pre>
       * @see generateURL
       * @see generatePageURL
       * @see createGUID
       */
      randomId: function (prefix) {
         return (prefix || 'ws-') + Math.random().toString(36).substr(2) + (+new Date());
      },
      /**
       * Создает "GUID".
       * @remark
       * В кавычках потому, что он не настоящий, только выглядит как GUID. Используется свой аглоритм, не такой как настоящий.
       * @returns {String} "GUID"
       * @see generateURL
       * @see generatePageURL
       * @see randomId
       */
      createGUID: function () {
         var
            i = 0, s = 0, pad = new Date().getTime().toString(16);
         pad = '000000000000'.substr(0, 12 - pad.length) + pad;
         var p = function () {
            return (pad.substring(s, s += (i++ % 2 ? 2 : 1)) + (((1 + Math.random()) * 0x10000) | 0).toString(16)).substring(0, 4);
         };
         return (p() + p() + '-' + p() + '-' + p() + '-' + p() + '-' + p() + p() + p());
      }
   }
});