define('Core/helpers/string-helpers-min', [
   'Core/base64'
], function (base64) {

    /**
     * @class Core/helpers/string-helpers-min
     * @public
     * @author Мальцев А.А.
     */
    return /** @lends  Core/helpers/string-helpers-min.prototype */{
      /**
       * Переводит строку адреса в строку base64.
       * @param {string} data Строка адреса.
       * @returns {String}
       * @see deserializeURLData
       */
      serializeURLData: function (data) {
         // Шилов 12.10.2012
         // Если это IE8 или мало ли какой паразит, то преобразуем в UTF-8
         // undefined должен обрабатываться как пустая строка, иначе
         // stringify выдаёт undefined, и всё ломается
         var stringified = JSON.stringify(data === undefined ? '' : data);
         if (stringified.indexOf("\\u") != -1)
            stringified = unescape(stringified.replace(/\\u/g, '%u'));

         return base64.encode(stringified);
      },
      /**
       * Переводит строку из base64 в обычную строку.
       * @param {string} serialized
       * @returns {*}
       * @see serializeURLData
       */
      deserializeURLData: function (serialized) {
         var parseFunc = JSON && JSON.parse ? JSON.parse : $.parseJSON;
         return parseFunc(base64.decode(serialized));
      },
      /**
       * Выполняет транслитерацию строки. Заменяет пробелы на _, вырезает мягкий и твердый знаки.
       * @param {String} string Исходная строка для преобразования.
       * @returns {String}
       */
      transliterate: (function () {
         var charMap = {
            "а":"a","б":"b","в":"v","г":"g","д":"d","е":"e","ё":"e","ж":"j","з":"z","и":"i","й":"j","к":"k",
            "л":"l","м":"m","н":"n","о":"o","п":"p","р":"r","с":"s","т":"t","у":"u","ф":"f","х":"h","ц":"ts",
            "ч":"ch","ш":"sh","щ":"sch","ъ":"","ы":"y","ь":"","э":"e","ю":"yu","я":"ya"," ":"_","А":"A","Б":"B",
            "В":"V","Г":"G","Д":"D","Е":"E","Ё":"E","Ж":"J","З":"Z","И":"I","Й":"J","К":"K","Л":"L","М":"M","Н":"N",
            "О":"O","П":"P","Р":"R","С":"S","Т":"T","У":"U","Ф":"F","Х":"H","Ц":"TS","Ч":"CH","Ш":"SH","Щ":"SCH","Ъ":"",
            "Ы":"Y","Ь":"","Э":"E","Ю":"YU","Я":"YA"};
         return function (string) {
            var result = [], c, i, l;
            for (i = 0, l = string.length; i < l; i++) {
               c = string.charAt(i);
               result[i] = (c in charMap) ? charMap[c] : c;
            }
            return result.join('');
         }
      })()
   }
});