define('Core/helpers/string-helpers', [
   'Core/helpers/String/linkWrap',
   'Core/helpers/i18n/moneyToWritten',
   'Core/helpers/i18n/wordCaseByNumber',
   'Core/helpers/string-helpers-min',
   'Core/IoC',
   'Core/helpers/String/diffAt',
   'Core/helpers/String/format',
   'Core/deprecated',
   'Deprecated/helpers/string-helpers',
   'Core/helpers/String/escapeHtml',
   'Core/helpers/String/escapeTagsFromStr',
   'Core/helpers/String/unEscapeHtml',
   'Core/helpers/String/escapeUnicode',
   'Core/helpers/String/unEscapeASCII',
   'Core/helpers/String/escapeASCII',
   'Core/constants',
   'Core/ConsoleLogger'
], function(
   linkWrap,
   moneyToWritten,
   wordCaseByNumber,
   strHelpersMin,
   IoC,
   diffAt,
   stringFormat,
   cDeprecated,
   deprecatedHelpers,
   escapeHtml,
   escapeTagsFromStr,
   unEscapeHtml,
   escapeUnicode,
   unEscapeASCII,
   escapeASCII
) {

   var helperDeprecator = cDeprecated.callNextWrapper;
   var getHelpersDeprecatorWrapper = function (helperName, newName) {
      return function () {
         cDeprecated.showInfoLog('Хелпер Core/helpers/string-helpers::' + helperName + ' помечен как deprecated и будет удален в 3.18.' + (newName ? ' Подключите явно ' + newName + ', чтобы им пользоваться.' : ''));
      }
   };

   var
       /**
        * @class Core/helpers/string-helpers
        * @public
        * @author Мальцев А.А.
        * @deprecated
        */
      strHelpers = /** @lends  Core/helpers/string-helpers.prototype */ {
      urlRegExpString: linkWrap.urlRegExpString,
      excludeLinkString: linkWrap.excludeLinkString,
      removeWhiteSpaces: helperDeprecator(deprecatedHelpers.removeWhiteSpaces, getHelpersDeprecatorWrapper('removeWhiteSpaces')),
      escapeTagsFromStr : escapeTagsFromStr,
      htmlToText: helperDeprecator(deprecatedHelpers.htmlToText, getHelpersDeprecatorWrapper('htmlToText')),
      escapeHtml: escapeHtml,
      unEscapeHtml: unEscapeHtml,
      escapeASCII: helperDeprecator(escapeASCII, getHelpersDeprecatorWrapper('escapeASCII', 'Core/helpers/String/escapeASCII')),
      unEscapeASCII: helperDeprecator(unEscapeASCII, getHelpersDeprecatorWrapper('unEscapeASCII', 'Core/helpers/String/unEscapeASCII')),
      unEscapeHtmlSpecialChars: helperDeprecator(deprecatedHelpers.unEscapeHtmlSpecialChars, getHelpersDeprecatorWrapper('unEscapeHtmlSpecialChars')),
      serializeURLData: strHelpersMin.serializeURLData,
      deserializeURLData: strHelpersMin.deserializeURLData,
      removeInvalidXMLChars: helperDeprecator(deprecatedHelpers.removeInvalidXMLChars, getHelpersDeprecatorWrapper('removeInvalidXMLChars')),
      numToWritten: helperDeprecator(moneyToWritten, getHelpersDeprecatorWrapper('numToWritten', 'Core/helpers/i18n/moneyToWritten')),
      wordCaseByNumber: helperDeprecator(wordCaseByNumber, getHelpersDeprecatorWrapper('wordCaseByNumber', 'Core/helpers/i18n/wordCaseByNumber')),
      escapeUnicode: escapeUnicode,

      /**
       * Выполняет транслитерацию строки. Заменяет пробелы на _, вырезает мягкий и твердый знаки.
       * @param {String} string Исходная строка для преобразования.
       * @returns {String}
       */
      transliterate: strHelpersMin.transliterate,

      parseComplexKey: helperDeprecator(deprecatedHelpers.parseComplexKey, getHelpersDeprecatorWrapper('parseComplexKey', 'Deprecated/helpers/string-helpers::parseComplexKey()')),

      searchSymbolsDifference: helperDeprecator(function(firstWord, secondWord) {
         var pos = diffAt(firstWord, secondWord);
         return pos === -1 ? false : [firstWord.substr(0, pos), firstWord.substr(pos), secondWord.substr(pos)];
      }, getHelpersDeprecatorWrapper('searchSymbolsDifference', 'Core/helpers/String/diffAt')),

      wrapURLs: linkWrap.wrapURLs,

      wrapFiles: linkWrap.wrapFiles,
      /**
       * Проверяет значение на соответсвие типу Деньги, дополняет нулями слева или обрезает, вещественную часть до заданной точности
       * Используется при сериализации значений в рекордсетах, рекордах, моделях итд.
       * Не используйте этот хелпер если у вас точность типа Деньги меньше 4, это будет просто число.
       * Например 1.001 руб - используем тип Number, а если 1.0015 руб то используем этот хелпер.
       *
       * @param {String} value  Значение
       * @param {Numeric} precision длинна вещественной части, по умолчанию 4
       * @returns {String}
       */
      //MOVE_TO КРАЙНОВ
      prepareMoneyByPrecision: function (value, precision ) {
         var groups = /^\-?(\d+)(\.\d{0,})?$/.exec(value);
         if (groups !== null) {
            precision = +precision||4;
            var floatPart = groups[2],
               realPart = groups[1] ;
            if (typeof floatPart === 'undefined') {
               floatPart = '.';
            }
            var len = precision - floatPart.length + 1;
            if (len > 0) {
               for (var i=0;i < len; i++)  floatPart += '0';
            } else if(len < 0) {
               floatPart = floatPart.slice(0, len);
            }
            return realPart+floatPart;
         } else {
            IoC.resolve('ILogger').error('prepareMoneyByPrecision', 'Value containing not valid characters only numbers allowed.');
         }
         return value;
      },

      format: helperDeprecator(stringFormat, getHelpersDeprecatorWrapper('format', 'Core/helpers/String/format'))
   };

   return strHelpers;
});