define('Core/helpers/unrelated-helpers', [
   'Core/constants',
   'Core/core-merge',
   'Core/Deferred',
   'Core/ParallelDeferred',
   'Deprecated/core-attach',
   'Core/Context',
   'Core/pathResolver',
   'Core/core-instance',
   'Core/helpers/fast-control-helpers',
   'Deprecated/ReaderSBIS',
   'Core/helpers/Hcontrol/saveToFile',
    'Core/helpers/getStackTrace'
], function (
   _const,
   coreMerge,
   Deferred,
   ParallelDeferred,
   attach,
   Context,
   pResolver,
   cInstance,
   fastHelpers,
   ReaderSBIS,
   saveToFile,
   getStackTrace
) {


   var
       /**
        * @class Core/helpers/unrelated-helpers
        * @public
        * @author Мальцев А.А.
        * @deprecated
        */
      helpers = /** @lends Core/helpers/unrelated-helpers.prototype */{

         saveToFile: saveToFile,
         /**
          * Подготавливает параметр "Фильтр" для запроса к БЛ.
          * @remark
          * Создаёт объект вида { d: [], s: [] }.
          * @param {Object} filter Установленный фильтр для табличного представления - {@link Deprecated/Controls/TableView/TableView}.
          * @returns {{d: Array, s: Array}} Возвращает объект вида { d: [], s: [] }.
          */
         prepareFilter: function(filter){
            var retval = { d: [], s: [] };
            if(filter && filter instanceof Object) {
               var keys = Object.keys(filter).sort();
               for(var i, j = 0; j < keys.length; ++j) {
                  i = keys[j];
                  if(i == 'pageNum' || i == 'pageCount' || i == 'usePages' || i == 'sorting'){
                     continue;
                  }
                  if(filter.hasOwnProperty(i) && filter[i] !== undefined) {
                     var serialized = ReaderSBIS.serializeParameter(i, filter[i]);
                     if(filter[i] instanceof Object && filter[i].hasOwnProperty('hierarchy')){
                        retval.d = retval.d.concat(serialized.d);
                        retval.s = retval.s.concat(serialized.s);
                     }else{
                        retval.d.push(serialized.d);
                        retval.s.push({ 'n': i, 't': serialized.s });
                     }
                  }
               }
            }
            return retval;
         },

         resolveComponentPath: pResolver.resolveComponentPath,
         /**
          * Вызываем селектор, выполняет функцию для каждой выбранной записи,
          * если функция возвращает Deferred, то они объединяются в ParallelDeferred;
          * и ParallelDeferred дождется их завершения с любым результатом, в этом случае будет показан индикатор.
          * @param cfg {object} - конфиг для селектора
          * @param func {Function} - функция вида func( record, номер_среди_выбранных, массив_выбранных_записей ).
          * @param ctx контекст, чтобы можно было обращаться из фильтра браузера к полям контекста.
          * @param moduleName {string} - подключаемый модуль
          * @returns {Core/Deferred} стрельнет, когда будет произведен выбор, и все результаты будут обработаны.
          * @private
          */
         _showSelector: function (cfg, func, ctx, moduleName) {
            var
               result = new Deferred(),
               finishContextDfr = new Deferred(),
               linkedCtx = Context.createContext(finishContextDfr, null, ctx),
               needIndic = false;

            result.addErrback(function (e) {
               return e;
            });

            attach.attachInstance(moduleName, coreMerge({
               linkedContext: linkedCtx,
               handlers: {
                  onChange: function (event, records) {
                     var self = this;
                     // если выбор только листьев, то для папки может прийти [ null ], проверка на это
                     if (records && records[0]) {
                        // чтобы дождаться всех
                        var ready = new ParallelDeferred({stopOnFirstError: false});
                        records.forEach(function () {
                           var funcRes = func.apply(self, arguments),
                              isDeferred = funcRes instanceof Deferred;

                           // может не нужна ансинхронная обработка, тогда индикатор не покажем
                           needIndic |= isDeferred;
                           if (isDeferred) {
                              ready.push(funcRes);
                           }
                        });
                        if (needIndic) {
                           fastHelpers.toggleIndicator(true);
                        }
                        this.close(ready.done().getResult());
                     }
                  },
                  onAfterClose: function (event, res) {
                     //  если просто закрыли диалог, то res будет false вроде
                     if (res && res instanceof Deferred)
                        result.dependOn(res);
                     else
                        result.callback(res);
                  },
                  onDestroy: function() {
                     finishContextDfr.callback();
                  }
               }
            }, cfg)).addErrback(function (err) {
               result.errback(err);
               return err;
            });
            result.addBoth(function (res) {
               if (needIndic)
                  helpers.toggleIndicator(false);
               return res;
            });
            return result;
         },
         /**
          * Возврат jQuery элемента в виде строки
          * @param {jQuery}jq
          * @return {String}
          */
         jQueryToString: function (jq) {
            var res = [];
            for (var i = 0, l = jq.length; i < l; i++) {
               res.push(jq.get(i).outerHTML);
            }
            return res.join('');
         },
         /**
          * Производит анализ переданного значения. Если массив, то возвращает строку из элементов массива, разделенных запятой.
          * @param {*} value
          * @returns {*} Если переданное значение - массив, то:
          * <ul>
          *    <li>возвращается null, когда массив состоит из одного элемента со значением null.</li>
          *    <li>возвращает строку из элементов массива, разделенных запятой.</li>
          * </ul>
          * Если переданное значение не массив, то оно же и возвращается.
          */
         parseIdentity: function (value) {
            if (value instanceof Array) {
               if (value.length === 1 && value[0] === null)
                  return null;
               return value.join(_const.IDENTITY_SPLITTER);
            }
            else
               return value;
         },
         /**
          * Проверяет поддерживается ли конкретным браузером указанный плагин
          * @param mimeType
          * @returns {boolean}
          */
         checkMimeSupport: function (mimeType) {
            if (navigator.mimeTypes) {
               return !!navigator.mimeTypes[mimeType];
            } else {
               // Невозможно точно выяснить. Скажем что поддерживается.
               return true;
            }
         },
         /**
          * Возвращает путь с готовым блоком картинки или sprite
          * @param {String} img Ссылка на картинку или sprite
          * @returns {String}
          */
         getImageBlock: function (img) {
            var imgBlock = document.createElement('div');
            var isSpriteImage = (img.indexOf('sprite:') !== -1);

            if (!isSpriteImage) {
               $(imgBlock).append('<img src="' + img + '" />');
            }
            else {
               $(imgBlock).addClass(img.replace("sprite:", ""));
            }
            return imgBlock;
         },
         /**
          * Возвращает стек ошибки
          * @param e - ошибка
          */
         getStackTrace: getStackTrace,
         instanceOfModule: cInstance.instanceOfModule,
         instanceOfMixin: cInstance.instanceOfMixin,
         /**
          *
          * @param value
          * @returns {*|boolean}
          */
         detectCommonDataContainer: function (value) {
            return value && value.s && value.d && (value._type == 'record' || value._type == 'recordset');
         },
         axo: (function () {
            var
               n = 'A' + ['ctiv'].concat('bject').join('eXO'),
               w = (function () {return this || (0, eval)('this');}()),
               axo = w[n];

            if (axo) {
               return function mkAxo(name) {
                  return new axo(name);
               }
            }
         })()
      };

   return helpers;
});