define('Core/load-contents', [
   'Core/constants',
   'Core/pathResolver',
   'Core/core-merge',
   'Core/polyfill/Object/assign'
], function (constants, pathResolver, cMerge) {
   /**
    * Прописывает оглавление
    * @param {Object} contents.
    * @param {Boolean} [replace] заменять ли содержимое.
    * @param {Object} [options] опции для указания пути до service (/, /auth/ etc.) и resources (resources, myresources etc.).
    */
   return function(contents, replace, options) {
      if (replace) {
         constants.hosts = [];
         constants.jsPackages = {};
         constants.xmlPackages = {};
         constants.xmlContents = {};
         constants.hdlBindings = {};
         constants.services = {};
         constants.modules = {};
         constants.htmlNames = {};
         constants.jsModules = {};
         constants.dictionary = {};
         constants.availableLanguage = {};
      }

      // Формируем options
      options = options || {};
      options.service = removeLeadingSlash(removeTrailingSlash(options.service || '/'));
      options.resources = removeLeadingSlash(removeTrailingSlash(options.resources || 'resources'));

      function removeLeadingSlash(path) {
         if (path) {
            var head = path.charAt(0);
            if (head === '/' || head === '\\') {
               path = path.substr(1);
            }
         }
         return path;
      }

      function removeTrailingSlash(path) {
         if (path) {
            var tail = path.substr(path.length - 1);
            if (tail === '/' || tail === '\\') {
               path = path.substr(0, path.length - 1);
            }
         }
         return path;
      }

      function resolvePath(path, obj) {
         var result = {},
            key,
            val;

         for (key in obj) {
            if (obj.hasOwnProperty(key)) {
               val = obj[key];
               if (val) {
                  result[key] = path + '/' + removeLeadingSlash(val) + (options.queryParams || '');
               } else {
                  result[key] = path + '/' + pathResolver.resolveModulePath(key);
               }
            }
         }
         return result;
      }

      function pathjoin(path1, path2) {
         if (path1 !== '') {
            var head = path1.charAt(0);

            if (head !== '/' && head !== '\\' && path1.charAt(1) !== ':' && path1.indexOf('http:') < 0) {
               path1 = '/' + path1;
            }
         }
         if (!path1 && /[a-z]+:\/{2}/i.test(path2)) {
            return path2;
         } else {
            return path1 + '/' + path2;
         }
      }

      if (contents) {
         //Detect path to the resources
         var path;
         if (options.service === '' && options.resources === '') {
            path = options.resources;
         } else {
            path = pathjoin(options.service, options.resources);
         }

         //Apply path for some contents keys
         var contentsCopy = Object.assign({}, contents);
         if(contents.jsModules) {
            contentsCopy.jsModules = resolvePath(path, contents.jsModules);
         }
         if (contents.xmlContents) {
            contentsCopy.xmlContents = resolvePath(path, contents.xmlContents);
         }

         //Translate contents to the constants
         constants = cMerge(constants, contentsCopy, {
            preferSource: options.preferSource || false
         });

         //Apply "requirejsPaths" contents section as "paths" in requirejs config
         if (constants.requirejsPaths) {
            requirejs.config({
               paths: constants.requirejsPaths
            });
         }
      }
   }
});
