define('Core/patchRequireJS', [], function() {
   var registerResourceLoadCallback = function (require) {
      //https://github.com/requirejs/requirejs/wiki/Internal-API:-onResourceLoad
      require.onResourceLoad = (function(source) {
         var makeFunctionSerializable = function (func, moduleName, path) {
               func.toJSON = function() {
                  var serialized = {
                     $serialized$: 'func',
                     module: moduleName
                  };
                  if (path) {
                     serialized.path = path;
                  }
                  return serialized;
               }
            },

            makeArraySerializable = function(arr, moduleName, prefix, depth) {
               var arrLength = arr.length,
                  i;

               prefix = prefix ? prefix + '.' : '';
               for (i = 0; i < arrLength; i++) {
                  makeSerializable(depth, arr[i], moduleName, prefix + i);
               }
            },

            makeObjectSerializable = function(obj, moduleName, prefix, depth) {
               var keys = Object.keys(obj),
                  keysLength = keys.length,
                  prop,
                  i;

               prefix = prefix ? prefix + '.' : '';
               for (i = 0; i < keysLength; i++) {
                  prop = keys[i];
                  makeSerializable(depth, obj[prop], moduleName, prefix + prop);
               }
            },

            /**
             * После require js модуля на все функции навешивается toJSON
             * функции ищутся рекурсивно вглубь объектов.
             * Модуль А: { f1 : function(){} }
             * Модуль В: { K :  {
             *                    someFunction: A.f1
             *                  }
             *            }
             * При require модуля B с зависимостью модулем А сначала toJSON будет вызван для
             * функции f1 от объекта А (при загрузке зависимостей)
             * А при загрузке самого модуля В, toJSON для f1 будет вызван от объекта B.K
             * соответственно правильная ссылка будет потеряна.
             */
            makeSerializable = function(depth, obj, moduleName, prefix) {
               if (depth === 0) {
                  return;
               }
               depth--;

               switch(obj && typeof obj) {
                  case 'function':
                     if (!obj.toJSON) {
                        makeFunctionSerializable(obj, moduleName, prefix);
                     }
                     if (obj.constructor) {
                        makeObjectSerializable(obj, moduleName, prefix, depth);
                     }
                     break;

                  case 'object':
                     if (Array.isArray(obj)) {
                        makeArraySerializable(obj, moduleName, prefix, depth);
                     } else if (Object.getPrototypeOf(obj) === Object.prototype) {
                        //is plain Object
                        makeObjectSerializable(obj, moduleName, prefix, depth);
                     }
                     break;
               }
            };

         return function(context, map) {
            var prefix = map.prefix || '',
               exports,
               proto,
               moduleName;

            if (!prefix || prefix === 'js') {
               exports = context.defined[map.id];
               moduleName = map.name;

               if (prefix) {
                  prefix += '!';
               }
               if (typeof exports === 'function') {
                  proto = exports.prototype;
                  if (!proto.hasOwnProperty('_moduleName')) {
                     proto._moduleName = prefix === 'js!' ? moduleName :  prefix + moduleName;
                  }
               }

               makeSerializable(4, exports, prefix + moduleName);
            }

            source && source.apply(this, arguments);
         };
      })(require.onResourceLoad);
   };

   return function patchRequireJS() {
      if (typeof requirejs !== 'undefined') {
         //Store module name in a prototype
         if (registerResourceLoadCallback) {
            registerResourceLoadCallback(requirejs);
            registerResourceLoadCallback = null;//Apply patch once
         }
      }
   }
});
