define('Core/vdom/Synchronizer/resources/DOMEnvironment', [
   'require',
   'Core/core-debug',
   'Core/constants',
   'Core/IoC',
   'Core/helpers/Array/findIndex',
   'Core/helpers/Function/runDelayed',
   'Core/vdom/Synchronizer/resources/VdomMarkup',
   'View/Runner/Vdom/third-party/dom',
   'Core/vdom/Synchronizer/resources/Hooks',
   'Core/vdom/Synchronizer/resources/SyntheticEvent',
   'Core/vdom/Synchronizer/resources/TabIndex',
   'Core/vdom/Utils/FocusHelper',
   'Core/vdom/Synchronizer/resources/Environment',
   'Core/core-extend'
 ],
function(require,
         coreDebug,
         constants, 
         IoC, 
         findIndex, 
         runDelayed,
         VdomMarkup, 
         vdom, 
         Hooks, 
         SyntheticEvent, 
         TabIndex, 
         FocusHelper, 
         Environment) {

   'use strict';
   /**
    * TODO: Изыгин
    * https://online.sbis.ru/opendoc.html?guid=6b133510-5ff0-4970-8540-d5be30e7587b&des=
    * Задача в разработку 01.06.2017 Поднять юнит тестирование VDOM Обеспечить покрытие тестами и прозрачность кода файлов…
    */
   var
      assert = coreDebug.checkAssertion,
      FOCUSABLE_ELEMENTS = {
         'a': true,
         'link': true,
         'button': true,
         'input': true,
         'select': true,
         'textarea': true
      },
      CLASS_HIDDEN_FLAG = 1,
      CLASS_DISABLED_FLAG = 2,
      CLASS_DELEGATES_TAB_FLAG = 4,
      CLASS_CREATES_CONTEXT = 8,
      CLASS_TAB_CYCLING = 16,
      CLASS_NAME_TO_FLAG = {
         hidden: CLASS_HIDDEN_FLAG,
         disabled: CLASS_DISABLED_FLAG,
         'delegates-tabfocus': CLASS_DELEGATES_TAB_FLAG,
         'creates-context': CLASS_CREATES_CONTEXT,
         'tab-cycling': CLASS_TAB_CYCLING
      },
      TAB_KEY = 9;

   function createRecursiveVNodeMapper(fn) {
      return function recursiveVNodeMapperFn(tagName, properties, children, key, controlNode) {
         var
            i, childrenRest,
            fnRes = fn(tagName, properties, children, key, controlNode),
            newChildren = fnRes[2];

         i = findIndex(newChildren, function finderRecursiveIndex(child) {
            var newChild = VdomMarkup.mapVNode(recursiveVNodeMapperFn, controlNode, child);
            return child !== newChild;
         });

         if (i !== -1 && i !== undefined) {
            childrenRest = newChildren.slice(i).map(VdomMarkup.mapVNode.bind(self, recursiveVNodeMapperFn, controlNode));
            newChildren = newChildren.slice(0, i).concat(childrenRest);
            fnRes = [fnRes[0], fnRes[1], newChildren, fnRes[3]];
         }

         return fnRes;
      };
   }

   function domToVDom(dom) {
      function getElementStyle(styleText) {
         var
            style, stylePair, styleArr, i, ln;

         if (styleText) {
            styleArr = styleText.split(';');
            ln = styleArr.length;
            if (ln) {
               for (i = 0; i !== ln; i++) {
                  stylePair = styleArr[i].split(':');
                  if (stylePair.length === 2) {
                     if (!style) {
                        style = {};
                     }
                     style[stylePair[0].trim()] = stylePair[1].trim();
                  }
               }
            }
         }

         return style;
      }

      function getElementProps(el) {
         var
            attrs = el.attributes,
            i, ln = attrs.length, attrsObj, style, attr, res, properties, $key = null;

         for(i = 0; i !== ln; i++) {
            attr = attrs[i];
            if (attr.name === 'key') {
               $key = attr.value;
            } else if (attr.name === 'style') {
               style = getElementStyle(attr.value);
            } else {
               if (!attrsObj) {
                  attrsObj = {};
               }
               attrsObj[attr.name] = attr.value;
            }
         }

         properties = {};
         res = {
            properties: properties
         };
         if (attrsObj) {
            properties.attributes = attrsObj;
         }

         if (style) {
            properties.style = style;
         }

         res.key = $key;

         if (el.tagName.toLowerCase() === 'input') {
            if (!properties.attributes) {
               properties.attributes = {
               };
            }

            if (!properties.attributes.hasOwnProperty('value')) {
               properties.attributes.value = undefined;
            }

            properties.value = properties.attributes.value;
         }

         return res;
      }

      function createVNode(domNode) {
         var  type = domNode.nodeType;

         if (type === 1) {
            return createFromElement(domNode);
         }

         if (type == 3) {
            return createFromTextNode(domNode);
         }
      }

      function createFromTextNode(tNode) {
         return new vdom.VText(tNode.nodeValue);
      }

      function createFromElement(el) {
         var
            tagName = el.tagName,
            namespace = el.namespaceURI == 'http://www.w3.org/1999/xhtml'? null : el.namespaceURI,
            props = getElementProps(el),
            children = [],
            i,
            ln = el.childNodes.length;

         for (i = 0; i !== ln; i++) {
            children.push(createVNode(el.childNodes[i]));
         }

         return new vdom.VNode(tagName.toLowerCase(), props.properties, children, props.key, namespace);
      }

      return createVNode(dom);
   }

   function getStyle(element, style) {
      return window.getComputedStyle(element)[style];
   }

   function getElementProps(element) {
      var
         elementPropsClassRe = /\bws-(hidden|disabled)\b/g,
         className = element.getAttribute('class'),
         classes, tabIndex, tabIndexAttr, validTabIndex, flags, enabled, result;

      flags = 0;
      while ((classes = (className && elementPropsClassRe.exec(className)))) {
         flags |= CLASS_NAME_TO_FLAG[classes[1]];
      }

      if (element.getAttribute('ws-creates-context') === 'true') {
         flags |= CLASS_NAME_TO_FLAG['creates-context'];
      }
      if (element.getAttribute('ws-delegates-tabfocus') === 'true') {
         flags |= CLASS_NAME_TO_FLAG['delegates-tabfocus'];
      }
      if (element.getAttribute('ws-tab-cycling') === 'true') {
         flags |= CLASS_NAME_TO_FLAG['tab-cycling'];
      }

      enabled = (flags & (CLASS_HIDDEN_FLAG | CLASS_DISABLED_FLAG)) === 0;
      if (enabled) {
         enabled = getStyle(element, 'display') !== 'none' && getStyle(element, 'visibility') !== 'invisible';
      }
      if (enabled) {
         tabIndexAttr = element.getAttribute('tabindex');
         tabIndex = parseInt(tabIndexAttr, 10);
         validTabIndex = !isNaN(tabIndex);

         result = {
            enabled: true,
            tabStop: (validTabIndex && tabIndex >= 0) || (tabIndexAttr === null && FOCUSABLE_ELEMENTS.hasOwnProperty(element.tagName.toLowerCase())),
            createsContext: (flags & CLASS_CREATES_CONTEXT) !== 0,
            tabIndex: tabIndex || 0, //обязательно хоть 0
            delegateFocusToChildren: (flags & CLASS_DELEGATES_TAB_FLAG) !== 0,
            tabCycling: (flags & CLASS_TAB_CYCLING) !== 0
         };
      } else {
         result = {
            enabled: false,
            tabStop: false,
            createsContext: false,
            tabIndex: 0,
            delegateFocusToChildren: false,
            tabCycling: false
         };
      }

      return result;
   }

   function controlNodesCompoundReduce(prev, next) {
      return !(prev && next);
   }

   function atLeastOneControlReduce(prev, next) {
      return next.control;
   }

   function isControlNodesCompound(controlNodes) {
      return !controlNodes.reduce(controlNodesCompoundReduce, true);
   }

   function atLeasOneControl(controlNodes) {
      return controlNodes.reduce(atLeastOneControlReduce, true);
   }

   function validatePreviousFocusedElements(savedFocusArray) {
      function isInPage(node) {
         return (node === document.body || node.offsetParent === null) ? false : document.body.contains(node);
      }
      var activeArray = [];
      for (var i = 0; i < savedFocusArray.length; i++) {
         if (isInPage(savedFocusArray[i])) {
            activeArray.push(savedFocusArray[i]);
         }
      }
      return activeArray;
   }

   function addControlsToFlatArray(controlNode, array) {
      array.push(controlNode.control);
      if (controlNode.parent) {
         addControlsToFlatArray(controlNode.parent, array);
      }
   }
   function addControlsToFlatArrayOld(control, array) {
      array.push(control);
      if (control.getParent()) {
         addControlsToFlatArrayOld(control.getParent(), array);
      }
   }

   function goUpByControlTree(target, array) {
      if (target) {
         if (constants.compat && target.wsControl) {
            // если встретили старый компонент, нужно собирать его парентов по старому API
            addControlsToFlatArrayOld(target.wsControl, array);
         } else {
            if (target.controlNodes && target.controlNodes.length) {
               for (var i = 0; i < target.controlNodes.length; i++) {
                  addControlsToFlatArray(target.controlNodes[i], array);
               }
            } else {
               goUpByControlTree(target.parentNode, array);
            }
         }
      }
      return array;
   }

   function fireUpEventHandlers(eventName, eventHandlers, event) {
      if (eventHandlers[eventName] && eventHandlers[eventName].count >= 2) {
         eventHandlers[eventName].handler.call(this, event);
      }
   }

   function checkControlNodes(controlNodes, controlNode) {
      if (controlNodes && controlNodes.length > 0) {
         for (var i = 0; i < controlNodes.length; i++) {
            if (controlNode.id === controlNodes[i].id) {
               return true;
            }
         }
      }
      return false;
   }

   var DOMEnvironment = Environment.extend({
      constructor: function(rootDOMNode, controlStateChangedCallback, rootAttrs) {
         assert(rootDOMNode !== document, 'Корневой контрол нельзя монтировать на document');
         this._savedFocusedElements = [];
         this._rootDOMNode = rootDOMNode;
         this._rootVNode = domToVDom(rootDOMNode);
         this._captureEventHandlers = {};
         this._markupNodeDecorator = createRecursiveVNodeMapper(Hooks.setEventHooks(this));
         this.addCaptureFocusBlurHandler('focus', this._handleFocusEvent);
         this.addCaptureFocusBlurHandler('blur', this._handleBlurEvent);
         this.addCaptureFocusBlurHandler('mousedown', this._handleMouseDown);
         this._handleTabKey = this._handleTabKey.bind(this);
         Environment.call(this, controlStateChangedCallback, rootAttrs);
      },
      destroy: function() {
         this.removeTabListener();
         this.removeCaptureEventHandler('focus');
         this.removeCaptureEventHandler('blur');
         this.removeCaptureEventHandler('mousedown');
         this._rootDOMNode = undefined;
         this._rootVNode = undefined;
         this._captureEventHandlers = {};
         this._handleTabKey = undefined;
      }
   });
   var proto = DOMEnvironment.prototype;

   proto.getElementProps = getElementProps;

   function afterMountProcess(controlNode){
      controlNode.control._afterMount && controlNode.control._afterMount();
      controlNode.control._mounted = true;
   }

   /**
    * Рекурсивная функция для обхода уже примененной корневой контрол ноды и её детей, с учетом массива
    * изменных нод вообще и вызова у контролов методов _afterMount и _afterUpdate
    * @param controlNodes
    * @param rebuildChanges
    */
   function callMountMethods(controlNodes, rebuildChanges) {
      controlNodes.forEach(function (controlNode) {
         if (controlNode.childrenNodes && controlNode.childrenNodes.length > 0) {
            callMountMethods(controlNode.childrenNodes, rebuildChanges);
         }
         if (~rebuildChanges.indexOf(controlNode.id)) {
            if (!controlNode.control._mounted) {
               if (controlNode.hasCompound){
                  runDelayed(function(){
                     afterMountProcess(controlNode);
                  });
               } else {
                  afterMountProcess(controlNode);
               }
            } else {
               /**
                * TODO: удалить после синхронизации с контролами
                */
               controlNode.control._afterUpdate && controlNode.control._afterUpdate(controlNode.oldOptions || controlNode.options, controlNode.oldContext);
            }
         }
      });
   }

   proto._handleTabKey = function(event) {
      if (!this._rootDOMNode){
         return;
      }

      var next, res;
      if (event.keyCode === TAB_KEY) {
         next = TabIndex.findWithContexts(this._rootDOMNode, event.target, !!event.shiftKey, getElementProps);

         // храним состояние о нажатой клавише таба до следующего такта. Нужно, чтобы различать приход фокуса по табу или по клику
         this._isTabPressed = true;
         setTimeout(function() {
            this._isTabPressed = false;
         }.bind(this), 0);

         if (next) {
            if (next.wsControl && next.wsControl.setActive) {
               next.wsControl.setActive(true);
            } else {
               next.focus();
            }
            event.preventDefault();
            event.stopImmediatePropagation();
         } else {
            if (this._rootDOMNode.wsControl) {
               res = this._rootDOMNode.wsControl._oldKeyboardHover(event);
            }
            if (res !== false) {
               // !!!!
               // this._lastElement.focus(); чтобы выйти из рута наружу, а не нативно в другой элемент внутри рута
               // тут если с шифтом вероятно нужно прокидывать в firstElement чтобы из него выйти
            } else {
               event.preventDefault();
               event.stopImmediatePropagation();
            }
         }
      }
   };

   proto.addTabListener = function() {
      if (this._rootDOMNode)
         this._rootDOMNode.addEventListener('keydown', this._handleTabKey, false);
   };
   proto.removeTabListener = function() {
      if (this._rootDOMNode)
         this._rootDOMNode.removeEventListener('keydown', this._handleTabKey);
   };

   proto.autofocusChanges = function autoFocusChanges(controlNode) {

      function findNodeForControlNode(rootNode, controlNode) {
         var children;
         if (checkControlNodes(rootNode.controlNodes, controlNode)) {
            return rootNode;
         }
         if (rootNode.children && rootNode.children.length > 0) {
            children = rootNode.children;
            for (var i = 0; i < children.length; i++) {
               if (children[i].controlNodes) {
                  if (checkControlNodes(children[i].controlNodes, controlNode)) {
                     node = children[i];
                     return node;
                  }
                  findNodeForControlNode(children[i], controlNode);
               }
            }
         }
         return node;
      }

      function elementProps(element) {
         var props = controlNode.environment.getElementProps(element);
         props.tabStop = element.getAttribute('ws-autofocus') === 'true';
         return props;
      }

      var node, nodeToFocus, control = controlNode.control, environment = controlNode.environment, focusingElement;

      if (control.__$focusing && environment._rootDOMNode) {
         //this.requestRebuild(controlNode.id);
         node = findNodeForControlNode(environment._rootDOMNode, controlNode);
         if (node) {
            nodeToFocus = TabIndex.findFirstInContext(node, undefined, elementProps);
            if (nodeToFocus) {
               nodeToFocus.focus();
            }
            control.__$focusing = false;
            // control.deactivateFocusing();
         }
      }
      // environment._savedFocusedElements = validatePreviousFocusedElements(environment._savedFocusedElements);
      // focusingElement = environment._savedFocusedElements[environment._savedFocusedElements.length - 1];
      focusingElement = validatePreviousFocusedElements(control._container)[0];
      if (focusingElement) {
         focusingElement.focus();
      }
   };

   proto._handleFocusEvent = function handleFocusEvent(e) {
      if (!this._rootDOMNode){
         return;
      }

      // если relatedTarget нет, это неправильное событие, оно срабатвыает если мы возвращаем фокус извне браузера, такое событие не надо обрабатывать
      if (!e.relatedTarget) {
         return;
      }
      // запускаем обработчик только для правильного DOMEnvironment, в который прилетел фокус
      if (!FocusHelper.closest(e.target, this._rootDOMNode)) {
         return;
      }
      fireUpEventHandlers('focus', this._captureEventHandlers, e);

      this._savedFocusedElements.push(e.target);

      handleFocus(e.target, e.relatedTarget, this._isTabPressed);

      // если нужна обратная совместимость
      if (constants.compat && this._rootDOMNode && this._rootDOMNode.controlNodes) {
         // если компонент уже активен, простреливаем событием onFocusInside
         if (this._rootDOMNode.controlNodes[0].control.isActive()) {
            this._rootDOMNode.controlNodes[0].control._callOnFocusInside();
         } else {
            // если еще не активен, активируем
            require('Lib/Control/AreaAbstract/AreaAbstract.compatible')._storeActiveChildInner.apply(this._rootDOMNode.controlNodes[0].control);
         }
      }
      this._focused = true;
   };
   function handleFocus(target, relatedTarget, isTabPressed) {
      var
         arrayMaker = goUpByControlTree(target, []),
         relatedArrayMaker = goUpByControlTree(relatedTarget, []);

      for (var i=0; i < arrayMaker.length; i++) {
         // если компонент уже активен, значит перевод фокуса осуществлялся внутри компонента и мы стреляем событием onFocusInside
         if (arrayMaker[i]._active) {
            arrayMaker[i]._notify('focusInside', [{
               from: relatedArrayMaker[0],
               to: arrayMaker[0],
               isTabPressed: !!isTabPressed
            }]);
         } else {
            arrayMaker[i]._active = true;
            // если компонент изменяет состояние активности, в этот компонент прилетел фокус, стреляем onFocusIn
            arrayMaker[i]._notify('focusIn', [{
               from: relatedArrayMaker[0],
               to: arrayMaker[0],
               isTabPressed: !!isTabPressed
            }]);
         }
      }
   }

   proto._handleBlurEvent = function handleBlurEvent(e) {
      // если relatedTarget нет, это неправильное событие, оно срабатвыает если мы уводим фокус вне браузера, такое событие не надо обрабатывать
      if (!e.relatedTarget) {
         return;
      }
      // запускаем обработчик только для правильного DOMEnvironment, из которого улетел фокус
      if (!FocusHelper.closest(e.target, this._rootDOMNode)) {
         return;
      }
      fireUpEventHandlers('blur', this._captureEventHandlers, e);

      handleBlur(e.target, e.relatedTarget, this._isTabPressed);
   };
   function handleBlur(target, relatedTarget, isTabPressed) {
      var
         arrayMaker = goUpByControlTree(target, []),
         relatedArrayMaker = goUpByControlTree(relatedTarget, []);

      for (var i = 0; i < arrayMaker.length; i++) {
         if (relatedArrayMaker.length && relatedArrayMaker.indexOf(arrayMaker[i]) !== -1) {
            break;
         }
         arrayMaker[i]._active = false;
         arrayMaker[i]._notify('focusOut', [{
            to: relatedArrayMaker[0],
            from: arrayMaker[0],
            isTabPressed: !!isTabPressed
         }]);
      }
   }

   proto._handleMouseDown = function handleMouseDown(e) {
      if (e.target.closest('[ws-no-focus="true"]')) {
         e.preventDefault();
      }
   };

   function closest(sourceElement, rootElement) {
      while (sourceElement.parentElement) {
         sourceElement = sourceElement.parentElement;
         if (sourceElement === rootElement) return true;
      }
      return false;
   }

   function fireEvent(e) {
      if (!this._rootDOMNode){
         return;
      }

      var relatedTarget = e.relatedTarget || document.body,
         target = e.target;

      var evt = document.createEvent("Events");
      evt.initEvent("keydown", true, true);

      var shifted = false;
      if (target.className === "vdom-focus-in") {
         if(closest(relatedTarget, this._rootDOMNode)) {
            shifted = true;
         }
      }
      if (target.className === "vdom-focus-out") {
         if(!closest(relatedTarget, this._rootDOMNode)) {
            shifted = true;
         }
      }

      evt.view = window;
      evt.altKey = false;
      evt.ctrlKey = false;
      evt.shiftKey = shifted;
      evt.metaKey = false;
      evt.keyCode = 9;

      target.dispatchEvent(evt);
   }
   var FocusHook = function(environment){
      this.fireTab = function (e) {
         fireEvent.call(environment, e);
      }
   };
   FocusHook.prototype.hook = function(node) {
      node.addEventListener('focus', this.fireTab);
   };
   FocusHook.prototype.unhook = function(node) {
      node.removeEventListener('focus', this.fireTab);
   };

   function findFirstVNode(arr) {
      if (!Array.isArray(arr)) {
         return null;
      }
      return arr.find(function(value) {
         return !!value;
      });
   }
   proto.applyNewVNode = function(newVNnode, rebuildChanges) {
      if (!this._rootDOMNode){
         return;
      }
      
      var
         vnode = this.decorateRootNode(newVNnode),
         hasCompound,
         control,
         patch,
         newRootDOMNode;

      var focusInElem = document.createElement('a');
      focusInElem.classList.add('vdom-focus-in');
      var focusOutElem = document.createElement('a');
      focusOutElem.classList.add('vdom-focus-out');
      var firstChild = findFirstVNode(vnode.children);
      // добавляем ноды vdom-focus-in и vdom-focus-out тольео если есть какие-то внутренние ноды
      if (firstChild && firstChild.key !== "vdom-focus-in") {
         var focusInNode = new vdom.VNode('a', {
            attributes: {class: 'vdom-focus-in', tabindex: '1'},
            focusHook: new FocusHook(this)
         }, [], 'vdom-focus-in');
         var focusOutNode = new vdom.VNode('a', {
            attributes: {class: 'vdom-focus-out', tabindex: '0'},
            focusHook: new FocusHook(this)
         }, [], 'vdom-focus-out');
         vnode.children = [].concat(focusInNode, vnode.children, focusOutNode);
      }

      patch = this._rootVNode ? vdom.diff(this._rootVNode, vnode) : null;

      if (patch === null) {
         newRootDOMNode = vdom.createElement(vnode);
         this._rootDOMNode.parentNode.replaceChild(newRootDOMNode, this._rootDOMNode);
      } else {
         newRootDOMNode = vdom.patch(this._rootDOMNode, patch);
      }
      this._rootDOMNode = newRootDOMNode;
      this._rootVNode = vnode;



      hasCompound = isControlNodesCompound(newRootDOMNode.controlNodes);

      if (hasCompound) {
         control = atLeasOneControl(newRootDOMNode.controlNodes);

         control._container = window.$?$(newRootDOMNode):newRootDOMNode;
         runDelayed(function(){
            control.reviveSuperOldControls && control.reviveSuperOldControls();
            callMountMethods(newRootDOMNode.controlNodes, rebuildChanges);//afterMount и afterUpdate
         });
      } else {
         callMountMethods(newRootDOMNode.controlNodes, rebuildChanges);//afterMount и afterUpdate
      }

      return patch;
   };

   proto.decorateFullMarkup = function(vnode, controlNode) {
      return VdomMarkup.mapVNode(Hooks.setControlNodeHook(controlNode), controlNode, vnode);
   };

   proto.getMarkupNodeDecorator = function() {
      return this._markupNodeDecorator;
   };

   proto.getDOMNode = function() {
      return this._rootDOMNode;
   };

   /**
    * Распространение происходит по DOM-нодам вверх по родителям, с использованием массива обработчиков eventProperties,
    * в котором указаны обработчики для каждого контрола, если эти контролы подписаны на событие
    * Таким образом, обходим всю иерархию, даже если на дом-ноде висит несколько контрол-нод.
    * @param eventObject - Объект распространения
    * @param controlNode - Контрол-нода, с элемента которой начинается распространение, если это кастомное событие
    * @param eventPropertiesStartId - индекс обработчика в массиве eventProperties у eventObject.target,
    * с которого нужно начать цепочку вызовов обработчиков события. Необходимо для того, чтобы не вызывать обработчики
    * контролов дочерних контрол-нод.
    * @param args - Аргументы, переданные в _notify
    */
   function vdomEventBubbling(eventObject, controlNode, eventPropertiesStartId, args, native) {
      var eventProperties,
         stopPropagation = false,
         eventPropertyName = "on:" + eventObject.type,
         curDomNode,
         fn,
         res,
         templateArgs,
         finalArgs;

      //Если событием стрельнул window, то распространение начинаем с body
      if(native) {
         curDomNode = (eventObject.target === window) ? window.body : eventObject.target;
      } else {
         curDomNode = controlNode.element;
      }
      curDomNode = native ? curDomNode : controlNode.element;

      //Цикл, в котором поднимаемся по DOM-нодам
      while(!stopPropagation) {
         eventProperties = curDomNode.eventProperties;
         if(eventProperties && eventProperties[eventPropertyName]) {
            //Вызываем обработчики для всех controlNode на этой DOM-ноде
            for (var i = eventPropertiesStartId; i < eventProperties[eventPropertyName].length && !stopPropagation; i++) {
               fn = eventProperties[eventPropertyName][i].fn;
               templateArgs = eventProperties[eventPropertyName][i].args;
               try {
                  if(!args.concat) {
                     throw new Error("Аргументы обработчика события " + eventPropertyName.slice(3) + " должны быть массивом.")
                  }
                  /* Составляем массив аргументов для обаботчика. Первым аргументом будет объект события. Затем будут
                   * аргументы, переданные в обработчик в шаблоне, и последними - аргументы в _notify */
                  finalArgs = templateArgs.concat(args);
                  finalArgs = [eventObject].concat(finalArgs);
                  fn.apply(fn.control, finalArgs);
                  finalArgs = [];
                  /* Проверяем, нужно ли дальше распространять событие по controlNodes */
                  if(!eventObject.propagating()) {
                     var needCallNext = !eventObject.isStopped() &&
                        eventProperties[eventPropertyName][i+1] &&
                        eventProperties[eventPropertyName][i+1].toPartial;
                     /* Если подписались на события из HOC'a, и одновременно подписались на контент хока, то прекращать
                      распространение не нужно, потому то необходимо вызвать оба этих обработчика.
                       Пример sync-tests/vdomEvents/hocContent/hocContent */
                     if (!needCallNext) {
                        stopPropagation = true;
                     }
                  }
               } catch (e) {
                  var pe = new Error(eventObject.type + " event handle error in " + fn.control._moduleName);
                  IoC.resolve('ILogger').error(pe, e.message || e);
               }
               res = eventObject.result;
               if(res && res.addCallback) {
                  res.addCallback(function () {
                     fn.control._forceUpdate();
                  });
               } else {
                  fn.control._forceUpdate();
               }
               res = undefined;
            }
         }
         curDomNode = curDomNode.parentElement;
         if(curDomNode === null || curDomNode === undefined || !eventObject.propagating()) {
            stopPropagation = true;
         }
         if(eventPropertiesStartId !== 0) {
            eventPropertiesStartId = 0;
         }
      }
   }

   /**
    * Находит индекс обработчика в массиве eventProperties у controlNode.element, с которого надо начинать распространение
    * @param controlNode
    * @returns {number}
    */
   function getEventPropertiesStartId(controlNode, eventName) {
      var eventProperties = controlNode.element.eventProperties,
         controlNodes = controlNode.element.controlNodes,
         handlerIdx = 0,
         nodeIdx = 0,
         eventPropertyName = "on:" + eventName,
         beforeTarget = true, //true, пока не дошли до @controlNode в массиве controlNodes
         curEventPropertyControl;
      if(eventProperties && eventProperties[eventPropertyName]) {

         curEventPropertyControl = eventProperties[eventPropertyName][0].fn.control;
         while (beforeTarget && nodeIdx < controlNodes.length) {
            if(controlNodes[nodeIdx].control === curEventPropertyControl) {
               handlerIdx++;
               if(handlerIdx < eventProperties[eventPropertyName].length) {
                  curEventPropertyControl = eventProperties[eventPropertyName][handlerIdx].fn.control;
               }
            }
            if(controlNodes[nodeIdx].control === controlNode.control) {
               beforeTarget = false;
            }
            nodeIdx++;
         }
      }
      return handlerIdx;
   }

   /**
    * Создается объект кастомного события с указанными в notify параметрами и вызывается функция его распространения
    * Возвращает результат выполнения последнего обработчика
    * @param controlNode
    * @param args
    */
   proto.startEvent = function(controlNode, args){
      var
         eventName = args[0].toLowerCase(),
         handlerArgs = args[1] || [],
         eventDescription = args[2],
         eventConfig = {},
         eventObject;
      eventConfig._bubbling = (eventDescription && eventDescription.bubbling !== undefined) ? eventDescription.bubbling : false;
      eventConfig.type = eventName;
      eventConfig.target = controlNode.element;
      if (!eventConfig.target){
         IoC.resolve('ILogger').error("Event "+ eventName +" has emited before mounting to DOM");
         return;
      }
      eventObject = new SyntheticEvent(null, eventConfig);
      vdomEventBubbling(eventObject, controlNode, getEventPropertiesStartId(controlNode, eventName), handlerArgs, false);
      return eventObject.result;
   };

   /**
    * Стартовая точка для распространения и обработки DOM-событий
    * @param event - объект события
    */
   function captureEventHandler(event) {
      if (!this._rootDOMNode) {
         return;
      }

      if (event.currentTarget === window && event.type === "scroll" ||
         event.currentTarget === window && event.type === "resize"){
         /*Всегда выполняем обработчики, потому что тут нет фазы захвата*/
      } else if (event.eventPhase !== 1) {
         return;
      }
      /* Если попали сюда, значит событие focusout/focusin - нативное.
      Остановим его распространение. Позже мы стрельнем аналогичным событием */
      if (event.type === 'focusout' || event.type === 'focusin') {
         return;
      }
      var synthEvent = new SyntheticEvent(event);
      vdomEventBubbling(synthEvent, null, 0, [], true);
   }

   proto.addCaptureEventHandler = function(eventName, element) {
      var handlers = this._captureEventHandlers;

      //В ие в слое совместимости дикая асинхронность, что приводит к тому, что подписка начинает вызываться для компонентов,
      //которые уже удалены из ДОМА механизмами CompoundControl
      if (this._rootDOMNode.parentNode) {

         /**Спец события BODY
          * обрабатываются спец образом
          * приходится подписываться прямо на BODY*/
         if (element.tagName && element.tagName.toLowerCase()==="body" && (eventName==="scroll" || eventName==="resize")){
            if (handlers[eventName + "body"] === undefined) {
               handlers[eventName + "body"] = {
                  handler: captureEventHandler.bind(this),
                  count: 1
               };
               window.addEventListener(eventName, handlers[eventName + "body"].handler);
            } else {
               handlers[eventName + "body"].count++;
            }
         } else {
            if (handlers[eventName] === undefined) {
               handlers[eventName] = {
                  handler: captureEventHandler.bind(this),
                  count: 1
               };
               this._rootDOMNode.parentNode.addEventListener(eventName, handlers[eventName].handler, true);
            } else {
               handlers[eventName].count++;
            }
         }
      }
   };

   proto.addCaptureFocusBlurHandler = function addCaptureFocusBlurHandler(eventName, method) {
      var handlers = this._captureEventHandlers;
      if (handlers[eventName] === undefined) {
         //В ие в слое совместимости дикая асинхронность, что приводит к тому, что подписка начинает вызываться для компонентов,
         //которые уже удалены из ДОМА механизмами CompoundControl
         if (this._rootDOMNode.parentNode) {
            handlers[eventName] = {
               handler: captureEventHandler.bind(this),
               count: 1
            };
            this._rootDOMNode.parentNode.addEventListener(eventName, method.bind(this), true);
         }
      } else {
         handlers[eventName].count++;
      }
   };

   proto.removeCaptureEventHandler = function(eventName) {
      var handlers = this._captureEventHandlers;
      handlers[eventName].count--;

      if (handlers[eventName].count === 0) {
         if (this._rootDOMNode.parentNode) {
            // так случилось, что если закрываем окно с виртуальной кнопкой, связи успевают разорваться быстрее чем все задестроиться, и parentNode тут уже нет
            this._rootDOMNode.parentNode.removeEventListener(eventName, handlers[eventName].handler, true);
         }
         delete handlers[eventName];
      }
   };

   proto.haveCapturePhase = function() {
      return !constants.browser.isIE || constants.browser.IEVersion > 8;
   };

   return DOMEnvironment;
});