define('Core/vdom/Synchronizer/resources/DirtyChecking',
[
   'require',
   'Core/helpers/Object/isEqual',
   'Core/constants',
   'Core/vdom/Utils/Monad',
   'Core/vdom/Utils/Functional',
   'View/Runner/expressions/subscriber',
   'Core/vdom/Synchronizer/resources/VdomMarkup',
   'View/Runner/common',
   'Core/vdom/vdomutils',
   'View/Runner/expressions/compatible',
   'Core/helpers/Function/shallowClone',
   'View/Runner/expressions/contextResolver',
   'Core/helpers/Function/runDelayed',
   'Core/Serializer',
   'Core/Deferred',
   'Core/ParallelDeferred'
],
function(
         require,
         isEqualObject,
         isJs,
         Monad,
         functionalHelpers,
         eventSubscriber,
         VdomMarkup,
         utils,
         vdomutils,
         compatibleHelpers,
         shallowClone,
         contextResolver,
         runDelayed,
         Serializer
) {
   'use strict';

   var DirtyCheckingCompatible;
   if (isJs.compat){
      require(['Core/vdom/Synchronizer/resources/DirtyCheckingCompatible'], function(_dcc){
         DirtyCheckingCompatible = _dcc;
      });
   }


   function subscribeToEvent(node) {
      if (node.control && node.control._getInternalOption('parent')) {
         var events = eventSubscriber.getEventsListFromOptions(node.options);
         eventSubscriber.applyEvents(node.control, node.control._getInternalOption('parent'), events);
      }
   }

   /**
    * Добавляет родителя во внутренние опции компонента, если он отсутствует
    * @param internalOptions
    * @param userOptions
    * @param parentNode
    */
   function fixInternalParentOptions(internalOptions, userOptions, parentNode) {
      // У compound-контрола parent может уже лежать в user-опциях, берем его оттуда, если нет нашей parentNode
      internalOptions.parent = internalOptions.parent || (parentNode && parentNode.control) || userOptions.parent  || null;
      internalOptions.logicParent = internalOptions.logicParent || (parentNode && parentNode.control && parentNode.control.logicParent) || userOptions.logicParent || null;
   }

   var DirtyKind = {
         NONE: 0,
         DIRTY: 1,
         CHILD_DIRTY: 2
      };

      function getModuleDefaultCtor(mod) {
         return typeof mod === 'function' ? mod : mod['constructor'];
      }

   var
      ARR_EMPTY = [],
      INVALID_CONTEXT = {},
      mapM                       = Monad.mapM,
      createWriterMonad          = Monad.createWriterMonad;

   var RebuildResultMonoid = {
      plus: function(memo1, memo2) {
         return {
            createdNodes: memo1.createdNodes.concat(memo2.createdNodes),
            destroyedNodes: memo1.destroyedNodes.concat(memo2.destroyedNodes),
            updatedNodes: memo1.updatedNodes.concat(memo2.updatedNodes),
            updatedChangedNodes: memo1.updatedChangedNodes.concat(memo2.updatedChangedNodes),
            updatedUnchangedNodes: memo1.updatedUnchangedNodes.concat(memo2.updatedUnchangedNodes),
            selfDirtyNodes: memo1.selfDirtyNodes.concat(memo2.selfDirtyNodes)
         };
      },

      zero: {
         createdNodes: [],
         destroyedNodes: [],
         updatedNodes: [],
         updatedUnchangedNodes: [],
         updatedChangedNodes: [],
         selfDirtyNodes: []
      }
   };

   var RebuildResultWriter = createWriterMonad(RebuildResultMonoid);

   function shallowMerge(dest, src) {
      var i;
      for (i in src) {
         if (src.hasOwnProperty(i)) {
            dest[i] = src[i];
         }
      }
      return dest;
   }

   function getChangedOptions(newOptions, oldOptions, ignoreDirtyChecking) {
      var
         i, def,
         changed = false,
         changedOptions = {},
         hasOld, hasNew;

      for (i in oldOptions) {
         /**
          * Игнорируем переменные dirtyChecking для compoundControl
          * Эти переменные могут появиться только когда внутри VDom контрола
          * есть CompoundControl внутри которого в контентных опциях
          * есть контролы
          */
         if (ignoreDirtyChecking && i && i.indexOf('__dirtyCheckingVars_')>-1) {
            continue;
         }
         hasOld = oldOptions.hasOwnProperty(i);
         if (hasOld) {
            hasNew = newOptions.hasOwnProperty(i);
            if (hasNew) {
               if (
                     (typeof newOptions[i] === 'function'
                        && typeof oldOptions[i] === 'function') ||
                     ((typeof newOptions[i] === 'number' && typeof oldOptions[i] === 'number') &&
                     (isNaN(newOptions[i]) && isNaN(oldOptions[i])))
               ){
                  continue;
               }
               /**
                * Для всех контролов контрола сравним объекты по ссылке.
                * CompoundControl сам мониторит изменения внутри объекта
                * и/или массива
                */
               if (newOptions[i]===oldOptions[i]) {
                  if (typeof newOptions[i] === 'object' && newOptions[i] && newOptions[i].changed){
                     newOptions[i].changed = false;
                     changed = true;
                     changedOptions[i] = newOptions[i];
                  } else {
                     continue;
                  }
               } else if (Array.isArray(newOptions[i]) && newOptions[i]) {
                  changed = true;
                  changedOptions[i] = newOptions[i];
               } else {
                  if (!isEqualObject(newOptions[i], oldOptions[i])) {
                     changed = true;
                     changedOptions[i] = newOptions[i];
                  }
               }
            }
         }
      }
      return changed ? changedOptions : null;
   }

   function flatten(arr) {
      arr = arr || [];
      return arr.reduce(function(flat, newChunk) {
         return flat.concat(Array.isArray(newChunk) ? flatten(newChunk) : newChunk);
      }, []);
   }

   function getControlNodeParams(control, controlClass, environment) {
      var
         getDecoratorFuncs = functionalHelpers.getHierarchyFunctions(controlClass, '_getMarkupDecorators'),
         decorators = flatten(getDecoratorFuncs.map(function (fn) {
            return fn.call(control);
         })),
         composedDecorator = functionalHelpers.composeWithResultApply.apply(undefined, decorators.concat(environment.getMarkupNodeDecorator())).bind(control);

      return {
         markupDecorator: composedDecorator,
         defaultOptions: {} //нет больше понятия опция по умолчанию
      };
   }

      function createNode(controlClass_, options, key, environment, parentNode, serialized, vnode) {
      var
         controlCnstr = getModuleDefaultCtor(controlClass_), // получаем конструктор из модуля
         compound = (vnode && vnode.compound),
         serializedState = (serialized && serialized.state) || { vdomCORE: true }, // сериализованное состояние компонента
         userOptions = options.user, // прикладные опции
         internalOptions = options.internal || {}, // служебные опции
         result;

      fixInternalParentOptions(internalOptions, userOptions, parentNode);

      if (!key) {
         /*У каждой ноды должен быть ключ
         * for строит внутренние ноды относительно этого ключа
         * */
         key = '_';
      }

      if (compound) {
         // Создаем виртуальную ноду для compound контрола
         if (!DirtyCheckingCompatible) {
            DirtyCheckingCompatible = require('Core/vdom/Synchronizer/resources/DirtyCheckingCompatible');
         }
         result = DirtyCheckingCompatible.createCompoundControlNode(controlClass_, controlCnstr, userOptions, internalOptions, key, parentNode, vnode);
      } else {
         // Создаем виртуальную ноду для не-compound контрола
         var
            invisible = (vnode && vnode.invisible),
            // подмешиваем сериализованное состояние к прикладным опциям
            optionsWithState = serializedState ? shallowMerge(userOptions, serializedState) : userOptions,
            control, params;

         if (typeof controlClass_ === 'function') {
            // создаем инстанс компонента
            var instCompat = compatibleHelpers.createInstanceCompatible(controlCnstr, optionsWithState, internalOptions);
            control = instCompat.instance;
            optionsWithState = instCompat.resolvedOptions;
         } else {
            // инстанс уже есть, работаем с его опциями
            optionsWithState = compatibleHelpers.combineOptionsIfCompatible(controlCnstr.prototype, optionsWithState, internalOptions);
            control = controlClass_;
            if (control._setInternalOptions) {
               control._options.doNotSetParent = true;
               control._setInternalOptions(internalOptions || {});
            }
         }

         params = getControlNodeParams(control, controlCnstr, environment);
         result = {
            attributes: options.attributes,
            control: control,
            errors: serialized && serialized.errors,
            controlClass: controlCnstr,
            options: optionsWithState,
            internalOptions: internalOptions,
            id: control.getHash && control.getHash(),
            parent: parentNode,
            key: key,
            defaultOptions: params && params.defaultOptions,
            markup: invisible ? vdomutils.textNode('') : undefined,
            fullMarkup: undefined,
            childrenNodes: ARR_EMPTY,
            markupDecorator: params && params.markupDecorator,
            serializedChildren: serialized && serialized.childrenNodes,
            hasCompound: false,
            receivedState: undefined,
            invisible: invisible,
            context: (vnode && vnode.context) || {}
         };

         environment.setupControlNode(result);
      }

      return result;
   }

   function copyNode(controlNode) {
      return shallowClone(controlNode);
   }


   function rebuildNodeWriter(environment, dirties, node, force, isRoot) {
      if (node.receivedState && node.receivedState.callback) {
         return node.receivedState.addCallback(function rebuildNodeWriterCbk(state) {
            node.receivedState = state;
            node.control = utils.plainMerge(node.control, state);
            return rebuildNode(environment, dirties, node, force, isRoot);
         });
      } else {
         return rebuildNode(environment, dirties, node, force, isRoot);
      }
   }

   function destroyReqursive(childControlNode, environment){
      try {
         environment.setRebuildIgnoreId(childControlNode.id);
         if (childControlNode.compound) {
            var oldOptions = childControlNode.options,
               instanceCtr = oldOptions.__vdomOptions && oldOptions.__vdomOptions.controlNode.instance;
            /**
             * CompoundControl может быть разрушен в любой момент времени
             * при этом он удалится из списка детей
             * Просто не разрушаем если его уже нет
             */
            if (instanceCtr) {
               runDelayed(function() {
                  instanceCtr.destroy();
               });
            }
         } else {
            for( var i=0; i<childControlNode.childrenNodes.length; i++) {
               destroyReqursive(childControlNode.childrenNodes[i], environment);
            }
            //Пометим контрол, как разрушаемый из DirtyChecking
            //слой совместимости попытается удалить контрол из дома,
            //этого не должно произойти, иначе синхронизатор упадет
            childControlNode.control.__$destroyFromDirtyChecking = true;
            childControlNode.control.destroy();
         }
      } finally {
         environment.setRebuildIgnoreId(childControlNode.id);
      }
   }

   function rebuildNode(environment, dirties, node, force, isRoot) {
      var
         id = node.id,
         def,
         dirty = dirties[id] || DirtyKind.NONE,
         isDirty = dirty !== DirtyKind.NONE || force,
         isSelfDirty = !!(dirty & DirtyKind.DIRTY) || force,
         oldMarkup = node.markup,
         shouldUpdate,
         Slr = new Serializer(),
         newNode, parentNode, diff, createdNodes, updatedNodes, updatedUnchangedNodes, updatedChangedNodes, selfDirtyNodes,
         destroyedNodes, result, childrenRebuild, childrenNodes, createdStartIdx, changedNodes, parentNodeContext;





      if (isDirty) {
         newNode = node.markup ? copyNode(node) : node;
         parentNodeContext = node.context;
         if (!newNode.compound) {
            /**
             *
             Вызываем _beforeUpdate и _shouldUpdate для корневой контрол ноды.
             */
            if (isSelfDirty && newNode.control._mounted) {
               newNode.control._beforeUpdate(newNode.options, parentNodeContext);
               shouldUpdate = newNode.control._shouldUpdate(newNode.options, parentNodeContext);
               isSelfDirty = isSelfDirty&&shouldUpdate;
            }

            if (isSelfDirty) {
               parentNode = newNode;

               newNode.control.saveFullContext(contextResolver.wrapContext(newNode.control, newNode.context));

               newNode.markup = VdomMarkup.getDecoratedMarkup(newNode, isRoot);

               diff = VdomMarkup.getMarkupDiff(oldMarkup, newNode.markup);

               destroyedNodes = diff.destroy.map(function rebuildDestroyNodes(vnode) {
                  var controlNodeIdx = vnode.controlNodeIdx,
                  childControlNode = parentNode.childrenNodes[controlNodeIdx];
                  destroyReqursive(childControlNode, environment);
                  return childControlNode;
               });


               changedNodes = new Array(diff.create.length + diff.update.length);

               createdStartIdx = diff.update.length;
               createdNodes = diff.create.map(function rebuildCreateNodes(vnode, idx) {
                  var
                     nodeIdx = createdStartIdx + idx,
                     serializedChildren = parentNode.serializedChildren,
                     serialized = serializedChildren && serializedChildren[nodeIdx],
                     options,
                     carrier,
                     controlNode;
                  if (!vnode.compound) {
                     options = vnode.controlProperties;
                     controlNode = createNode(vnode.controlClass, { user: options, internal: vnode.controlInternalProperties, attributes:  vnode.controlAttributes}, vnode.key, environment, parentNode, serialized, vnode);
                     if(!controlNode.control._mounted) {
                        carrier = vdomutils.getReceivedState(controlNode, vnode, Slr);
                        if (carrier) {
                           controlNode.receivedState = carrier;
                        }
                        if (controlNode.control.saveOptions) {
                           controlNode.control.saveOptions(controlNode.options, controlNode);
                        } else {
                           /**
                            * Поддержка для совместимости версий контролов
                            */
                           controlNode.control._options = controlNode.options;
                           controlNode.control._setInternalOptions(vnode.controlInternalProperties || {});
                           controlNode.control._container = $(controlNode.element);
                        }
                     }
                     subscribeToEvent(controlNode);//TODO Кусок слоя совместимости https://online.sbis.ru/opendoc.html?guid=95e5b595-f9ea-45a2-9a4d-97a714d384af
                  } else {
                     controlNode = createNode(vnode.controlClass,
                                              {user: shallowMerge(vnode.controlProperties, vnode.controlInternalProperties)},
                                              vnode.key, environment, parentNode, serialized, vnode);
                  }
                  vnode.controlNodeIdx = nodeIdx;
                  changedNodes[vnode.controlNodeIdx] = true;
                  return controlNode;
               });

               updatedNodes = diff.update.map(function rebuildUpdateNodes(diffPair, idx) {
                  var
                     newVNode = diffPair.newNode,
                     controlNodeIdx = diffPair.oldNode.controlNodeIdx,
                     childControlNode = parentNode.childrenNodes[controlNodeIdx],
                     childControl = childControlNode.control,
                     shouldUpdate = true,
                     newOptions = newVNode.compound ? compatibleHelpers.createCombinedOptions(newVNode.controlProperties, newVNode.controlInternalProperties) : newVNode.controlProperties,
                     oldOptions = childControlNode.options,
                     newChildNodeContext = newVNode.context,
                     oldChildNodeContext = childControlNode.context,
                     changedOptions = getChangedOptions(newOptions, oldOptions, newVNode.compound),
                     changedContext = getChangedOptions(newChildNodeContext, oldChildNodeContext, false),
                     oldAttrs = childControlNode.controlAttributes || childControlNode.attributes,
                     changedAttrs = getChangedOptions(newVNode.controlAttributes, oldAttrs, newVNode.compound),
                     changedInternalOptions;
                  if (newVNode.compound) {
                     newOptions.__vdomOptions = oldOptions.__vdomOptions;
                     if (changedOptions) {
                        /**
                         * Сложнейшая логика синхронизации CompoundControls
                         * Суть в том, что разные контролы при получении нового значения опции
                         * реагируют по разному.
                         * Отсюда получается, что мы устанавливаем в контрол одни данные, а по факту
                         * устанавливаются другие и в том месте, куда мы что-то присвоили уже лежит не то, что мы туда присваивали
                         * Поэтому здесь двойная проверка, которая не позволяет синхронно установить
                         * какие-либо данные в контрол дважды.
                         */
                        var instanceCtr = oldOptions.__vdomOptions && oldOptions.__vdomOptions.controlNode.instance;
                        if (instanceCtr && !instanceCtr.__$$blockSetProperties) {
                           if (!instanceCtr.___$$realOptions) {
                              instanceCtr.___$$realOptions = instanceCtr._options;
                           }
                           var realCh = getChangedOptions(newOptions, instanceCtr.___$$realOptions || {}, newVNode.compound);
                           if (realCh) {
                              instanceCtr.__$$blockSetProperties = true;
                              changedOptions = DirtyCheckingCompatible.clearNotChangedOptions(changedOptions);
                              instanceCtr.setProperties(changedOptions);
                              instanceCtr.___$$realOptions = changedOptions;
                              runDelayed(function(){
                                 instanceCtr.__$$blockSetProperties = false;
                              });
                           }
                        }
                     }
                  } else {
                     // для не-compound контролов делаем проверку изменения служебных опций
                     changedInternalOptions = getChangedOptions(newVNode.controlInternalProperties, childControlNode.internalOptions);
                     //Атрибуты тоже учавствуют в DirtyChecking
                     if (changedOptions || changedInternalOptions || changedAttrs || changedContext) {
                        try {
                           environment.setRebuildIgnoreId(childControlNode.id);
                           childControl._beforeUpdate && childControl._beforeUpdate(newOptions, newChildNodeContext);
                           childControl._options = newOptions;
                           shouldUpdate = (childControl._shouldUpdate ? childControl._shouldUpdate(newOptions, newChildNodeContext) : true) || changedInternalOptions;
                           childControl._setInternalOptions(changedInternalOptions || {});
                           childControlNode.oldOptions = oldOptions;//TODO Для afterUpdate подумать, как еще можно передать
                           childControlNode.oldContext = oldChildNodeContext;//TODO Для afterUpdate подумать, как еще можно передать
                           childControlNode.attributes = newVNode.controlAttributes;
                           childControl._contextObj = contextResolver.resolveContext(childControlNode.controlClass, newChildNodeContext);
                           childControl.saveFullContext(contextResolver.wrapContext(childControl, childControl._contextObj));
                        } finally {
                           /**
                            * TODO: удалить после синхронизации с контролами
                            */
                           var shouldUp = childControl._shouldUpdate ? childControl._shouldUpdate(newOptions, newChildNodeContext) || changedInternalOptions : true;
                           childControl._setInternalOptions(changedInternalOptions || {});
                           if (shouldUp) {
                              environment.setRebuildIgnoreId(null);
                           }
                        }
                     }
                  }
                  changedNodes[idx] = (!!changedOptions || !!changedInternalOptions || !!changedContext || !!changedAttrs) && shouldUpdate;
                  newVNode.controlNodeIdx = idx;
                  childControlNode.options = newOptions;
                  childControlNode.context = newChildNodeContext;
                  if (!newVNode.compound) {
                     childControlNode.internalOptions = newVNode.controlInternalProperties;
                  }

                  return childControlNode;
               });

                  childrenNodes = updatedNodes.concat(createdNodes);

                  updatedUnchangedNodes = ARR_EMPTY;
                  updatedChangedNodes = ARR_EMPTY;
                  selfDirtyNodes = ARR_EMPTY;

                  updatedNodes.forEach(function(node, idx) {
                     if (changedNodes[idx]) {
                        if (updatedChangedNodes === ARR_EMPTY) {
                           updatedChangedNodes = [];
                        }
                        updatedChangedNodes.push(node);
                     } else {
                        if (updatedUnchangedNodes === ARR_EMPTY) {
                           updatedUnchangedNodes = [];
                        }
                        updatedUnchangedNodes.push(node);
                     }
                  });

                  // храним еще незаапдейченные ноды, которые сами были грязными
                  if (childrenNodes.indexOf(newNode) === -1) {
                     if (selfDirtyNodes === ARR_EMPTY) {
                        selfDirtyNodes = [];
                     }
                     selfDirtyNodes.push(newNode);
                  }
               } else {
                  childrenNodes = node.childrenNodes;

                  createdNodes   = ARR_EMPTY;
                  updatedNodes   = ARR_EMPTY;
                  updatedUnchangedNodes = ARR_EMPTY;
                  updatedChangedNodes = ARR_EMPTY;
                  destroyedNodes = ARR_EMPTY;
                  selfDirtyNodes = ARR_EMPTY;
               }
            } else {
               childrenNodes = node.childrenNodes;
               createdNodes   = ARR_EMPTY;
               updatedNodes   = ARR_EMPTY;
               updatedUnchangedNodes = ARR_EMPTY;
               updatedChangedNodes = ARR_EMPTY;
               destroyedNodes = ARR_EMPTY;
               selfDirtyNodes = ARR_EMPTY;
               if (newNode.compound) { // Слой совместимости
                  //Нужно установить hasCompound только тогда когда нода действительно со
                  //старым контролом. В эту точку также попадаем и при создании контролллера
                  //компонента без шаблона
                  newNode.parent.hasCompound = true;
               }
            }

            childrenRebuild = mapM(childrenNodes, function(node, idx) {
               return rebuildNodeWriter(environment, dirties, node, changedNodes && changedNodes[idx]);
            }, RebuildResultWriter);

            if (childrenRebuild.callback) {
               return childrenRebuild.addCallback(function (childrenRebuild) {
                  newNode.childrenNodes = childrenRebuild.value;

                  newNode.fullMarkup = environment.decorateFullMarkup(VdomMarkup.getFullMarkup(newNode.childrenNodes, newNode.markup), newNode);

                  return {
                     value: newNode,
                     memo: RebuildResultMonoid.plus(childrenRebuild.memo, {
                        createdNodes: createdNodes,
                        updatedNodes: updatedNodes,
                        destroyedNodes: destroyedNodes,
                        updatedChangedNodes: updatedChangedNodes,
                        updatedUnchangedNodes: updatedUnchangedNodes,
                        selfDirtyNodes: selfDirtyNodes
                     })
                  };
               });
            } else {
               newNode.childrenNodes = childrenRebuild.value;

               newNode.fullMarkup = environment.decorateFullMarkup(VdomMarkup.getFullMarkup(newNode.childrenNodes, newNode.markup), newNode);

               result = {
                  value: newNode,
                  memo: RebuildResultMonoid.plus(childrenRebuild.memo, {
                     createdNodes: createdNodes,
                     updatedNodes: updatedNodes,
                     destroyedNodes: destroyedNodes,
                     updatedChangedNodes: updatedChangedNodes,
                     updatedUnchangedNodes: updatedUnchangedNodes,
                     selfDirtyNodes: selfDirtyNodes
                  })
               };

            }


         } else {
            result = {
               value: node,
               memo: RebuildResultMonoid.zero
            };
         }

      return result;
   }

   return {
      DirtyKind: DirtyKind,
      rebuildNode: rebuildNode,
      RebuildResultWriter: RebuildResultWriter,
      createNode: createNode
   }
});