define('Core/vdom/Synchronizer/resources/VdomMarkup',
[
   'Core/core-debug',
   'Core/vdom/vdomutils',
   'Core/vdom/Utils/Monad',
   'Core/vdom/Utils/Functional'
],
function(coreDebug, vdomutils, Monad) {
   'use strict';

   function getVNodeChidlren(vnode) {
      return vdomutils.isVNodeType(vnode) ? vnode.children : [];
   }
   function mapVNode(fn, controlNode, vnode) {
      if (vdomutils.isVNodeType(vnode)) {
         var
            newNodeArgs = fn(vnode.tagName, vnode.properties, vnode.children, vnode.key, controlNode),
            sameNode = vnode.tagName === newNodeArgs[0] && vnode.properties === newNodeArgs[1] &&
               vnode.children === newNodeArgs[2] && vnode.key === newNodeArgs[3];

         return sameNode ? vnode : vdomutils.htmlNode.apply(vdomutils, newNodeArgs);
      } else {
         return vnode;
      }
   }
   function mapVNodeChildren(recursive, vnode, mapFn, filterFn) {
      function join(vnode) {
         var mapped = filterFn(vnode) ? [mapFn(vnode)] : [];
         return recursive ? mapped.concat(Monad.ListMonad.join(getVNodeChidlren(vnode), join)) : mapped;
      }

      return Monad.ListMonad.join(getVNodeChidlren(vnode), join);
   }

   function identity(v) {
      return v;
   }
   function collectChildControlVNodes(vnode) {
      return mapVNodeChildren(true, vnode, identity, vdomutils.isControlVNodeType);
   }

   function getMarkupDiff(oldNode, newNode) {
      var
         result = {
            create: [],
            destroy: [],
            update: []
         },
         childrenResult, oldChildren, newChildren, childLn, i, oldChild, newChild;

      function reorder(oldChildren, newChildren) {
         function haveKeys(children) {
            for (var i = 0, length = children.length; i !== length; i++) {
               if (children[i].key !== undefined) {
                  return true;
               }
            }
            return false;
         }

         function keyIndex(children) {
            var
               keys = {},
               free = [],
               length = children.length;

            for (var i = 0; i < length; i++) {
               var child = children[i], key = child.key;

               if (key !== undefined) {
                  keys[key] = i;
               } else {
                  free.push(i);
               }
            }

            return { keys: keys, free: free };
         }

         var
            result, oldKeys, newKeys, newFree, newFreeLn, oldKI, newKI, i, freeIndex, lastFreeIndex,
            oldLength, newLength, oldChild;

         if (haveKeys(oldChildren) && haveKeys(newChildren)) {
            oldKI = keyIndex(oldChildren);
            newKI = keyIndex(newChildren);
            oldKeys = oldKI.keys;
            oldLength = oldChildren.length;
            newKeys = newKI.keys;
            newLength = newChildren.length;
            newFree = newKI.free;
            newFreeLn = newFree.length;

            freeIndex = 0;
            result = new Array(length);

            for (i = 0; i !== oldLength; i++) {
               oldChild = oldChildren[i];
               if (oldChild.key !== undefined) {
                  if (newKeys.hasOwnProperty(oldChild.key)) {
                     result[i] = newChildren[newKeys[oldChild.key]];
                  } else {
                     result[i] = null;
                  }
               } else if (freeIndex !== newFreeLn) {
                  result[i] = newChildren[newFree[freeIndex]];
                  freeIndex++;
               } else {
                  result[i] = null;
               }
            }

            lastFreeIndex = freeIndex !== newFreeLn ? newFree[freeIndex] : newLength;
            for (i = 0; i !== newLength; i++) {
               newChild = newChildren[i];
               if (newChild.key !== undefined) {
                  if (!oldKeys.hasOwnProperty(newChild.key)) {
                     result.push(newChild);
                  }
               } else if (i >= lastFreeIndex) {
                  result.push(newChildren[i]);
               }
            }
         } else {
            result = newChildren;
         }
         return result;
      }

      function isEqualNode(n1, n2) {
         var
            isControl1 = n1 && vdomutils.isControlVNodeType(n1),
            isControl2 = n2 && vdomutils.isControlVNodeType(n2),
            result = isControl1 === isControl2;

         if (result) {
            if (isControl1) {
               result = n1.controlClass === n2.controlClass &&
                  n1.key === n2.key;
            } else {
               result = n1.type === n2.type;
               if (result && vdomutils.isVNodeType(n1))
               {
                  result = n1.tagName   === n2.tagName &&
                     n1.namespace === n2.namespace &&
                     n1.key       === n2.key;
               }
            }
         }

         return result;
      }


      function concatResults(result, partName, part) {
         result[partName] = result[partName].concat(part);
      }

      if (oldNode !== newNode) {
         coreDebug.checkAssertion(!!newNode, 'newNode !== null');

         if (isEqualNode(oldNode, newNode)) {
            if (vdomutils.isControlVNodeType(newNode)) {
               result.update.push({
                  oldNode: oldNode,
                  newNode: newNode
               });
            } else {
               oldChildren = getVNodeChidlren(oldNode);
               newChildren = reorder(oldChildren, getVNodeChidlren(newNode));

               childLn = Math.max(oldChildren.length, newChildren.length);
               for (i = 0; i !== childLn; i++) {
                  oldChild = oldChildren[i];
                  newChild = newChildren[i];
                  if (oldChild && newChild) {
                     childrenResult = getMarkupDiff(oldChild, newChild);
                     concatResults(result, 'create', childrenResult.create);
                     concatResults(result, 'destroy', childrenResult.destroy);
                     concatResults(result, 'update', childrenResult.update);
                  } else if (oldChild) {
                     if (vdomutils.isControlVNodeType(oldChild)) {
                        concatResults(result, 'destroy', [oldChild]);
                     }
                     else {
                        concatResults(result, 'destroy', collectChildControlVNodes(oldChild));
                     }
                  } else {
                     if (vdomutils.isControlVNodeType(newChild)) {
                        concatResults(result, 'create', [newChild]);
                     }
                     else {
                        concatResults(result, 'create', collectChildControlVNodes(newChild));
                     }
                  }
               }
            }
         } else {
            if (vdomutils.isControlVNodeType(newNode)) {
               result.create.push(newNode);
            } else {
               concatResults(result, 'create', collectChildControlVNodes(newNode));
            }

            if (oldNode) {
               if (vdomutils.isControlVNodeType(oldNode)) {
                  result.destroy.push(oldNode);
               } else {
                  concatResults(result, 'destroy', collectChildControlVNodes(oldNode));
               }
            }
         }
      }

      return result;
   }

   function getFullMarkup(controlNodes, vnode) {
      var result, i, children, ln, childMarkup, newChildren, childrenAfter;

      if (vdomutils.isControlVNodeType(vnode)) {
         result = controlNodes[vnode.controlNodeIdx].fullMarkup;
      } else if (!vdomutils.isVNodeType(vnode) || vnode.children.length === 0) {
         result = vnode;
      } else {
         i = 0;
         children = getVNodeChidlren(vnode);
         ln = children.length;

         while (i !== ln) {
            childMarkup = getFullMarkup(controlNodes, children[i]);
            if (childMarkup !== children[i]) {
               break;
            }

            i++;
         }

         if (i === ln) {
            result = vnode;
         } else {
            childrenAfter = children.slice(i + 1).map(getFullMarkup.bind(undefined, controlNodes));
            newChildren = children.slice(0, i).concat([childMarkup]).concat(childrenAfter);

            result = vdomutils.htmlNode(vnode.tagName, vnode.properties, newChildren, vnode.key);
         }
      }

      return result;
   }

   function createErrVNode(err) {
      return vdomutils.htmlNode('span', {}, [vdomutils.textNode(err)]);
   }

   function getDecoratedMarkup(controlNode, isRoot) {
      //Теперь передададим еще и атрибуты, которые нам дали сверху для построения верстки
      //там они будут мержиться
      var
         markupRes = controlNode.control._getMarkup(controlNode.key, isRoot, controlNode.attributes), result;

      if (vdomutils.isVNodeType(markupRes)) {
         result = mapVNode(controlNode.markupDecorator, controlNode, markupRes);
      } else if (vdomutils.isControlVNodeType(markupRes)) {
         result = mapVNode(controlNode.markupDecorator, controlNode, markupRes);
      } else {
         result = markupRes;
      }

      if (!vdomutils.isControlVNodeType(result) && !vdomutils.isVNodeType(result)) {
         result = createErrVNode('control._getMarkup должен отдавать dom-узел');
      }

      return result;
   }

   return {
      mapVNode: mapVNode,
      getVNodeChidlren: getVNodeChidlren,
      getMarkupDiff: getMarkupDiff,
      getFullMarkup: getFullMarkup,
      getDecoratedMarkup: getDecoratedMarkup
   };
});