define('Core/vdom/vdomutils', [
   'View/Runner/Vdom/third-party/dom',
   'View/Runner/expressions/contextResolver'], function(
      vdom,
      contextResolver) {

   'use strict';

   var receivedName = 'inline',
       configName = 'cfg-';

   /**
    * Для того чтобы всегда брать верхний компонент из конфига
    * @param configId
    * @returns {*}
    */
   function findTopConfig(configId) {
      return configId.replace(configName, '').split(',')[0];
   }


   /**
    * Для того что бы звать сам метод, если он есть или с готовым состоянием
    * @param stateVar
    * @param control
    * @param vnode
    * @param serializer
    * @returns {*}
    */
   function getStateReadyOrCall(stateVar, control, vnode, serializer) {
      var data = window[stateVar];
      if (window[stateVar]){
         window[stateVar] = undefined;
      }

      var res = data ?
         control._beforeMount(
            vnode.controlProperties,
            vnode.context,
            JSON.parse(data, serializer.deserialize)
         ) : control._beforeMount(vnode.controlProperties, vnode.context);

      control._contextObj = contextResolver.resolveContext(vnode.controlClass, vnode.context);
      control.saveFullContext(contextResolver.wrapContext(control, vnode.context || {}));
      return res;
   }



   var VDomUtils = /** @lends $ws.proto.VDomUtils.prototype */{

      htmlNode: function(tagName, props, children, key) {
         return new vdom.VNode(tagName, props, children, key);
      },

      textNode: function(text) {
         return new vdom.VText(text);
      },


      controlNode: function(controlClass, controlProperties, key) {
         return {
            controlClass: controlClass,
            controlProperties: controlProperties,
            key: key,
            controlNodeIdx: -1
         };
      },

      isVNodeType: function(vnode) {
         return vnode && typeof vnode === 'object' && vnode.type === 'VirtualNode';
      },

      isTextNodeType: function(vnode) {
         return vnode && typeof vnode === 'object' && vnode.type === 'VirtualText';
      },

      isControlVNodeType: function(vnode) {
         return vnode && typeof vnode === 'object' && 'controlClass' in vnode;
      },

      /**
       * Получаем state из сгенерированного script
       * @param controlNode
       * @param vnode
       * @param Slr
       * @returns {*}
       */
      getReceivedState: function getReceivedState(controlNode, vnode, serializer) {
         var control = controlNode.control,
             rstate = control.getDataId && control.getDataId() ?  receivedName + findTopConfig(control.getDataId()) : '';
         if (control._beforeMount) {
            return getStateReadyOrCall(rstate, control, vnode, serializer);
         }
      }
   };

   return VDomUtils;
});
