define('Deprecated/helpers/fast-control-helpers', [
   'require',
   'Core/IoC',
   'Core/constants',
   'Core/core-functions',
   'Core/Deferred',
   'Core/ParallelDeferred',
   'Core/helpers/String/escapeTagsFromStr',
   'Core/Context',
   'Core/moduleStubs',
   'Core/WindowManager',
   'Core/core-ready'
], function(require, IoC, constants, cFunc, Deferred, ParallelDeferred, escapeTagsFromStr, Context, moduleStubs, WindowManager, cReady) {
    /**
     * кеш данных по выводимой ошибке
     * @type {Array<alertData>}
     * @type {String} alertData.text
     * @type {String} alertData.detail
     * @type {Core/Deferred} alertData.def
     */
   var alertStack = [];
   /**
    * @class Deprecated/helpers/fast-control-helpers
    * @public
    * @deprecated
    */
   var fastHelpers = /** @lends Deprecated/helpers/fast-control-helpers.prototype */{
      /**
       * Производит создание календаря на основе контрола {@link Deprecated/Controls/FieldDate/FieldDate}.
       * @param {String|jQuery} id Идентификатор элемента, либо jQuery элемент, в который вставляем календарь.
       * @param {Function} func Устанавливает функцию для изменения отображения дней в календаре.
       * @param {Date} [firstDayToShow] Дата дня, на котором нужно сфокусировать календарь. Вызов showDatePicker("datepicker", dayFormat);
       * @returns {Core/Deferred} Нужно для того, чтобы знать, что календарик уже нарисовался.
       * @example
       * Пример функции func:
       * <pre>
       * // <div id="test"></div> Должен существовать блок с идентификатором 'test'
       * helpers.showDatePicker('test', function dayFormat(date) {
       *    // Если отображаемый день месяца меньше 15
       *    if (date.getDate() < 15) {
       *       // [может ли быть выбрано, 'css-класс для добавления к ячейке дня', "текст всплывающей подсказки"]
       *       return [true, 'ui-state-highlight', "tooltip"];
       *    }
       *    return [true, ''];
       * });
       * </pre>
       * @see question
       * @see alert
       * @see message
       * @see newContextMenu
       * @see showHTMLForPrint
       * @see toggleLocalIndicator
       * @see showFloatArea
       * @see showHoverFloatArea
       * @see showDialogSelector
       * @see showFloatAreaSelector
       * @see openErrorsReportDialog
       */
      showDatePicker: function (id, func, firstDayToShow) {
         var def = new Deferred();
         cReady.addCallback(function () {
            moduleStubs.require(["cdn!jquery-ui/1.8.9/jquery-ui-1.8.5.2.custom.min.js",
                         "cdn!css!jquery-ui/1.8.9/jquery-ui-1.8.5.custom.css",
                         "Deprecated/Controls/FieldDate/resources/ext/jquery.ui.datepicker-local"])
               .addCallback(function () {
                  var el = (id instanceof $) ? id : $('#' + id);
                  el.datepicker({
                     beforeShowDay: func,
                     defaultDate: firstDayToShow
                  });
                  def.callback();
               });
         });
         return def;
      },
      /**
       * Вызывает информационное окно типа "Общий вопрос".
       * @remark
       * Пользователю будет доступно 2 варианта ответа: "Да" и "Нет".
       * Создаёт модальное окно.
       * @param {String} text Текст вопроса.
       * @param {Object} [cfg] Параметры информационного окна. В параметр передают объект с опциями класса {@link Deprecated/Controls/DialogConfirm/DialogConfirm}.
       * @param {Boolean} [cfg.invertDefaultButton = false] Переопределяет кнопку (Да/Нет), которая будет выделена оранжевым цветом по умолчанию.
       * В значении true - оранжевая кнопка "Да".
       * @returns {Core/Deferred} Возвращает ответ, который выбрал пользователь.
       * </ul>
       *    <li>true при нажатии пользователем кнопки "Да".</li>
       *    <li>false при нажатии пользователем кнопки "Нет".</li>
       *    <li>undefined при нажатии пользователем на клавишу "Esc".</li>
       *    <li>cancel при клике пользователя вне области окна.</li>
       * </ul>
       * @example
       * 1. Чтобы вызвать окно типа "Общий вопрос", выполните следующий код:
       * <pre>
       *    helpers.question('Сохранить изменения?');
       * </pre>
       * 2. Чтобы обрабатывать ответ, который выбрал пользователь, используют следующий код:
       * <pre>
       *    // в result - результат выбора пользователя
       *    helpers.question('Сохранить изменения?').addCallback(function(result){
       *       // обработка
       *    });
       * </pre>
       * 3. Чтобы установить, какая из кнопок "Да"/"Нет" по умолчанию будет оранжевой, устанавливают дополнительную конфигурацию:
       * <pre>
       *    // Для этого вторым параметром передаём объект с invertDefaultButton: false/ true:
       *    // Этим свойством мы инвертируем кнопку, выбранную по умолчанию:
       *    helpers.question('Сохранить изменения?', {invertDefaultButton: true});
       * </pre>
       * 4. При обработке результата производите строгое сравнение с результатом ответа.
       * <pre>
       *    helpers.question('Сохранить изменения?').addCallback(function(result){
       *       if (result === true) { … }
       *    });
       * </pre>
       * @see showDatePicker
       * @see alert
       * @see message
       * @see newContextMenu
       * @see showHTMLForPrint
       * @see toggleLocalIndicator
       * @see showFloatArea
       * @see showHoverFloatArea
       * @see showDialogSelector
       * @see showFloatAreaSelector
       * @see openErrorsReportDialog
       */
      question: function (text, cfg, opener) {
         var res = new Deferred(),
            escPressed = false,
            windowManager = WindowManager;

         moduleStubs.require(['Deprecated/Controls/DialogConfirm/DialogConfirm']).addCallback(function (result) {
            new result[0](cFunc.merge({
               message: text,
               opener: opener || (windowManager && windowManager.getActiveWindow()),
               handlers: {
                  onKeyPressed: function (event, result) {
                     if (result.keyCode === constants.key.esc) {
                        escPressed = true;
                     }
                  },
                  onConfirm: function (event, result) {
                     //Он может зайти сюда ещё раз из какого-то обработчика в res.callback
                     // (например, из opener-a - FloatArea, которая будет закрываться сама и закрывать свои дочерние окна,
                     // и это окно в том числа),
                     // тогда он может попытаться стрельнуть ещё раз, и будет ошибка
                     if (!res.isReady()) {
                        if (escPressed) {
                           res.callback(undefined);
                        }
                        else {
                           res.callback(result);
                        }
                     }
                  }
               }
            }, cfg || {}));
         }).addErrback(function (err) {
            res.errback(err);
         });
         return res;
      },
      /**
       * Показывает сообщение.
       * @param {String} text Текст.
       * @param {Object} cfg Параметры, которые могут переопределять/дополнять стандартную конфигурацию окна.
       * @param {String} type Тип: 'info', 'error' или 'complete'.
       * @param {Object} opener контрол, который вызвал функцию (нужен в случае вызова из floatArea)
       * @returns {Core/Deferred} Возвращает Deferred, который сработает при закрытии окна.
       * @private
       */
      _messageBox: function (text, cfg, type, opener) {
         var res = new Deferred(),
            windowManager = WindowManager;
         res.addErrback(function (e) {
            return e;
         });
         moduleStubs.requireModule('Deprecated/Controls/DialogAlert/DialogAlert').addCallback(function (result) {
            var config = cFunc.merge({
               message: text,
               type: type,
               opener: opener || (windowManager && windowManager.getActiveWindow()),
               handlers: {
                  onAfterClose: function () {
                     res.callback();
                  }
               }
            }, cfg || {});
            return new result[0](config);
         }).addErrback(function (err) {
            IoC.resolve('ILogger').log('$ws.helpers._messageBox', err.message);

            var result = text.replace(/<br>/g, '\n');

            alert(escapeTagsFromStr(result, '\\w+'));//Если не удалось создать диалог, показываем простой браузерный алёрт
            res.errback(err);
         });
         return res;
      },
      /**
       * Вызывает информационное окно типа "Ошибка".
       * @param {String|Error} text Текст сообщения.
       * @param {Object} [cfg] Параметры информационного окна. В параметр передают объект с опциями класса {@link Deprecated/Controls/DialogAlert/DialogAlert}.
       * @param {Boolean} [cfg.checkAlreadyProcessed=false] Если передан true и вместо текста передан объект ошибки, проверяет флаг processed (указывает на то, что ошибка уже обработана и не надо о ней более извещать) и если флаг выставлен - не показывает сообщение.
       * @param {Object} [opener] Контрол, который вызвал функцию. Нужен в случае вызова из floatArea.
       * @returns {Core/Deferred} Срабатывает при закрытии окна.
       * @example
       * 1. Окно "Ошибка" (предупреждение, alert) можно вызвать так:
       * <pre>
       *    helpers.alert('Сообщение об ошибке!');
       * </pre>
       * 2. Окно возвращает Core/Deferred, который сработает при закрытии окна. Но как подписаться на результат? Делаем следующее:
       * <pre>
       * helpers.alert('Соообщение об ошибке!').addBoth(function(){
       *    //внутрь функции всегда передаётся одинаковое значение, вне зависимости от действий пользователя: нажал кнопку "ОК" или "Ecs"
       *    //тело функции, которая должна быть выполнена
       * });
       * </pre>
       * 3. Если требуется, например, изменить состояние флага после нажатия кнопки "ОК" в окне "Ошибка"(Alert), то:
       * <pre>
       * helpers.alert('Сообщение об ошибке').addBoth(function(){
       *    //устанавливаем флаг сброшенным
       *    $ws.single.ControlStorage.getByName('ИмяФлага').setValue(false);
       * });
       * </pre>
       * 4. Переопределение конфигурации кнопки:
       * <pre>
       *    helpers.message('Это информационное окно "Сообщение"!', {
       *        buttons : [
       *            {
       *                width : "100%",
       *                caption : "Отмена",
       *                handlers: {
       *                    onActivated  : function(){
       *                         this.getParent().close();
       *                    }
       *               }
       *            }
       *        ]
       *    });
       * </pre>
       * @see showDatePicker
       * @see question
       * @see message
       * @see newContextMenu
       * @see showHTMLForPrint
       * @see toggleLocalIndicator
       * @see showFloatArea
       * @see showHoverFloatArea
       * @see showDialogSelector
       * @see showFloatAreaSelector
       * @see openErrorsReportDialog
       */
      alert: function (text, cfg, opener) {
         var message = text, detail;
         cfg = cfg || {};
         if (text instanceof Error) {
            //Обработка отмены запроса пользователем
            if (text.httpError === 0) {
               return new Deferred().callback();
            }
            if (cfg.checkAlreadyProcessed) {
               if (text.processed) {
                  return new Deferred().callback();
               } else {
                  text.processed = true;
               }
            }
            message = (text.message || '').replace(/\n/mg, '<br />');
            detail = text.details;
         }
         detail = cfg.detail || detail;

         if (!message) {
            return new Deferred().callback();
         }
         /*
          * Пользователь не должен закрывать кучу одинаковых ошибок, которые наплывают друг на друга
          * При этом все опции пользователю не важны, показатель для него только текст и заголовок ошибки
          * поэтому кешируем заголовок и текст сообщения, а также deferred открытия диалога
          */
         var cashAlert = alertStack.filter(function (d) {
             return (d.text === message) && (d.detail === detail)
         })[0];
         if (cashAlert) {
            return cashAlert.def.createDependent();
         }
         cashAlert = {
            detail: detail,
            text: message,
            def: fastHelpers._messageBox(message, cfg, 'error', opener).addBoth(function(res) {
               // почистим стек открытых сообщений с ошибками от себя
               alertStack = alertStack.filter(function (d) {
                  return !((d.text === message) && (d.detail === detail))
               });
               return res;
            })
         };
         alertStack.push(cashAlert);
         return cashAlert.def;
      },
      /**
       * Вызывает информационное окно типа "Сообщение".
       * @param {String} text  Текст сообщения.
       * @param {Object} [cfg] Параметры информационного окна. В параметр передают объект с опциями класса {@link Deprecated/Controls/DialogAlert/DialogAlert}.
       * @returns {Core/Deferred} Сработает при закрытии окна.
       * @example
       * 1. Вызвать информационное окно "Сообщение" можно так:
       * <pre>
       *    helpers.message('Это информационное окно "Сообщение"!');
       * </pre>
       *
       * 2. Это окно возвращает Core/Deferred. Как это использовать? Можно, например, выполнять функции только закрытия окна:
       * <pre>
       *    helpers.message('Это информационное окно "Сообщение"!').addBoth(function(){
       *        //тело функции
       *    });
       * </pre>
       * 3. Переопределение конфигурации кнопки:
       * <pre>
       *    helpers.message('Это информационное окно "Сообщение"!', {
       *        buttons : [
       *            {
       *                width : "100%",
       *                caption : "Отмена",
       *                handlers: {
       *                    onActivated  : function(){
       *                         this.getParent().close();
       *                    }
       *               }
       *            }
       *        ]
       *    });
       * </pre>
       * @see showDatePicker
       * @see question
       * @see alert
       * @see newContextMenu
       * @see showHTMLForPrint
       * @see toggleLocalIndicator
       * @see showFloatArea
       * @see showHoverFloatArea
       * @see showDialogSelector
       * @see showFloatAreaSelector
       * @see openErrorsReportDialog
       */
      message: function (text, cfg, opener) {
         return fastHelpers._messageBox(text, cfg, 'info', opener);
      },
      /**
       * Создает контекстное меню по упрощенным параметрам - список пунктов и м.б. функция активации.
       * @param {Array} items Пункты меню.
       * @param {Function} [onActivate] Функция, которая будет выполняться при выборе пункта меню.
       * @returns {Core/Deferred} Возвращает созданным контекстным меню.
       * @see showDatePicker
       * @see question
       * @see alert
       * @see message
       * @see showHTMLForPrint
       * @see toggleLocalIndicator
       * @see showFloatArea
       * @see showHoverFloatArea
       * @see showDialogSelector
       * @see showFloatAreaSelector
       * @see openErrorsReportDialog
       */
      newContextMenu: function (items, onActivate) {
         var config = [];
         for (var k = 0, l = items.length; k < l; k++) {
            config[k] = {
               caption: items[k],
               id: items[k],
               handlers: {}
            };
            if (typeof(onActivate) == 'function')
               config[k].handlers['onActivated'] = onActivate;
         }

          var res = new Deferred();
          require(['Deprecated/core-attach'], function (attach) {
              res.dependOn(attach.attachInstance('Deprecated/Controls/Menu/Menu', {
                  data: config
              }));
          });

          return res;
      },
      /**
       *
       */
      toggleIndicator: (function () {
         var indicInstance, indicDeffered;
         function indicatorVisible(toggle, delay) {
            if (!!toggle) {
               // Если передали Deferred
               if (toggle instanceof Deferred) {
                  // 1) Если он еще не готов.
                  // 2) toggle может быть в цепочке дефередов, такое может случиться, если какой-то callback
                  // в качестве результата вернул деферед, проверим, что цепочка не была запущена
                  if (!toggle.isReady() && !toggle.isCallbacksLocked()) {
                     // Покажем
                     indicInstance.setDelay(delay);
                     indicInstance.show();
                     // Подпишемся на завершение с любым статусом ...
                     toggle.addBoth(function (res) {
                        // и скроем индикатор
                        indicInstance.hide();
                        return res;
                     });
                  }
                  else { // Скроем индикатор, если Deferred сразу готов
                     indicInstance.hide();
                  }
               }
               else {
                  indicInstance.setDelay(delay);
                  indicInstance.show();
               }
            } else {
               indicInstance.hide();
            }
         }
         return function (toggle, delay) {
            indicDeffered = indicDeffered || moduleStubs.requireModule('Deprecated/Controls/LoadingIndicator/LoadingIndicator').addCallback(function (LoadingIndicator) {
                  indicInstance = new LoadingIndicator[0]({
                     message: rk('Пожалуйста, подождите…'),
                     delay: delay
                  });
                  return indicInstance;
               });
            if (!indicDeffered.isReady()) {
               //Идет попытка загрузить LoadingIndicator, но он еще не готов
               indicDeffered.addCallback(function () {
                  indicatorVisible(toggle, delay);
               })
            } else {
               // Был определен LoadingIndicator, обрабатываем индикатор
               indicatorVisible(toggle, delay);
            }
         }
      })(),

      /**
       * Вызываем селектор, выполняет функцию для каждой выбранной записи,
       * если функция возвращает Deferred, то они объединяются в ParallelDeferred;
       * и ParallelDeferred дождется их завершения с любым результатом, в этом случае будет показан индикатор.
       * @param cfg {object} - конфиг для селектора
       * @param func {Function} - функция вида func( record, номер_среди_выбранных, массив_выбранных_записей ).
       * @param ctx контекст, чтобы можно было обращаться из фильтра браузера к полям контекста.
       * @param moduleName {string} - подключаемый модуль
       * @returns {Core/Deferred} стрельнет, когда будет произведен выбор, и все результаты будут обработаны.
       * @private
       */
      _showSelector: function (cfg, func, ctx, moduleName) {
         var
            result = new Deferred(),
            finishContextDfr = new Deferred(),
            linkedCtx = Context.createContext(finishContextDfr, null, ctx),
            needIndic = false;

         result.addErrback(function (e) {
            return e;
         });

          require(['Deprecated/core-attach'], function (attach) {
              attach.attachInstance(moduleName, cFunc.merge({
                  linkedContext: linkedCtx,
                  handlers: {
                      onChange: function (event, records) {
                          var self = this;
                          // если выбор только листьев, то для папки может прийти [ null ], проверка на это
                          if (records && records[0]) {
                              // чтобы дождаться всех
                              var ready = new ParallelDeferred({stopOnFirstError: false});
                              records.forEach(function () {
                                  var funcRes = func.apply(self, arguments),
                                      isDeferred = funcRes instanceof Deferred;

                                  // может не нужна ансинхронная обработка, тогда индикатор не покажем
                                  needIndic |= isDeferred;
                                  if (isDeferred) {
                                      ready.push(funcRes);
                                  }
                              });
                              if (needIndic) {
                                  fastHelpers.toggleIndicator(true);
                              }
                              this.close(ready.done().getResult());
                          }
                      },
                      onAfterClose: function (event, res) {
                          //  если просто закрыли диалог, то res будет false вроде
                          if (res && res instanceof Deferred)
                              result.dependOn(res);
                          else
                              result.callback(res);
                      },
                      onDestroy: function() {
                          finishContextDfr.callback();
                      }
                  }
              }, cfg)).addErrback(function (err) {
                  result.errback(err);
                  return err;
              });
          });

         result.addBoth(function (res) {
            if (needIndic)
               fastHelpers.toggleIndicator(false);
            return res;
         });
         return result;
      },

      /**
       * Вызываем DialogSelector, выполняет функцию для каждой выбранной записи, если функция возвращает Deferred, то они объединяются в ParallelDeferred;
       * и ParallelDeferred дождется их завершения с любым результатом, в этом случае будет показан индикатор.
       * @param {Object} cfg Конфиг класса {@link Deprecated/Controls/DialogSelector/DialogSelector}, не стоит использовать handlers.
       * @param {Function} func Функция вида func(record, номер_среди_выбранных, массив_выбранных_записей).
       * @param {Core/Context} [ctx] Контекст, чтобы можно было обращаться из фильтра браузера к полям контекста.
       * @return {Core/Deferred} стрельнет, когда будет произведен выбор, и все результаты будут обработаны.
       * @see showDatePicker
       * @see question
       * @see alert
       * @see message
       * @see newContextMenu
       * @see showHTMLForPrint
       * @see toggleLocalIndicator
       * @see showFloatArea
       * @see showHoverFloatArea
       * @see showFloatAreaSelector
       * @see openErrorsReportDialog
       */
      showDialogSelector: function (cfg, func, ctx) {
         return fastHelpers._showSelector(cfg, func, ctx, 'Deprecated/Controls/DialogSelector/DialogSelector');
      },
      /**
       * Вызываем flowAreaSelector, выполняет функцию для каждой выбранной записи, если функция возвращает Deferred, то они объединяются в ParallelDeferred;
       * и ParallelDeferred дождется их завершения с любым результатом, в этом случае будет показан индикатор.
       * @param {Object} cfg Конфиг класса {@link Deprecated/Controls/DialogSelector/FloatAreaSelector}.
       * @param {Function} func Функция вида func(record, номер_среди_выбранных, массив_выбранных_записей).
       * @param {Core/Context} [ctx] Контекст, чтобы можно было обращаться из фильтра браузера к полям контекста.
       * @return {Core/Deferred} стрельнет, когда будет произведен выбор, и все результаты будут обработаны.
       * @see showDatePicker
       * @see question
       * @see alert
       * @see message
       * @see newContextMenu
       * @see showHTMLForPrint
       * @see toggleLocalIndicator
       * @see showFloatArea
       * @see showHoverFloatArea
       * @see showDialogSelector
       * @see openErrorsReportDialog
       */
      showFloatAreaSelector: function (cfg, func, ctx) {
         return fastHelpers._showSelector(cfg, func, ctx, 'Deprecated/Controls/DialogSelector/FloatAreaSelector');
      }
   };

   return fastHelpers;
});