define('Deprecated/helpers/transport-helpers', [
   'Core/detection',
   'Core/core-functions',
   'Core/IoC',
   'Core/moduleStubs',
   'Transport/prepareGetRPCInvocationURL',
   'Core/i18n',
   'Transport/RPCJSON'
], function (detection, cFunc, IoC, moduleStubs, prepareGetRPCInvocationURL, i18n, RPCJSON) {
    /**
     *
     * @class Deprecated/helpers/transport-helpers
     * @public
     * @deprecated
     */
   var transHelpers = /** @lends Deprecated/helpers/transport-helpers.prototype */{
      /**
       * Подготавливает пакет для отправки запроса JSON RPC.
       * @param {String} method Название метода.
       * @param {Object} params Параметры метода.
       * @param {String|Number} [id=1] Идентификатор.
       * @returns {Object} Объект с двумя полями:
       * <ul>
       *    <li>reqBody — тело запроса в виде строки,</li>
       *    <li>reqHeaders — объект с необходимыми заголовками.</li>
       * </ul>
       * @see prepareSorting
       * @see prepareGetRPCInvocationURL
       * @see fetchFile
       * @see newRecordSet
       */
      jsonRpcPreparePacket: RPCJSON.jsonRpcPreparePacket,
      /**
       * Подготавливает объект сортировки для передачи на бизнес-логику.
       * @remark
       * Создаёт объект вида { d: [], s: [] }.
       * @param {Object} filter Текущий фильтр табличного представления.
       * @returns {*} Возвращает объект или null.
       * @see jsonRpcPreparePacket
       * @see prepareGetRPCInvocationURL
       * @see fetchFile
       * @see newRecordSet
       */
      prepareSorting: function (filter) {
         if (!filter || filter['sorting'] === undefined) {
            return null;
         }
         var result = {
            s: [
               {'n': 'n', 't': 'Строка'},
               {'n': 'o', 't': 'Логическое'},
               {'n': 'l', 't': 'Логическое'}
            ],
            d: []
         };
         for (var len = filter['sorting'].length, i = 0; i < len; ++i) {
            result.d.push([filter['sorting'][i][0], filter['sorting'][i][1], !filter['sorting'][i][1]]);
         }
         return result;
      },
      /**
       * Формирует GET-запрос к бизнес-логике.
       * @remark
       * Подготавливает адрес для вызова метода БЛ через GET запрос.
       * Например, чтобы при клике на ссылку начиналась загрузка какого-либо файла, расположенного в базе данных бизнес-логики.
       * @param {String} object Объект бизнес-логики.
       * @param {String} method Метод бизнес-логики.
       * @param {Object} args Аргументы  вызова бизнес-логики. Можно передать пустой объект {}.
       * @param {Core/Context} [context=$ws.single.GlobalContext] Контекст для параметров (см. пример).
       * @param {String} [serviceUrl=$ws._const.defaultServiceUrl] Адрес сервиса бизнес-логики.
       * @param {String} [customId] Пользовательский id для GET запроса(работает только если IE > 9).
       * @returns {String} Возвращает строку GET запроса.
       * @example
       * <pre>
       *     // параметр 'Версия' берётся из поля 'version' глобального контекста
       *     helpers.prepareGetRPCInvocationURL('ВерсияВнешнегоДокумента', 'ПолучитьФайл', {'ИдО': 123, 'Версия' : {fieldName : 'version'}})
       * </pre>
       * @see jsonRpcPreparePacket
       * @see prepareSorting
       * @see fetchFile
       * @see newRecordSet
       */
      //MOVE_TO КРАЙНОВ
      prepareGetRPCInvocationURL: prepareGetRPCInvocationURL,
      /**
       * Выполняет GET-запрос на получение файла с помощью метода бизнес-логики.
       * @param {String} objectName Имя объекта БЛ. Например "Контакт".
       * @param {String} methodName Имя метода БЛ. Например "Список".
       * @param {Object} params JSON-объект параметров запроса.
       * @param {String} serviceUrl Путь до сервиса, на котором надо выполнить метод.
       * @return {Core/Deferred} Объект deferred с результатом запроса.
       * @see jsonRpcPreparePacket
       * @see prepareSorting
       * @see prepareGetRPCInvocationURL
       * @see newRecordSet
       */
      fetchFile: function (objectName, methodName, params, serviceUrl) {
         return IoC.resolve('ITransport', {
            method: 'GET',
            dataType: 'text',
            url: transHelpers.prepareGetRPCInvocationURL(objectName, methodName, params, null, serviceUrl)
         }).execute();
      },
      /**
       * Метод для создания запросов.
       * @param {String} object Имя таблицы, у которой будет вызван метод.
       * @param {String} [method] Имя метода БЛ.
       * @param {Object} [filter] Параметры фильтрации.
       * @param {String} [reader=ReaderUnifiedSBIS] Используемый Reader.
       * @param {Boolean} [autoQuery=true] Производить ли начальное заполнение RecordSet'а.
       * @param {String} [url] Адрес сервиса.
       * @param {String} [hierarchyField=''] Поле иерархии.
       * @param {Object} [readerParams = {}] Дополнительные параметры Reader, например, readMethodName.
       * @returns {Core/Deferred} В случае успеха подписант на результа deferred получит RecodSet {@link Deprecated/RecordSet}.
       * В противном случае происходит ошибка.
       * @example
       * <pre>
       *    fieldLink.subscribe('onBeforeShowRecord', function(event, record){
       *       if (record.isBranch()) {
       *          //не показываем диалог для выбранного подразделения.
       *          event.setResult(false);
       *       } else if (record.get('Сотрудник.Лицо1') !== $ws.single.GlobalContext.getValue('currentOrganization')) {
       *          //если сотрудник в текущей организации, то её запись уже есть и отдадим на показ её.
       *          event.setResult(window.currentOrgRecord);
       *       } else {
       *          //иначе вычитаем.
       *          var waitOrg = new Deferred();
       *          event.setResult(wait);
       *          helpers.newRecordSet('Организация', 'Список', {}, undefined, false).addCallback(function(rs){
       *             waitOrg.dependOn(rs.readRecord(record.get('Сотрудник.Лицо1')));
       *          }).addErrback(function(error){
       *             waitOrg.errback(error);
       *          });
       *       }
       *    });
       * </pre>
       * @see jsonRpcPreparePacket
       * @see prepareSorting
       * @see prepareGetRPCInvocationURL
       * @see fetchFile
       */
      newRecordSet: function (object, method, filter, reader, autoQuery, url, hierarchyField, readerParams) {
         var
            attachDfr,
            canceled = false,
            internalRs = null;

         autoQuery = ((autoQuery === undefined) ? true : !!autoQuery);

         hierarchyField = (hierarchyField === undefined ? '' : hierarchyField);

         readerParams = cFunc.merge({
            otherUrl: url,
            dbScheme: '',
            linkedObject: object,
            queryName: method
         }, (!!readerParams && readerParams.constructor === Object) ? readerParams : {});

         attachDfr = moduleStubs.require('Deprecated/RecordSet')
            .addCallback(function (mods) {
               return new mods[0]({
                  hierarchyField: hierarchyField,
                  firstRequest: false,
                  filterParams: filter,
                  readerType: reader || 'ReaderUnifiedSBIS',
                  readerParams: readerParams
               })
            }).addCallback(function (rs) {
               var res;

               if (autoQuery && !!method && !canceled) {
                  internalRs = rs;
                  res = rs.reload().createDependent().addCallback(function () {
                       return rs;
                   });
               } else {
                  res = rs;
               }

               return res;
            });

         return attachDfr.createDependent().addErrback(function (err) {
            var rs;
            if (err.canceled) {
               canceled = true;
               if (internalRs) {
                  rs = internalRs;
                  internalRs = null;
                  rs.abort();
               }
            }
            return err;
         });
      }
   };

   return transHelpers;
});