(function () {
   var
      //Global variables root
      global = (function() {
         return this || (0, eval)('this');
      })(),

      //RequireJS root context
      requirejs = global.requirejs,

      //Check if on server side
      isServerScript = typeof window === 'undefined',

      isSplittedCore = function(wsPath) {
         return wsPath.substr(-7) === 'WS.Core' || wsPath.substr(-8) === 'WS.Core/';
      },

      isDebugMode = function() {
         return global.document && global.document.cookie && global.document.cookie.indexOf('s3debug=true') > -1;
      },

      //Join path parts together
      pathJoin = (function() {
         var removeLeadingSlash = function(path) {
               if (path) {
                  var head = path.charAt(0);
                  if (head === '/' || head === '\\') {
                     path = path.substr(1);
                  }
               }
               return path;
            },
            removeTrailingSlash = function (path) {
               if (path) {
                  var tail = path.substr(path.length - 1);
                  if (tail === '/' || tail === '\\') {
                     path = path.substr(0, path.length - 1);
                  }
               }
               return path;
            };

         return function() {
            var count = arguments.length,
               path = [],
               item,
               i;
            for (i = 0; i < count; i++) {
               item = arguments[i];
               if (i > 0) {
                  item = removeLeadingSlash(item);
               }
               if (i < count - 1) {
                  item = removeTrailingSlash(item);
               }
               path.push(item);
            }

            return path.join('/');
         };
      })(),

      addPaths = function(baseUrl, paths, add) {
         var pathPrefix;
         for (pathPrefix in add) {
            if (add.hasOwnProperty(pathPrefix)) {
               paths[pathPrefix] = pathJoin(baseUrl, add[pathPrefix]);
            }
         }
      },

      //Create startup config for RequireJS
      createConfig = function (baseUrl, wsPath, resourcesPath, options) {
         global.wsConfig = global.wsConfig || {};

         var //Detect splitted-core mode
            splittedCore = isSplittedCore(wsPath),
            //Build config
            cfg = {
               baseUrl: baseUrl,
               paths: {
                  'WS': wsPath,
                  'tslib': pathJoin(wsPath, 'lib/Ext/tslib'),
                  'Lib': pathJoin(wsPath, 'lib'),
                  'Ext': pathJoin(wsPath, 'lib/Ext'),
                  'Core': pathJoin(wsPath, 'core'),
                  'unit': '/~ws/test/unit',
                  'Demo': '/~ws/Demo',
                  'Deprecated': splittedCore ? pathJoin(resourcesPath, 'WS.Deprecated') : pathJoin(wsPath, 'deprecated'),
                  'Helpers': pathJoin(wsPath, 'core/helpers'),
                  'Transport': pathJoin(wsPath, 'transport'),
                  'Resources': resourcesPath || '.',
                  'css': pathJoin(wsPath, 'ext/requirejs/plugins/css'),
                  'js': pathJoin(wsPath, 'ext/requirejs/plugins/js'),
                  'native-css': pathJoin(wsPath, 'ext/requirejs/plugins/native-css'),
                  'normalize': pathJoin(wsPath, 'ext/requirejs/plugins/normalize'),
                  'html': pathJoin(wsPath, 'ext/requirejs/plugins/html'),
                  'tmpl': pathJoin(wsPath, 'ext/requirejs/plugins/tmpl'),
                  'text': pathJoin(wsPath, 'ext/requirejs/plugins/text'),
                  'is': pathJoin(wsPath, 'ext/requirejs/plugins/is'),
                  'is-api': pathJoin(wsPath, 'ext/requirejs/plugins/is-api'),
                  'i18n': pathJoin(wsPath, 'ext/requirejs/plugins/i18n'),
                  'json': pathJoin(wsPath, 'ext/requirejs/plugins/json'),
                  'order': pathJoin(wsPath, 'ext/requirejs/plugins/order'),
                  'template': pathJoin(wsPath, 'ext/requirejs/plugins/template'),
                  'cdn': pathJoin(wsPath, 'ext/requirejs/plugins/cdn'),
                  'datasource': pathJoin(wsPath, 'ext/requirejs/plugins/datasource'),
                  'xml': pathJoin(wsPath, 'ext/requirejs/plugins/xml'),
                  'preload': pathJoin(wsPath, 'ext/requirejs/plugins/preload'),
                  'browser': pathJoin(wsPath, 'ext/requirejs/plugins/browser'),
                  'optional': pathJoin(wsPath, 'ext/requirejs/plugins/optional'),
                  'remote': pathJoin(wsPath, 'ext/requirejs/plugins/remote'),
                  'bootup' : pathJoin(wsPath, 'res/js/bootup'),
                  'bootup-min' : pathJoin(wsPath, 'res/js/bootup-min'),
                  'old-bootup' : pathJoin(wsPath, 'res/js/old-bootup')
               },
               testing: typeof jstestdriver !== 'undefined',
               waitSeconds: 60
            };

         //Save splitted-core mode in config
         global.wsConfig.isSplittedCore = splittedCore;

         if (splittedCore) {
            cfg.paths['WS.Core'] = wsPath;
         }

         if (typeof window !== 'undefined' && window.buildnumber) {
            cfg.cacheSuffix = '.v' + window.buildnumber;
         }
         if (splittedCore && isDebugMode()) {
            cfg.cacheSuffix = '.source';
            cfg.cacheMatch = /^((?!\/(cdn|rtpackage|webrtc|vmsg)\/).)*\.(js|xhtml|tmpl)$/;
         }

         if (options) {
            var prop,
               value;
            for (prop in options) {
               if (!options.hasOwnProperty(prop)) {
                  continue;
               }
               value = options[prop];
               if (prop === 'requirejsPaths') {
                  addPaths(baseUrl, cfg.paths, value);
               } else {
                  cfg[prop] = value;
               }
            }
         }

         return cfg;
      },

      //Setup startup config for RequireJS
      setupConfig = function (require) {
         global.wsConfig = global.wsConfig || {};

         var
            //Application root default path
            APP_ROOT_DEFAULT = '/',
            //Resources default path
            RESOURCES_PATH_DEFAULT = 'resources/',
            //Bundles config received
            bundles = global.bundles,
            //Leading slash selector
            leadSlash = /^\//,
            //Default options
            options = global.contents || {},
            //Application root
            baseUrl,
            //Resources path
            resourcesPath,
            //WS path
            wsPath;

         if (global.wsConfig.wsRoot && isSplittedCore(global.wsConfig.wsRoot)) {
            baseUrl = global.wsConfig.appRoot || APP_ROOT_DEFAULT;
            resourcesPath = RESOURCES_PATH_DEFAULT;
            wsPath = RESOURCES_PATH_DEFAULT + 'WS.Core/';
         } else {
            baseUrl = APP_ROOT_DEFAULT;
            resourcesPath = (global.wsConfig.resourceRoot || RESOURCES_PATH_DEFAULT).replace(leadSlash, '');
            wsPath = (global.wsConfig.wsRoot || 'ws/').replace(leadSlash, '');
         }

         //Bundles post processing
         if (
             bundles &&
             (isSplittedCore(wsPath) ? !isDebugMode() : resourcesPath.indexOf('debug/') === -1)
         ) {
            var bundle,
                wsPrefix = /^ws\//,
                resourcesPrefix = /^resources\//;
            for (bundle in bundles) {
                if (bundle.indexOf(wsPath) !== 0 && bundle.match(wsPrefix)) {
                    bundles[bundle.replace(wsPrefix, wsPath)] = bundles[bundle];
                    delete bundles[bundle];
                } else if (bundle.indexOf(resourcesPath) !== 0 && bundle.match(resourcesPrefix)) {
                    bundles[bundle.replace(resourcesPrefix, resourcesPath)] = bundles[bundle];
                    delete bundles[bundle];
                }
            }
            options.bundles = bundles || {};
         }

         require.config(createConfig(
            baseUrl,
            wsPath,
            resourcesPath,
            options
         ));
      };

   if (isServerScript) {
      // Just return config constructor on server
      module.exports = createConfig;
   } else {
      // Apply RequireJS config in browser
      setupConfig(requirejs);
   }
}());
