(function() {
   var global = (function(){ return this || (0,eval)('this'); }()),
      define = global.define || (global.requirejs && global.requirejs.define) || (requirejsVars && requirejsVars.define);

   define('remote', ['Core/IoC', 'Core/load-contents', 'Core/constants', 'text'], function(IoC, loadContents, _const) {
      'use strict';
      //добавляем проверку в isNode на наличие http. Если его нет, то это СП и запросы там надо обрабатывать иначе.
      var http,
         isNode = typeof process !== 'undefined' && (http = require('http')),
         isBrowser = typeof window !== 'undefined',
         trailServicePath = /\/service\/(sbis-rpc-service300.dll)?/,
         loadedServices = {};

      function parse(name) {
         var
            serviceAndModule = name.split('?'),
            service = serviceAndModule[1] ? serviceAndModule.shift() : '',
            module = serviceAndModule.join('?');

         return {
            service: service,
            module: module
         };
      }

      function mkRemoteUrl(servicePath) {
         var
            req = process.domain && process.domain.req && process.domain.req || {},
            headers = req.headers || {},
            host = headers.host,
            proto = headers['x-forwarded-proto'] || headers['x-forwarded-protocol'] || 'http';

         proto = proto.split(',')[0];

         return proto + '://' + host + servicePath;
      }

      return {
         load: function(name, require, onLoad, conf) {
            var start = new Date().getTime(),
               logger = IoC.resolve('ILogger'),
               meta = parse(name),
               services = _const.services || {},
               servicePath = (meta.service in services ? services[meta.service] : meta.service) || '',
               url;

            function callback(name) {
               require([name], function (result) {
                  if (logger && typeof logger.debug === 'function') {
                     logger.debug('REMOTE', 'Module %s lоaded from remote service %s for %d ms', name, url, (new Date().getTime()) - start);
                  }
                  onLoad(result);
               }, function(err) {
                  onLoad.error(err);
               });
            }

            servicePath = servicePath.replace(trailServicePath, '');
            if (!servicePath) {
               onLoad.error('Remote service not found!');
            } else {
               if (!loadedServices[servicePath]) {
                  if (_const.appRoot && _const.appRoot.indexOf('/debug/') > -1) {
                     servicePath += '/debug';
                  }

                  url = isNode ? mkRemoteUrl(servicePath) : servicePath;

                  var
                     d = new Date(),
                     today = '' + d.getFullYear() + d.getMonth() + d.getDate();

                  var load = function(json) {
                     try {
                        var contents = JSON.parse(json);
                        if (!loadedServices[servicePath]) {
                           loadedServices[servicePath] = true;

                           // Теперь надо получить contents.js с другого сервиса и уже потом загружать нужный модуль
                           loadContents(contents, false, {
                              service: url,
                              preferSource: true,
                              queryParams: '?remote=' + (contents.buildnumber || today)
                           });
                        }
                        callback(meta.module);
                     } catch (e) {
                        onLoad.error(e);
                     }
                  };

                  load.error = function(e) {
                     onLoad.error(e);
                  };

                  var contentsUrl = url + '/resources/contents.json';
                  if (isNode) {
                     var protocol = contentsUrl.startsWith('https') ? require('https') : http;
                      protocol.get(contentsUrl, function (res) {
                        var statusCode = res.statusCode;
                        if (statusCode > 399 && statusCode < 600) {
                           res.resume();
                           return load.error(new Error(url + ' HTTP status: ' + statusCode));
                        }

                        res.setEncoding('utf8');
                        var rawData = '';
                        res.on('data', function(chunk) {
                           rawData += chunk;
                        });
                        res.on('end', function() {
                           load(rawData);
                        });
                     }).on('error', load.error);
                  } else if(isBrowser) {
                     var xhr = new XMLHttpRequest();
                     xhr.open('GET', contentsUrl, true);

                     xhr.onreadystatechange = function () {
                        var statusCode, err;

                        if (xhr.readyState === 4) {
                           statusCode = xhr.status || 0;
                           if (statusCode > 399 && statusCode < 600) {
                              //An http 4xx or 5xx error. Signal an error.
                              err = new Error(url + ' HTTP status: ' + statusCode);
                              err.xhr = xhr;
                              load.error(err);
                           } else {
                              load(xhr.responseText);
                           }
                        }
                     };
                     xhr.send(null);
                  } else {
                     load(TextRequest(contentsUrl));
                  }
               } else {
                  callback(meta.module);
               }
            }
         }
      }
   });
})();
