define('Lib/Control/FileBrowse/FileBrowse', [
   "Core/core-extend",
   "Core/core-merge",
   "Lib/Control/FileButton/FileButton",
   "Lib/Control/FileLoader/FileLoader",
   "Core/IoC"
], function(cExtend, cMerge, Button, FileLoader, IoC) {

   'use strict';

    IoC.resolve('ILogger').log('Lib/Control/FileBrowse/FileBrowse', 'Module is deprecated and will be removed, use "Lib/File/Attach/SbisAttach"');

   /**
    * Модуль 'Компонент кнопка загрузки файла'. Умеет загружать файлы на сервер. Использует модуль загрузки и выбора файлов FileLoader.
    * @deprecated Использование класса FileBrowse признано устаревшим.
    * Для загрузки файлов используйте класс {@link Lib/File/Attach/SbisAttach}.
    * Для получения файлов из файловой системы используйте один из классов:
    * <ul>
    *     <li> {@link Lib/File/ResourceGetter/FSGetter} - для выбора посредством нативного окна браузера;</li>
    *     <li> {@link SBIS3.Plugin/components/Extensions/Integration/FileGetter/DialogsGetter} - для выбора с помощью окла СБИС Плагин;</li>
    *     <li> {@link SBIS3.FL.RichGetter} - для открытия голубоко окошка выбора.</li>
    * </ul>
    * @class Lib/Control/FileBrowse/FileBrowse
    * @extends Lib/Control/FileButton/FileButton
    * @author Крайнов Д.О.
    * @control
    * @public
    * @category Button
    * @initial
    * <component data-component='Lib/Control/FileBrowse/FileBrowse'>
    *    <option name='caption'>FileBrowse</option>
    * </component>
    */
   var FileBrowse = Button.extend(/** @lends Lib/Control/FileBrowse/FileBrowse.prototype */{
      /**
       * @cfg {String[]} Список расширений выбираемых файлов
       * @name Lib/Control/FileBrowse/FileBrowse#extensions
       * @description
       *
       * Массив расширений, разрешенных для выбора, в случае выбора через плагин.
       * @example
       * <pre>
       *    extensions: ['exe','jpg','png']
       * </pre>
       */
      /**
       * @cfg {Boolean} Выбрать несколько файлов
       * @name Lib/Control/FileBrowse/FileBrowse#multiple
       * @description
       *
       * Позволяет выбрать несколько файлов.
       * @example
       * <pre>
       *    multiple: true
       * </pre>
       */
      /**
       * @cfg {Boolean} Выбрать несколько файлов
       * @name Lib/Control/FileBrowse/FileBrowse#dropElement
       *
       * @cfg {String} Элемент для drag n drop
       * Позволяет перетащить на него файлы для загрузки
       * Может быть селектором, jQuery объектом или DOM Element
       * @example
       * <pre>
       *    dropElement: $('div') || 'div' || document.getElementById('div')
       * </pre>
       */

      _createButton: function(options, mixin) {
         var fileLoaderContainer = $('<div class="ws-file-loader"></div>');
         this._container.append(fileLoaderContainer);
         var loader = mixin ? cExtend.mixin(FileLoader, mixin) : FileLoader;
         return new loader(cMerge(options, {
            element: fileLoaderContainer,
            dropElement: options.dropElement || this.getContainer(),
            name: this._options.name ? this._options.name + '-FileLoader' : ''
         }));
      },

      /**
       *
       * Вызывает выбор файла из кода
       * Поскольку клик происходит в коде, в IE8 и в Chrome не работатет.
       * В этом случае работаем через плагин
       */
      selectFile: function(event, onlyPlugin) {
         return this.activate(event, onlyPlugin);
      },

      _activate: function(event, onlyPlugin) {
         this._fileButton.selectFile(event, onlyPlugin);
      }
   });

   return FileBrowse;

});