define('Lib/Control/FileButton/FileButton', [
   "Core/core-merge",
   "Core/Deferred",
   "Deprecated/Controls/Button/Button",
   "Core/IoC",
   'Core/constants'
], function(cMerge, cDeferred, Button, IoC, constants) {

   'use strict';

    IoC.resolve('ILogger').log('Lib/Control/FileButton/FileButton', 'Module is deprecated and will be removed, use "Lib/File/Attach/SbisAttach"');

   /**
    * @deprecated Использование класса FileButton признано устаревшим.
    * Для загрузки файлов используйте класс {@link Lib/File/Attach/SbisAttach}/
    * Для получения файлов используйте один из классов:
    * <ul>
    *     <li> {@link Lib/File/ResourceGetter/FSGetter} - для выбора посредством нативного окна браузера</li>
    *     <li> {@link SBIS3.Plugin/components/Extensions/Integration/FileGetter/DialogsGetter} - для выбора с помощью окла СБИС Плагин</li>
    *     <li> {@link SBIS3.FL.RichGetter} - для открытия голубоко окошка выбора</li>
    *     <li> {@link SBIS3.Plugin/components/Extensions/Integration/FileGetter/ClipboardGetter} - для получения из буфера обмена</li>
    *     <li> {@link SBIS3.Plugin/components/Extensions/Integration/FileGetter/ScannerGetter} - для получения скана документа</li>
    *     <li> {@link Lib/File/ResourceGetter/PhotoCam} - для получения фото с камеры</li>
    * </ul>
    * @class Lib/Control/FileButton/FileButton
    * @extends Deprecated/Controls/Button/Button
    * @author Крайнов Д.О.
    * @category Button
    * @public
    * @designTime plugin /design/DesignPlugin
    */
   var FileButton = Button.extend(/** @lends Lib/Control/FileButton/FileButton.prototype */{
      /**
       * @event onActivated Происходит при клике на кнопку.
       * @param {Core/EventObject} eventObject Дескриптор события.
       * @return {Boolean} Если результат false, то будет отменена загрузка файла
       * @example
       * <pre>
       *    loadFile.subscribe('onActivated', function(event) {
       *       if (this.getFileName() === 'sbis.exe') {
       *          event.setResult( true );
       *       } else {
       *
       *          // Отменим загрузку файла
       *          event.setResult( false );
       *       }
       *    });
       * </pre>
       */
      /**
       * @event onLoadStarted Событие, происходящее перед началом загрузки файла
       *
       * Происходит перед началом загрузки файла. Позволяет изменять параметры фильтрации
       * — Обработка результата:
       * False – отмена загрузки файла.
       * cDeferred - дожидаемся результата выполнения асинхронной операции
       * @param {Core/EventObject} eventObject Дескриптор события описание в классе Core/Abstract
       * @example
       * <pre>
       *    loadFile.subscribe('onLoadStarted', function(event){
       *       if( this.getFileName() === 'sbis.exe' ){
       *          event.setResult( true );
       *          this.setUploadParams({'ИдО': 12});
       *       } else
       *          event.setResult( false ); // Отменим загрузку файла
       *    });
       * </pre>
       */
      /**
       * @event onLoaded Событие, происходящее после загрузки файла
       *
       * Происходит после загрузки файла. Позволяет обработать результат загрузки файла
       * — Обработка результата:
       * Результат не обрабатывается
       * @param {Core/EventObject} eventObject Дескриптор события описание в классе Core/Abstract
       * @param {Object} response Ответ бизнес-логики
       * @param {Boolean} isFinished Сигнализирует о том, что загруженый файл был последний
       * @example
       * <pre>
       *    loadFile.subscribe('onLoaded', function(event, response, isFinished){
       *       if( isFinished && response.result > 0 )
       *          this.getTopParent().close();
       *    });
       * </pre>
       */
      /**
       * @event onAppletReady Событие, происходящее после запуска плагина
       *
       * Происходит после запуска плагина. Позволяет выполнить какие-нибудь действия с плагином
       * — Обработка результата:
       * Результат не обрабатывается
       * @param {Core/EventObject} eventObject Дескриптор события описание в классе Core/Abstract
       * @param {String} id Идентификатор контрола
       * @param {Object} plugin Сам плагин
       * @param {Object} error Ошибка загрузки плагина
       * @example
       * <pre>
       *    loadFile.subscribe('onAppletReady', function(event, id, plugin, error){
       *       if( plugin ){
       *          var params = plugin.getParams();
       *          $('id').innerHtml = generateParams( params );
       *       } else
       *          alert( error.message );
       *    });
       * </pre>
       */
      /**
       * @event onChange Событие, происходящее после выбора файла, возвращает путь до файла
       *
       * Происходит после выбора файла, возвращает путь до файла
       * — Обработка результата:
       * Результат не обрабатывается
       * @param {Core/EventObject} eventObject Дескриптор события описание в классе Core/Abstract
       * @param {String} fileName Путь до файла
       * @example
       * <pre>
       *    loadFile.subscribe('onChange', function(event, fileName){
       *       record.set('ИмяФайла', fileName);
       *    });
       * </pre>
       */
      /**
       * @cfg {Boolean} Показывать индикатор при загрузке файла
       * @name Lib/Control/FileButton/FileButton#showIndicator
       * @description
       *
       * Возможные значения:
       * <ul>
       *    <li>true - отображать индикатор во время загрузки файла;</li>
       *    <li>false - не показывать индикатор загрузки.</li>
       * </ul>
       */
      /**
       * @cfg {String[]} Названия параметров метода
       * @name Lib/Control/FileButton/FileButton#filterParams
       * @description
       *
       * Массив названий параметров метода.
       * @example
       * <pre>
       *    ['ИдО', 'Дополнительный параметр', 'Ид']
       * </pre>
       * <pre>
       *     <options name="filterParams" type="array">
       *        <option>ИдО</option>
       *     </options>
       * </pre>
       */
      /**
       * @cfg {Object.<String, Boolean|Number|String>} Название полей из контекста
       * @name Lib/Control/FileButton/FileButton#paramsMapping
       * @description
       *
       * Объект с названиями полей из контекста, которые должны попасть в параметры
       * @example
       * <pre>
       *    {
       *       'ИдО': 123,
       *       'Дополнительный параметр' : '123'
       *    }
       * </pre>
       * <pre>
       *     <options name="paramsMapping">
       *        <option name="ИдО">@ДляКартинки</option>
       *     </options>
       * </pre>
       */
      /**
       * @cfg {Object.<String, Boolean|Number|String>} Фиксированные параметры метода
       * @name Lib/Control/FileButton/FileButton#queryFilter
       * @description
       *
       * Объект со значениями параметров метода, которые надо взять из контекста
       * @example
       * <pre>
       *    {
       *       'Ид': 'UUID файла'
       *    }
       * </pre>
       */
      /**
       * @cfg {String} Метод загрузки файла на бизнес-логике
       * @name Lib/Control/FileButton/FileButton#method
       * @description
       *
       * Метод загрузки файла на бизнес-логике
       * <pre>
       *    method: 'Документ.ПрикрепитьФайл'
       * </pre>
       * @editor MethodBLChooser
       */
      /**
       * @cfg {String} Адрес сервиса загрузки данных
       * @name Lib/Control/FileButton/FileButton#otherUrl
       * @description
       *
       * Адрес сервиса на который будет загружаться файл
       * <pre>
       *    otherUrl: 'https://wi.sbis.ru/service/'
       * </pre>
       */
      $protected: {
         _fileButton: undefined,
         _options: {
            /**
             * @cfg {String} Стиль отображения элемента
             *
             * Как правило используется для кнопок: обычная или в виде ссылки.
             * @variant classic классическая стандартное отображение в виде кнопки;
             * @variant asLink отображение кнопки ввиде ссылки.
             * @example
             * <pre>
             *     <option name="renderStyle">asLink</option>
             * </pre>
             */
            renderStyle : 'classic',
            /**
             * @cfg {String} Имя миксина для подмешивания к загрузке файлов
             *
             * Имя миксина для подмешивания к загрузке файлов
             * <pre>
             *    mixin: 'SBIS3.EDO.SecureFileLoaderMixin'
             * </pre>
             */
            mixin: null
         }
      },

      $constructor: function() {
         this._publish('onLoadStarted','onLoaded','onAppletReady','onChange','onDrop','onChangeError');

         var self = this;
         if (typeof this._options.mixin === 'string') {
            require([(this._options.mixin in constants.jsModules) ? 'js!' + this._options.mixin : this._options.mixin], function(mixin) {
               self._prepareLoader(mixin);
            }, function(error) {
               throw error;
            });
         } else {
            this._prepareLoader();
         }
      },

      _prepareLoader: function(mixin) {
         var self = this,
             cfg = {},
             opts = [
                 'createZip', 'extensions', 'multiple', 'showIndicator', 'filterParams', 'paramsMapping',
                 'queryFilter', 'method', 'otherUrl', 'loadTimeout', 'dropElement', 'installDlg'
             ];

         opts.forEach(function(opt) {
            if (self._options[opt] !== undefined) {
               cfg[opt] = self._options[opt];
            }
         });
         var handlerWrappers = {};
         ['onLoadStarted','onLoaded','onAppletReady','onChange','onDrop','onChangeError'].forEach(function(eventName) {
             handlerWrappers[eventName] = function(eventObject) {
                var args = [eventName].concat(Array.prototype.slice.call(arguments, 1));
                eventObject.setResult(self._notify.apply(self, args));
             }
         });
         this._fileButton = this._createButton(cMerge(cfg, {
            parent: this.getParent(),
            linkedContext: this.getLinkedContext(),
            handlers: handlerWrappers
         }), mixin);

         // Setters
         opts.forEach(function(opt) {
            var func = 'set' + opt[0].toUpperCase() + opt.substr(1);
            if (self._fileButton[func]) {
               self[func] = function() {
                  self._fileButton[func].apply(self._fileButton, arguments);
               }
            }
         });

         // Getters
         opts.forEach(function(opt) {
            var func = 'get' + opt[0].toUpperCase() + opt.substr(1);
            if (self._fileButton[func]) {
               self[func] = function() {
                  return self._fileButton[func].apply(self._fileButton, arguments);
               }
            }
         });
      },

      /**
       * Подписывается на клик по кнопке
       * @private
       */
      _onClickHandler: function(event) {
         if (event.which <= 1) {
            var result = this.activate(event);
            if (result instanceof cDeferred) {
               this._toggleProcess(true);
               result.addBoth(this._toggleProcess.bind(this, false));
            }

            event.stopPropagation();
            event.stopImmediatePropagation();
            event.preventDefault();
            return false;
         }
         return true;
      },

      /**
       * Устанавливает параметры загрузки файла.
       * Методы для загрузки файлов описаны в документации:
       * * {@link https://wi.sbis.ru/doc/platform/developmentapl/workdata/logicworkapl/objects/blmethods/blfile/upload/ метод "Загрузить"},
       * * {@link https://wi.sbis.ru/doc/platform/developmentapl/workdata/logicworkapl/objects/blmethods/blfile/upload-and-attach/ метод "Загрузить с прикреплением"}.
       * @param {Object} params Параметры загрузки файла.
       * @returns {*}
       * @example
       * Пример 1.
       * <pre>
       *     // Проверяем выбрано ли изображение
       *     if (photo._fileButton._curval) {
       *         // Сохраняем @Лицо созданной записи в параметрах загрузки изображения
       *         photo.setUploadParams({'ИдО': this._getCurrentRecord().get("@Лицо")});
       *         . . .
       *     }
       * </pre>
       * Пример 2.
       * <pre>
       *     this.setUploadParams({'Параметры':editParamsHandler(new Record()), 'ИдСессии':''});
       * </pre>
       * @see Lib/Control/FileButton/FileButton#queryFilter
       * @see Lib/Control/FileButton/FileButton#filterParams
       */
      setUploadParams: function(params) {
         return this._fileButton.setUploadParams(params);
      },

      /**
       * Устанавливает метод БЛ, который будет использован для загрузки файла.
       * Методы для загрузки файлов описаны в документации:
       * * {@link https://wi.sbis.ru/doc/platform/developmentapl/workdata/logicworkapl/objects/blmethods/blfile/upload/ метод "Загрузить"},
       * * {@link https://wi.sbis.ru/doc/platform/developmentapl/workdata/logicworkapl/objects/blmethods/blfile/upload-and-attach/ метод "Загрузить с прикреплением"}.
       * @param {String} method Метод загрузки файла.
       * @returns {*}
       * @example
       * Пример 1.
       * <pre>
       *     fileBrowse.setTargetMethod(options.uploadPhotoMethod);
       * </pre>
       * Пример 2.
       * <pre>
       *    // Для загрузки изображения создаём объект и устанавливаем параметры
       *    var fileCamLoader = new FileCamLoader();
       *    fileCamLoader.setUploadParams(params);
       *    fileCamLoader.setTargetMethod(methodName);
       *    . . .
       * </pre>
       */
      setTargetMethod: function(method) {
         return this._fileButton.setTargetMethod(method);
      },

      /**
       *
       * Возвращает имя последнего выбранного файла
       * @example
       * <pre>
       *    if( this.getFileName() === 'sbis.exe' )
       *       return true;
       * </pre>
       * @return {String}
       */
      getFileName: function() {
         return this._fileButton.getFileName();
      },

      isLoading: function() {
         return this._fileButton.isLoading();
      },

      /**
       * Создать кнопку загрузки файлов или сканирования
       * @param {Object} options
       * @param {Object} [mixin]
       * @private
       */
      _createButton: function(options, mixin) {
         throw new Error('FileButton:_createButton must be implemented in child classes');
      },

      _activate: function() {
         throw new Error('FileButton:_activate must be implemented in child classes');
      },

      /**
       * Активирует кнопку
       */
      activate: function() {
         if (this.isEnabled() && !this.isLoading() && this._notify('onActivated') !== false) {
            return this._activate.apply(this, arguments);
         }

         return false;
      },

      destroy: function() {
         this._fileButton && this._fileButton.destroy();
         FileButton.superclass.destroy.apply(this, arguments);
      },

      //////////////////////////////////////////
      /// Setters
      //////////////////////////////////////////
      setMixin: function(mixin) {
         if (typeof(mixin) === 'string') {
            this._options.mixin = mixin;
         }
      },

      //////////////////////////////////////////
      /// Getters
      //////////////////////////////////////////
      getMixin: function() {
         return this._options.mixin;
      }
   });

   return FileButton;

});