/**
 * Модуль 'Компонент загрузки файла'.
 * Умеет загружать файлы на сервер
 * Использует стандартный input для выбора файлов
 *
 * @description
 */
define('Lib/Control/FileLoader/FileLoader', [
   "Core/EventBus",
   "Core/CommandDispatcher",
   "Core/constants",
   "Core/Deferred",
   "Lib/Control/FileLoaderAbstract/FileLoaderAbstract",
   "tmpl!Lib/Control/FileLoader/FileLoader",
   "json!Lib/Control/FileLoader/resources/MimeTypes",
   "Core/IoC",
   "Deprecated/helpers/collection-helpers",
   "Core/detection",
   "css!Lib/Control/FileLoader/FileLoader",
   "i18n!Lib/Control/FileLoader/FileLoader"
], function(cEventBus, CommandDispatcher, cConstants, Deferred, FileLoaderAbstract, dotTplFn, MimeTypes, IoC) {

   'use strict';

    IoC.resolve('ILogger').log('Lib/Control/FileLoader/FileLoader', 'Module is deprecated and will be removed, use "Lib/File/Attach/SbisAttach"');

   var openedDialogs = {};

   function openFileDialogResult(event, data) {
      var guid = data.DialogGuid,
         result = data.Result;

      if (openedDialogs[guid]) {
         openedDialogs[guid]._postPlugin(result);
         delete openedDialogs[guid];
      }
   }

   /**
    * <b>Внимание</b>: Использование класса FileCamLoader признано устаревшим, используйте класс {@link Lib/File/Attach/SbisAttach}
    *
    * @class Lib/Control/FileLoader/FileLoader
    * @extends Lib/Control/FileLoaderAbstract/FileLoaderAbstract
    * @author Крайнов Д.О.
    * @category Loaders
    * @control
    * @public
    * @nonVisual
    * @deprecated
    */
   var FileLoader = FileLoaderAbstract.extend(/** @lends Lib/Control/FileLoader/FileLoader.prototype */{
      _dotTplFn: dotTplFn,
      $protected: {
         _fakeButton: undefined,
         _inputControl: null,
         _options: {
            /**
             * @cfg {Array} Список расширений выбираемых файлов
             *
             * Массив расширений, разрешенных для выбора, в случае выбора через плагин
             * <pre>
             *    extensions: ['exe','jpg','png']
             * </pre>
             */
            extensions: [],
            /**
             * @cfg {Boolean} Выбрать несколько файлов
             *
             * Позволяет выбрать несколько файлов
             * <pre>
             *    multiple: true
             * </pre>
             */
            multiple: false,
            /**
             * @cfg {String} Элемент для drag n drop
             * Позволяет перетащить на него файлы для загрузки
             * Может быть селектором, jQuery объектом или DOM Element
             */
            dropElement: null
         },
         _pluginOpen: false
      },

      /**
       * Инициализирует инпут выбора файла
       */
      $constructor: function() {
         CommandDispatcher.declareCommand(this, 'load', this.selectFile);

         var self = this;
         this._fakeButton = $(this._container).find('.FileLoader-input_file')
            .change(function() {
               self._changeInput($(this)[0].files);
            })
            .bind('click', function(event) {
               event.stopPropagation();
            });

         if (this._options.extensions.length) {
            if (typeof this._options.extensions == 'string') {
               this._options.extensions = this._options.extensions.split(',');
            }

            this._fakeButton.attr('accept', this._createExtensionMimeTypes());
         }

         this._preparedDropElement();

         var channel = cEventBus.channel('SbisPluginEvent');
         channel.subscribe('openFileDialogResult', openFileDialogResult, this);
         this._publish('onChangeError');
      },

      _preparedDropElement: function() {
         var dropElement = this._options.dropElement,
             self = this,
             element;

         if (!dropElement) {
            // Нет поддержки FileAPI
            return;
         }

         if (dropElement instanceof jQuery) {
            element = dropElement;
         } else if (typeof dropElement === 'string' || dropElement instanceof HTMLElement) {
            element = $(dropElement);
         } else {
            // Неизвестный тип
            return;
         }

         if (element && element.length > 0) {
            element = element[0];
            this._options.dropElement = element;

            element.ondragenter = function(event) {
                event.preventDefault();
                return true;
            };
            element.ondragover = function(event) {
                return false;
            };
            element.ondrop = function(event) {
               var dataTransfer,files;
               dataTransfer = event && event.dataTransfer;
               if (!dataTransfer){
                  return ;
               }
               event.stopPropagation();
               event.preventDefault();
               files = dataTransfer.files;
               return Deferred.callbackWrapper(self._notify('onDrop', dataTransfer, files), function(notifyResult) {
                   if ((notifyResult !== false) && files && files.length) {
                       self._changeInput(files);
                   }
               });
            };
         }
      },

       /**
        * Обработчик события выбора файлов
        * @param {FileList} selectedFiles
        * @private
        */
      _changeInput: function(selectedFiles) {
         var self = this, file, i,
             files = {};
         /*
          * Нельзя обходить FileList через "for in" + ".hasOwnProperty"
          * https://w3c.github.io/FileAPI/#filelist-section
          * Обход надо делать только по числовому индексу и получать через FileList.item({Number}) или FileList[{Number}]
          */
         for (i = 0; i < selectedFiles.length; i++) {
            file = selectedFiles.item && selectedFiles.item(i) || selectedFiles[i];
               // Отфильтруем расширения
               if (self._options.extensions.length && !self._verifyFileExtension(file)) {
                  self._notify('onChange', new Error(rk('Выбранный файл неверного типа')), file);
               } else if (self._notify('onChange', file.name, file) !== false) {
                  files[file.name] = file;
               }
         }
         self._curval = Object.keys(files).join(',');
         if (!Object.isEmpty(files)) {
            self._postInput(files);
         } else {
            var errorText = selectedFiles.length? rk('Выбранные файлы неверного типа') : rk('Не выбраны файлы для загрузки.');
            self._notify('onChangeError', new Error(errorText));
            self._log(errorText, 'FileLoader');
         }
      },

      /**
       * Формируем массив mime-type из extensions
       * @returns {string}
       * @private
       */
      _createExtensionMimeTypes: function() {
         var str = [];
         this._options.extensions.forEach(function(ext) {
            if (ext == 'audio' || ext == 'video' || ext == 'image') {
               str.push(ext + '/*');
            } else if (ext in MimeTypes) {
               str.push(MimeTypes[ext]);
            }
         });

         return str.join(',');
      },

      /**
       * Проверяет файл на валидность расширения
       * @param file
       * @returns {Boolean}
       * @private
       */
      _verifyFileExtension: function(file) {
         var extension = file.name.match(/^\S[\S\s]*\.([\S]+)$/),
             type = file.type || '',
             ext, key, len;

         extension = extension ? extension[1].toLowerCase() : '';
         if (this._options.extensions.indexOf(extension) != -1) {
            return true;
         } else if (type) {
            for (key in MimeTypes) {
               if (MimeTypes.hasOwnProperty(key) && MimeTypes[key] === type &&
                  this._options.extensions.indexOf(key) != -1) {
                  return true;
               }
            }
         }

         for (key = 0, len = this._options.extensions.length; key < len; key++) {
            ext = this._options.extensions[key];
            if ((ext == 'audio' || ext == 'video' || ext == 'image') &&
               MimeTypes[ext].indexOf(extension) != -1) {
               return true;
            }
         }

         return false;
      },

      /**
       *
       * Непосредство загружает файл на сервер,
       * предпологается, что файл уже выбран
       */
      _postInput: function(files) {
         this._xhrPost(files);
      },

      //////////////////////////////////////////
      /// Вся работа с плагином тут
      //////////////////////////////////////////

      /**
       * Инициализирует плагин
       * @private
       */
      _runFileChooserPlugin: function() {
         var self = this;
         this._getPlugins('Dialogs', '1.0.0.0').addCallbacks(function(plugin) {
            if (!self._isDestroyed) {
               self._notify('onAppletReady', self._randomId, plugin);
               plugin.call('openFileDialog', {
                  extensions: self._options.extensions,
                  multiple: self._options.multiple
               }).addCallbacks(function(guid) {
                  openedDialogs[guid] = self;
               }, function(error) {
                  self._logError(error, 'FileLoader');
               });
            }
         }, function(error) {
            self._notify('onAppletReady', self._randomId, null, error);
         });
      },

      /**
       * Вызывается перед загрузкой файлов
       * @param {Object} files Выбранный файл
       */
      _postPlugin: function(files) {
         this._post(files);
      },

      //////////////////////////////////////////
      /// Конец
      //////////////////////////////////////////

      /**
       *
       * Вызывает выбор файла из кода
       * Поскольку клик происходит в коде, в IE8 и в Chrome не работатет.
       * В этом случае работаем через плагин
       */
      selectFile: function(event, onlyPlugin) {
         var _event;
	      _event = event || window.event || this.getLinkedContext().getValueSelf('Event');
         if(!_event && cConstants.browser.firefox){
	         _event = new MouseEvent("click", {
		         bubbles: true,
		         cancelable: true,
		         view: window
	         });
         }
         // В линухе нет нотификатора, поэтому только под виндой
         // Из-за политики безопасности не работают программные клики
         if (navigator.appVersion.indexOf("Win") != -1 && (
             (
               !(_event && _event.which) &&
               (cConstants.browser.chrome || cConstants.browser.firefox)
             ) || onlyPlugin))
         {
            this._runFileChooserPlugin();
         } else {
            // Перестает стрелять change'ом если выбрать тот же файл
            // Значение теперь вообще занулять напрямую нельзя
            // Поэтому просто перезагружаем форму
            this._fakeButton.parent()[0].reset();
            this._fakeButton.click();
         }
      },

      destroy: function() {
         this._options.dropElement = ($());
         if (this._fakeButton) {
            this._fakeButton.empty().remove();
            this._fakeButton = null;
         }

         var channel = cEventBus.channel('SbisPluginEvent');
         channel.unsubscribe('openFileDialogResult', openFileDialogResult, this);
         FileLoader.superclass.destroy.apply(this, arguments);
      },

      //////////////////////////////////////////
      /// Setters
      //////////////////////////////////////////
      setExtensions: function(extensions) {
         if (extensions instanceof Array) {
            this._options.extensions = extensions;
         }
      },

      setDropElement: function(dropElement) {
         if (!dropElement) {
            // Нет поддержки FileAPI
            return;
         }

         this._options.dropElement.ondragenter = null;
         this._options.dropElement.ondragover = null;
         this._options.dropElement.ondrop = null;

         this._options.dropElement = dropElement;
         this._preparedDropElement();
      },

      //////////////////////////////////////////
      /// Getters
      //////////////////////////////////////////
      getExtensions: function() {
         return this._options.extensions;
      },

      getDropElement: function() {
         return this._options.dropElement;
      }
   });

   return FileLoader;

});
/**
 * @event onDrop Событие, перетаскивания на элемент
 * <wiTag group="Управление">
 * — Обработка результата:
 * false - отменяет загрузку файлов
 * @param {Core/EventObject} eventObject Дескриптор события
 * @param {DataTransfer} dataTransfer, оъект хранения данных, перетаскиваемые мышью во время drag & drop
 * @param {FileList} files Набор файлов из dataTransfer, если набор не пустой, файлы будут загружены на сервер
 */
/**
 * @event onChangeError Событие ошибки, когда после открытия окна выбора файлов не было выбрано ничего, либо все выбранные файлы не прошли фильтрацию по типу
 * @param {Core/EventObject} eventObject Дескриптор события
 */