/**
 * Модуль 'Компонент общий класс загрузки файла'.
 * @description
 */
define('Lib/Control/FileLoaderAbstract/FileLoaderAbstract', [
   "Core/core-merge",
   "Deprecated/ReaderSBIS",
   "Deprecated/Record",
   "Core/helpers/random-helpers",
   "Core/Indicator",
   "Core/IoC",
   "Core/ParallelDeferred",
   "Core/Deferred",
   'Core/UserInfo',
   "Core/constants",
   "Lib/Control/Control",
   "Transport/RPCJSON",
   "Core/moduleStubs",
   "Deprecated/helpers/collection-helpers",
   "Deprecated/Controls/LoadingIndicator/LoadingIndicator",
   "i18n!Lib/Control/FileLoaderAbstract/FileLoaderAbstract"
], function(cMerge, dReaderSBIS, dTransportRecord, randomHelpers, Indicator, IoC, cParallelDeferred, cDeferred, userInfo, cConstants, Control, RPCJSON, moduleStubs) {

   'use strict';

   var MESSAGES = {
       filesIsNotSelected: rk('Не выбраны файлы для загрузки.'),
       loading: rk('Загрузка...'),
       getFileNameDeprecated: rk('Функция getFileName помечена как deprecated и будет удалена с 3.8. Используйте событие onChange')
   };

    /**
     * @private
     * @param {Object<Deferred> | Array<Deferred>} steps
     * @return {Deferred<*>}
     */
    var getParallelDeferred = function (steps) {
        return new cParallelDeferred({
            steps: steps,
            stopOnFirstError: false
        }).done().getResult();
    };
   /**
    * @class Lib/Control/FileLoaderAbstract/FileLoaderAbstract
    * @extends Lib/Control/Control
    * @author Крайнов Д.О.
    * @public
    * @deprecated
    */
   var FileLoaderAbstract = Control.Control.extend(/** @lends Lib/Control/FileLoaderAbstract/FileLoaderAbstract.prototype */{
      /**
       * @event onLoadStarted Событие, происходящее перед началом загрузки файла
       *
       * Происходит перед началом загрузки файла. Позволяет изменять параметры фильтрации
       * — Обработка результата:
       * False – отмена загрузки файла.
       * cDeferred - дожидаемся результата выполнения асинхронной операции
       * @param {Core/EventObject} eventObject Дескриптор события описание в классе Core/Abstract
       * @example
       * <pre>
       *    loadFile.subscribe('onLoadStarted', function(event){
       *       if( this.getFileName() === 'sbis.exe' ){
       *          event.setResult( true );
       *          this.setUploadParams({'ИдО': 12});
       *       } else
       *          event.setResult( false ); // Отменим загрузку файла
       *    });
       * </pre>
       */
      /**
       * @event onLoaded Событие, происходящее после загрузки файла
       *
       * Происходит после загрузки файла. Позволяет обработать результат загрузки файла
       * — Обработка результата:
       * Результат не обрабатывается
       * @param {Core/EventObject} eventObject Дескриптор события описание в классе Core/Abstract
       * @param {Object} response Ответ бизнес-логики
       * @param {Boolean} isFinished Сигнализирует о том, что загруженый файл был последний
       * @example
       * <pre>
       *    loadFile.subscribe('onLoaded', function(event, response, isFinished){
       *       if( isFinished && response.result > 0 )
       *          this.getTopParent().close();
       *    });
       * </pre>
       */
      /**
       * @event onAppletReady Событие, происходящее после запуска плагина
       *
       * Происходит после запуска плагина. Позволяет выполнить какие-нибудь действия с плагином
       * — Обработка результата:
       * Результат не обрабатывается
       * @param {Core/EventObject} eventObject Дескриптор события описание в классе Core/Abstract
       * @param {String} id Идентификатор контрола
       * @param {Object} plugin Сам плагин
       * @param {Object} error Ошибка загрузки плагина
       * @example
       * <pre>
       *    loadFile.subscribe('onAppletReady', function(event, id, plugin, error){
       *       if( plugin ){
       *          var params = plugin.getParams();
       *          $('id').innerHtml = generateParams( params );
       *       } else
       *          alert( error.message );
       *    });
       * </pre>
       */
      /**
       * @event onChange Событие, происходящее после выбора файла, возвращает путь до файла
       *
       * Происходит после выбора файла, возвращает путь до файла
       * — Обработка результата:
       * Результат не обрабатывается
       * @param {Core/EventObject} eventObject Дескриптор события описание в классе Core/Abstract
       * @param {String} fileName Путь до файла
       * @param {Object} file Информация о файле (размер, дата изменения, тип)
       * @example
       * <pre>
       *    loadFile.subscribe('onChange', function(event, fileName, file){
       *       record.set('ИмяФайла', fileName);
       *    });
       * </pre>
       */

      /**
       * @event onReady
       *
       */
      /**
       * @event onDestroy
       *
       */
      /**
       * @event onFocusIn
       *
       */
      /**
       * @event onFocusOut
       *
       */
      /**
       * @event onKeyPressed
       *
       */
      /**
       * @event onClick
       *
       */
      /**
       * @event onSizeChanged
       *
       */
      /**
       * @event onStateChanged
       *
       */
      /**
       * @event onTooltipContentRequest
       *
       */

      $protected : {
         _options : {
            /**
             * @cfg {String[]} Названия параметров метода
             *
             * Массив названий параметров метода
             * <pre>
             *    ['ИдО', 'Дополнительный параметр', 'Ид']
             * </pre>
             */
            filterParams : [],
            /**
             * @cfg {Object.<string, boolean|number|string>} Название полей из контекста
             *
             * Объект с названиями полей из контекста, которые должны попасть в параметры
             * <pre>
             *    {
             *       'ИдО': 123,
             *       'Дополнительный параметр' : '123'
             *    }
             * </pre>
             */
            paramsMapping : {},
            /**
             * @cfg {Object} Фиксированные параметры метода
             *
             * Объект со значениями параметров метода, которые надо взять из контекста
             * <pre>
             *    {
             *       'Ид': 'UUID файла'
             *    }
             * </pre>
             */
            queryFilter : {},
            /**
             * @cfg {String} Метод загрузки файла на бизнес-логике
             *
             * Метод загрузки файла на бизнес-логике
             * <pre>
             *    method: 'Документ.ПрикрепитьФайл'
             * </pre>
             * @editor MethodBLChooser
             */
            method : '',
            /**
             * @cfg {String} Адрес сервиса загрузки данных
             *
             * Адрес сервиса на который будет загружаться файл
             * <pre>
             *    otherUrl: 'https://wi.sbis.ru/service/'
             * </pre>
             */
            otherUrl: '',
            /**
             * @cfg {Boolean} Показывать индикатор при загрузке файла
             * Если true, то при загрузке файла будет показан индикатор загрузки
             *
             */
            showIndicator: true,
            /**
             * @cfg {Number} Время ожидания ответа при загрузке в секундах
             * Должно быть больше 60
             */
            loadTimeout: 60
         },
         _eventBusChannelPlugins: null,
         _plugin: {},
         _randomId: '',
         _targetUrl: '',
         _loading: false
      },

      $constructor: function(){
         this._publish('onLoadStarted', 'onLoaded', 'onAppletReady');

         this._targetUrl = this._getTargetUrl();
         this._randomId = randomHelpers.randomId();
      },

      _getTargetUrl: function() {
         var url = this._options.otherUrl ? this._options.otherUrl : cConstants.defaultServiceUrl;

         var host= [window.location.protocol,window.location.hostname].join('//');
         if (window.location.port && window.location.protocol == 'http:') {
            host += ':' + window.location.port;
         }

         url = /^https?:/.test(url) ? url : host + url;

         return url;
      },

      _log: function(message, className) {
         IoC.resolve('ILogger').log(className || 'FileLoaderAbstract', message);
      },

      _logError: function(error, className) {
         IoC.resolve('ILogger').error(className || 'FileLoaderAbstract', error.message || error);
      },

      /**
       * Возвращает имя последнего выбранного файла
       * @private
       */
      _notFormatedVal: function() {
         return this._curval;
      },

      /**
       * Показывает индикатор загрузки файла
       * @protected
       */
      _showIndicator: function() {
         Indicator.setMessage(MESSAGES.loading);
      },

      /**
       * Скрывает индикатор загрузки файла
       * @protected
       */
      _hideIndicator: function() {
         Indicator.hide();
      },

      /**
       * Возвращает объект с параметрами загрузки файла
       * @returns {Object}
       * @protected
       */
      _getRequestParameters: function() {
         var requestParams = {
            fileName : '',                      //The user selected file name (it is full path to file on client machine)
            targetURL : this._targetUrl,        //The service URL where need to send file
            method :  this._options.method,     // The name of Business Logic method which is handle the file
            XSBISSessionID : userInfo.getCurrent(),   // get XSbisCookie for file upload auth
            requestParams : {                   // additional params for request
               //ИдО: ''
               //some other parameters
            },
            fileParams: {                       // additional params for file
               //Ид:'9ec02050-4660-ce8c-910e-9ee97a9f69a0'
               //some other parameters
            }
         };

         var params = this._prepareParams();

         for (var i in params) {
            if (params.hasOwnProperty(i)) {
               var
                  param = params[i];
               if (param instanceof dTransportRecord) {
                  param = param.toObject(true);
               }
               var serializedVal = dReaderSBIS.serializeParameter(i, param).d;
               if (i == 'Ид' || i == 'ИмяФайла' || i == 'Размер') {
                  requestParams.fileParams[i] = serializedVal;
               } else {
                  requestParams.requestParams[i] = serializedVal;
               }
            }
         }
         return requestParams;
      },

      /**
       * Подготавливает объект с параметрами запроса
       * @returns {Object}
       * @protected
       */
      _prepareParams: function(){
         var params = cMerge({}, this._options.queryFilter),
             filter = this._options.filterParams;
         for (var i = 0, l = filter.length; i < l; i++) {
            if (!(filter[i] in params)) {
               params[filter[i]] = this.getLinkedContext().getValue(
                  filter[i] in this._options.paramsMapping ? this._options.paramsMapping[filter[i]] : filter[i]
               );
            }
         }
         return params;
      },
      _parseResponse: function(resp) {
         var json;
         if (resp instanceof Error) {
            if (resp.message.indexOf('System.Net.WebException') > -1){
               resp.message = resp.message
                  .replace('System.Net.WebException: ', '')
                  .replace(/\n[\s\S]*/g, '')
                  .replace(/\n/g, '')
                  .replace(/\r/g, '');
               try{
                   resp.code = resp.message.match(/\(?(\d+)\)?/)[1];
               }catch (err){
                   resp.code = "";
               }
            }

            json = {error: {code: resp.code, message: resp.message}};
         } else if (Object.prototype.toString.call(resp) == '[object Object]' && (resp.result || resp.error)) {
            if (resp.error && resp.error.message) {
               var temp;
               try {
                  temp = JSON.parse(resp.error.message);
                  if (temp && temp.error) {
                     temp.error.code = resp.error.code;
                     resp.error = temp.error;
                  }
               } catch (e) { }
            }
            json = resp;
         } else {
            try {
               json = JSON.parse(resp);
            } catch (e) { }
         }

         if (!json) {
            json = {error: {}};
         }
         return json;
      },

      /**
       * После загрузки файла на сервер
       * @param {String|Object} resp Ответ бизнес-логики
       */
      _postFinished: function(resp) {
         var self = this;
         if (this._loading) {
            if (Object.prototype.toString.call(resp) == '[object Object]') {
               for (var index in resp) {
                  if (resp.hasOwnProperty(index)) {

                     resp[index] = self._parseResponse(resp[index]);
                  }
               }
               if (Object.keys(resp).length == 1) {
                  resp = resp[Object.keys(resp)[0]];
               }
            } else {
               resp = self._parseResponse(resp);
            }

            this._notify('onLoaded', resp, true);
            if (this._options.showIndicator) {
               this._hideIndicator();
            }
            this._loading = false;
         }
      },
       /**
        * Обработка ошибки
        * @param error
        * @return {Deferred<HTTPError | OfflineError>}
        * @private
        */
      _processingError: function (error) {
         var self = this;
         return moduleStubs.require(["Transport/XHRTransport", "Transport/HTTPError"]).addCallback(function(modules){
            var processedError, XHRTransport, HTTPError, message, code;

            XHRTransport = modules[0];
            HTTPError = modules[1];

            code = error.data && error.data.error_code || error.code || 500;
            message = error.message || XHRTransport.ERRORS_TEXT[code] || "";
            processedError = new HTTPError(
                message,
                code,
                self._targetUrl,
                undefined,
                error.details
            );
            processedError.type = error.type || "error";
            // Если не ошибка в офлайне, то пробрасываем дальше
            if (!XHRTransport.isOfflineError(processedError)){
               return processedError
            }
            var offlineModeError = HTTPError.offlineModeError(
                message,
                self._targetUrl
            );
            offlineModeError.type = error.type || "error";
            // Обработал ли кто-то результат
            offlineModeError.processed = !!XHRTransport.notifyOfflineError(self._options.method.split("."));
            return offlineModeError;
          });
      },

       /**
        * После загрузки файла на сервер
        * @param {Deferred} deferred  Асинхронный ответ бизнес-логики
        */
      _addCallbacksAfterUpload: function(deferred){
           /**
            * При contentType == false && dataType == false
            * ответ запроса в виде текста, который надо распарсить и обработать ошибку, если имеется.
            */
         var self = this;
         deferred.addBoth(function(result) {
             return self._parseResponse(result);
         });
         deferred.addBoth(function(result) {
            // Есть ошибка - обрабатываем, нет - отдадим результат.
            var error = result instanceof Error? result: result.error;
            if (!error){
               return result;
            }
            return self._processingError(error).addBoth(function(processedError){
               if (result instanceof Error){
                  return processedError;
               }
               result.error = processedError;
               return result;
            });
         });
      },
      /**
       * Отсылает файлы с помощью xhr
       * Метод не рекомендован к использованию. Реализация нового загрузчика:
       * https://inside.tensor.ru/opendoc.html?guid=adfd5c4f-2e7d-4e29-a072-f5fb85f83e99
       * @deprecated
       * @param {Array|Object <File>} files
       * @protected
       */
      xhrSend: function(files){
          return this._xhrPost(files);
      },
      _xhrPost: function(files) {
         var self = this;
         this._upload(function() {

            var steps = {};
            for (var key in files) {
               if (files.hasOwnProperty(key)) {
                  var
                     file = files[key],
                     def;
                  def = self._xhrSendOneFile(file, file.name);
                  self._addCallbacksAfterUpload(def);
                  steps[file.name] = def;
               }
            }
            return self._parallelLoad(steps);
         });
      },

      /**
       * Загружает файл на сервер через xhr
       * @param {File | Blob} blob - файл
       * @param {String} [fileName] - имя файла
       * @return {Core/Deferred}
       * @protected
       */
      _xhrSendOneFile: function(blob, fileName) {
         var
            def,
            self = this,
            params = self._getRequestParameters(),
            formData = new FormData(),
            request_params = {
               'Файл': {
                  'Данные': {
                     'href': 'file1'
                  }
               }
            };

         cMerge(request_params['Файл'], params['fileParams']);
         cMerge(request_params, params['requestParams']);

         if (fileName) {
            formData.append('file1', blob, fileName);
         } else {
            formData.append('file1', blob);
         }
         formData.append('Запрос', RPCJSON.jsonRpcPreparePacket(this._options.method, request_params).reqBody);

         def = moduleStubs.require("Transport/XHRTransport");
         def.addCallback(function(modules){
             var xhrTransport = new modules[0]({
                 method: "POST",
                 url: self._targetUrl,
                 /** Поля ниже необходимы для отправи FormData с File|Blob
                  * В случае contentType&dataType == false ответ от сервера придёт в виде строки
                  * Обрабатываем ошибки сами
                  */
                 processData: false,
                 contentType: false,
                 dataType: false
             });
             return xhrTransport.execute(formData);
         });
         return def;
      },

      /**
       * Вызывается перед загрузкой файла
       * @param {Function} callback Callback-функция, которая вызовется после события onLoadStarted
       * @protected
       */
      _upload: function(callback) {
          cDeferred.callbackWrapper(this._notify('onLoadStarted'), function(r) {
            if (r !== false) {
               callback(r);
            }
            return r;
         });
      },

      //////////////////////////////////////////
      /// Вся работа с плагином тут
      //////////////////////////////////////////

      /**
       * Инициализация плагина загрузки файла
       * @protected
       */
      _getFileLoaderPlugin: function() {
         return this._getPlugins('FileLoader', '0.0.0.5');
      },

      _getFileSystemPlugin: function () {
         return this._getPlugins('FileSystem', '0.0.0.5');
      },

      /**
       * Инициализация плагина
       * @param {String} pluginName - Название плагина
       * @param {String} pluginVersion - Версия плагина
       * @param {Object} [params] - Параметры
       * @protected
       */
      _getPlugins: function(pluginName, pluginVersion, params) {
         var self = this;
         if (self._plugin[pluginName] && self._plugin[pluginName].getState() === self._plugin[pluginName].readyState.OPEN) {
            return (new cDeferred()).callback(self._plugin[pluginName]);
         }

         var def = new cDeferred();
         require(['SBIS3.Plugin/Source/LocalService'], function (LocalService) {
            var config = params || {mode: 'silent'};
            var service = new LocalService({
               endpoint: {
                  address: pluginName + '-' + pluginVersion,
                  contract: pluginName
               },
               options: config
            });
            service.isReady().addCallbacks(
               function () {
                  self._plugin[pluginName] = service;
                  if (self._options.loadTimeout > 60) {
                     self._plugin[pluginName].setQueryTimeout(self._options.loadTimeout * 1000);
                  }

                  def.callback(service);
               },
               function (error) {
                  self._logError(error, pluginName);
                  self._plugin[pluginName] = null;

                  def.errback(error);
               }
            );
         });

         return def;
      },

      _pluginSend: function(requestParams) {
         return this._getFileLoaderPlugin().addCallback(function(plugin) {
            return plugin.call('uploadFile', {requestParams: requestParams});
         });
      },

      /**
       * Вызвает метод загрузки файлов для основного выбора файлов
       * Из-за мультивыбора пришлось извращаться
       * @param file
       * @param {Boolean} [removeAfterUpload]
       * @private
       */
      _pluginSendOneFile: function(file, removeAfterUpload) {
         var
            self = this,
            requestParams = this._getRequestParameters();

         return moduleStubs.require(['browser!SBIS3.Plugin/Source/Helper/UserDataManager']).addCallback(function (results) {
            var UserDataManager = results[0];
            requestParams.fileName = file.path;
            requestParams.newFileName = file.name;
            requestParams.AccountID = UserDataManager.getAccountId();
            if (removeAfterUpload) {
               requestParams.removeAfterUpload = true;
            }
            return self._pluginSend(requestParams);
         });
      },

       /**
        * Отправка через плагин
        * Метод не рекомендован к использованию. Реализация нового загрузчика:
        * https://inside.tensor.ru/opendoc.html?guid=adfd5c4f-2e7d-4e29-a072-f5fb85f83e99
        * @deprecated
        * @param {Array<string>} uploadedFiles - array of paths to file
        */
       pluginSend: function(uploadedFiles){
           return this._post(uploadedFiles)
       },

      /**
       * После завершении получения файла вызвает onChange и затем загружает файл на сервер
       * Метод вызвается только тогда, когда работаем через плагин
       * @param {String} uploadedFile Имя загружаемого файла
       * @param {Boolean} [removeAfterUpload]
       * @protected
       */
      _post: function(uploadedFile, removeAfterUpload) {
         if (!uploadedFile || uploadedFile == 'undefined') {
            return;
         }
         var self = this;

         this._getFileSystemPlugin().addCallback(function(plugin) {
            var paths, fileInfoDeferreds, files;

            files = {};
            paths = uploadedFile instanceof Array ? uploadedFile : uploadedFile.split('|');
            fileInfoDeferreds = [];
            paths.forEach(function(path) {
                fileInfoDeferreds.push(self._getFileInfoDeferred(path, plugin, files));
            });
            return getParallelDeferred(fileInfoDeferreds).addBoth(function() {
                self._curval = Object.keys(files).join(',');
                if (Object.isEmpty(files)) {
                    self._log(MESSAGES.filesIsNotSelected, 'FileLoaderAbstract');
                    return false;
                }

                self._upload(function() {
                    if (self._isDestroyed){
                        return;
                    }
                    var fileLoadDeferreds = {};

                   for (var key in files) {
                      if (files.hasOwnProperty(key)) {
                         var
                            file = files[key],
                            def;
                         def = self._pluginSendOneFile(file, removeAfterUpload);
                         self._addCallbacksAfterUpload(def);
                         fileLoadDeferreds[file.name] = def;
                      }
                   }
                    self._parallelLoad(fileLoadDeferreds);
                });
            });
         });
      },

       _getFileInfoDeferred: function(path, plugin, files){
          var self = this;
          return plugin.call('getFileInfo', { path : path }).addCallback(function(fileInfo) {
               fileInfo = JSON.parse(fileInfo);
               fileInfo.lastModifiedDate = Date.fromSQL(fileInfo.lastModifiedDate);
               if (self._notify('onChange', fileInfo.name, fileInfo) !== false) {
                   files[fileInfo.name] = fileInfo;
               }
           })
       },

      //////////////////////////////////////////
      /// Конец
      //////////////////////////////////////////

       /**
        * @param {Object.<Deferred> | Array.<Deferred>} steps набор Deferred'ов загрузки
        * @return {Deferred}
        */
       _parallelLoad: function(steps){
          var self = this;
          if (self._options.showIndicator) {
              self._showIndicator();
          }
          self._loading = true;
          return getParallelDeferred(steps).addBoth(function(results) {
              if (self._options.showIndicator) {
                  self._hideIndicator();
              }
              self._loading = false;
              if (Object.keys(results).length == 1) {
                  results = results[Object.keys(results)[0]];
              }
              self._notify('onLoaded', results, true);
              return results;
          });
       },

      /**
       *
       * Устанавливает параметры при загрузке файла
       * @param {Object} params Объект с параметрами
       * @example
       * <pre>
       *       {
       *          field: 10, // параметр и значение
       *          field2: {
       *             fieldName: '@Лицо' // параметр связанный с полем контекста
       *          }
       *       }
       * </pre>
       */
      setUploadParams: function(params) {
         this._options.queryFilter = {};
         this._options.paramsMapping = {};
         this._options.filterParams = [];
         for (var i in params) {
            if (params.hasOwnProperty(i)) {
               var p = params[i];
               this._options.filterParams.push(i);
               if (p && typeof(p) == 'object' && 'fieldName' in p) {
                  this._options.paramsMapping[i] = p.fieldName;
               } else {
                  this._options.queryFilter[i] = p;
               }
            }
         }
      },

      /**
       *
       * Изменяет целевой метод бизнес-логики для отправки файла
       * @param {String} methodName имя целевого метода бизнес-логики
       * @example
       * <pre>
       *    var control = $ws.single.ControlStorage.gtByName('Кнопка загрузки файла');
       *    control.setTargetMethod( 'Документ.ПрикрепитьФайл' );
       * </pre>
       */
      setTargetMethod: function(methodName) {
         this._options.method = methodName;
      },

      /**
       *
       * Возвращает имя последнего выбранного файла
       * @example
       * <pre>
       *    if( this.getFileName() === 'sbis.exe' )
       *       return true;
       * </pre>
       * @deprecated
       * @return {String}
       */
      getFileName: function() {
         IoC.resolve('ILogger').log('FileLoaderAbstract', MESSAGES.getFileNameDeprecated);
         return this._notFormatedVal();
      },

      /**
       * Происходит ли выбор или загрузка файла в текущий момент
       */
      isLoading: function() {
         return this._loading;
      },

      //////////////////////////////////////////
      /// Setters
      //////////////////////////////////////////
      setShowIndicator: function(showIndicator) {
         this._options.showIndicator = !!showIndicator;
      },

      setFilterParams: function(filterParams) {
         if (filterParams instanceof Array) {
            this._options.filterParams = filterParams;
         }
      },

      setParamsMapping: function(paramsMapping) {
         if (typeof(paramsMapping) === 'object') {
            this._options.paramsMapping = paramsMapping;
         }
      },

      setQueryFilter: function(queryFilter) {
         if (typeof(queryFilter) === 'object') {
            this._options.queryFilter = queryFilter;
         }
      },

      /**
       * Установить метод загрузки файлов
       * @param method
       */
      setMethod: function(method) {
         if (typeof(method) === 'string') {
            this._options.method = method;
         }
      },

      setOtherUrl: function(otherUrl) {
         if (typeof(otherUrl) === 'string') {
            this._options.otherUrl = otherUrl;
            this._targetUrl = this._getTargetUrl();
         }
      },

      setLoadTimeout: function(timeout) {
         if (timeout > 60) {
            this._options.loadTimeout = timeout;
         }
      },

      //////////////////////////////////////////
      /// Getters
      //////////////////////////////////////////
      getShowIndicator: function() {
         return this._options.showIndicator;
      },

      getFilterParams: function() {
         return this._options.filterParams;
      },

      getParamsMapping: function() {
         return this._options.paramsMapping;
      },

      getQueryFilter: function() {
         return this._options.queryFilter;
      },

      getMethod: function() {
         return this._options.method;
      },

      getOtherUrl: function() {
         return this._options.otherUrl;
      },

      getLoadTimeout: function() {
         return this._options.loadTimeout;
      }
   });

   return FileLoaderAbstract;

});
