/**
 * Модуль 'Компонент кнопка сканирования'.
 * Умеет сканировать документы, и если надо создавать свой даилог сканирования
 *
 * @description
 */
define('Lib/Control/FileScanLoader/FileScanLoader', [
   "Deprecated/helpers/fast-control-helpers",
   "Core/WindowManager",
   "Core/Context",
   "Core/ParallelDeferred",
   "Core/Deferred",
   "Core/CommandDispatcher",
   "Lib/Control/FileLoaderAbstract/FileLoaderAbstract",
   "Core/moduleStubs",
   "Core/IoC",
   "i18n!Lib/Control/FileScanLoader/FileScanLoader"
], function(dFCHelpers, WindowManager, cContext, cParallelDeferred, cDeferred, CommandDispatcher, FileLoaderAbstract, moduleStubs, IoC) {

   'use strict';

    IoC.resolve('ILogger').log('Lib/Control/FileScanLoader/FileScanLoader', 'Module is deprecated and will be removed, use "Lib/File/Attach/SbisAttach"');

    /// В отличии от FileBrowse, при сканировании нам нужен всегда один экземпляр плагина,
   /// поэтому метод _getPlugin вызывается один раз и надеемся, что он не умрет.
   /// Да пребудет с нами Сила.

   //////////////////////////////////////////
   /// Все вызовы методов у плагина тут
   //////////////////////////////////////////

   /**
    * Получить текущую информацию о сканерах
    * @param plugin
    * @returns {Core/Deferred}
    */
   function getScannersInfo(plugin) {
      return new cParallelDeferred({
         steps: {
            scanners: getScanners(plugin),
            lastScanner: getLastScanner(plugin)
         },
         stopOnFirstError: false
      }).done().getResult();
   }

   /**
    * Выдает все сканеры, которые доступны
    * @param plugin
    * @returns {Core/Deferred}
    */
   function getScanners(plugin) {
      return plugin.call('getDeviceNames', {}).addErrback(LogError);
   }

   /**
    * Последний используемый сканер
    * @param plugin
    * @returns {Core/Deferred}
    */
   function getLastScanner(plugin) {
      return plugin.call('getLastSelectedDevice', {}).addErrback(LogError);
   }

   /**
    * Выбрать сканер
    * @param plugin
    * @param scanner
    */
   function selectScanner(plugin, scanner) {
      return plugin.call('selectDevice', {scannerName:scanner}).addErrback(LogError);
   }

   /**
    * Вызывает диалог сканера, ждет 10 минут ответа
    * @param plugin
    * @returns {Core/Deferred}
    */
   function scan(plugin) {
      plugin.setQueryTimeout(10 /*min*/ * 60 /*sec*/ * 1000);
      return plugin.call('scan', {}).addErrback(LogError);
   }

   /**
    * Говорим плагину, чтобы начал подготовку к загрузке файлов, и дал нам путь до файла, который будем грузить
    * @param plugin
    * @param id
    * @returns {Core/Deferred}
    */
   function upload(plugin, id) {
      return plugin.call('getScansFileName', {saveScansOption: 0}).addErrback(LogError);
   }

   //////////////////////////////////////////
   /// Конец
   //////////////////////////////////////////

   /**
    * Показывает диалог выбора сканера
    * @param deferred
    * @param plugin
    */
   function showDialog(deferred, plugin) {
      // Показываем диалог выбора сканеров
      var
         finishContextDfr = new cDeferred(),
         linkedCtx = cContext.createContext(finishContextDfr);
      moduleStubs.require(["Lib/Control/Dialog/Dialog", "Deprecated/RecordSetStatic"]).addCallback(function (modules) {
         new modules[0]({
            template: 'Lib/Control/FileScanLoader/resources/SelectScannerDialog/SelectScannerDialog',
            context: linkedCtx,
            caption: rk('Выбор устройства сканирования'),
            opener: WindowManager.getActiveWindow(),
            resizable: false,
            border: false,
            handlers: {
               onBeforeShow: function () {
                  var dialog = this,
                      recordSet = new modules[1](),
                      browser = dialog.getChildControlByName('selectBrowser'),
                      button = dialog.getChildControlByName('reloadScanners'),
                      close = dialog.getChildControlByName('closeButton');

                  this._reloadScanners = reloadScanners.bind(this, plugin, recordSet, browser);
                  this._reloadScanners();

                  // Создадим рекордсет
                  recordSet.addColumn('Название', 'Текст');
                  browser.subscribe('onRowClick', function (event, row, rec) {
                     selectScanner(plugin, rec.getKey()).addBoth(function () {
                         dialog.close(true);
                     });
                  });

                  button.subscribe('onActivated', function () {
                     dialog._reloadScanners();
                  });

                  close.subscribe('onActivated', function () {
                     dialog.close(false);
                  });
               },
               onAfterClose: function (event, result) {
                  if (result) {
                     deferred.callback();
                  } else {
                     deferred.errback();
                  }
               },
               onDestroy: function () {
                  finishContextDfr.callback();
               }
            }
         })
       });
   }

   /**
    * Перезагружает список сканеров
    * @param plugin
    * @param recordSet
    * @param browser
    */
   function reloadScanners(plugin, recordSet, browser) {
      var Record,
          records = [];
      recordSet.clear();
      var def = moduleStubs.require("Deprecated/Record");
      def.addCallback(function(modules){
          Record = modules[0];
          return getScannersInfo(plugin);
      });
      def.addCallback(function(info) {
         info.scanners = info.scanners || [];
         info.lastScanner = info.lastScanner || '';
         info.scanners.forEach(function(id, i) {
            records[i] = new Record({row: [id], colDef: recordSet.getColumns(), pkValue: id});
         });

         if (info.lastScanner && info.scanners.toString().indexOf(info.lastScanner) === -1) {
            records[records.length] = new Record({row: [info.lastScanner + ' (не подключен)'], colDef: recordSet.getColumns(), pkValue: info.lastScanner});
         }

         recordSet.setRecords(records);
         browser.setData(recordSet);

         if (info.lastScanner && info.scanners.toString().indexOf(info.lastScanner) === -1) {
            browser.getContainer().find('tr[rowkey="' + info.lastScanner + '"]').addClass('not_available');
         }
      });
   }

   /**
    * Узнаем какой сканер использовать
    * 1. Если нет ни одного выбранного сканера, и вообще ни одного сканера, сообщаем пользователю
    * 2. Если нет ни одного выбранного сканера, и сканеров больше 2х, выдаем диалог с выбором сканеров
    * 3. Если выбран сканер, но он не подключен, выдаем диалог с выбором сканеров
    * 4. Если выбран сканер, но он не подключен, и есть другой сканер, выдаем диалог с выбором сканеров, и
    *    неподключеный будет серым, а второй сканер черным и его можно выбрать
    * @param plugin
    */
   function prepareScanner(plugin) {
      var deferred = new cDeferred();

      deferred.addErrback(function (e) {
         dFCHelpers.alert(e.message);
         return e;
      });

      function _selectScanner(scanner) {
         selectScanner(plugin, scanner).addCallbacks(function() {
            deferred.callback();
         }, function() {
            deferred.errback(new Error(rk('Не удалось выбрать устройство сканирования')));
         });
      }

      getScannersInfo(plugin).addCallback(function(info) {
         info.scanners = info.scanners || [];
         info.lastScanner = info.lastScanner || '';
         if (!info.lastScanner) {
            if (info.scanners.length === 0) {
               deferred.errback(new Error(rk('Устройства сканирования не найдены')));
            } else if (info.scanners.length === 1) {
               _selectScanner(info.scanners[0]);
            } else if (info.scanners.length > 1) {
               showDialog(deferred, plugin);
            }
         } else if (info.scanners.toString().indexOf(info.lastScanner) === -1) {
            showDialog(deferred, plugin);
         } else {
            _selectScanner(info.lastScanner);
         }
      });

      return deferred;
   }

   function Log(message) {
      FileLoaderAbstract.prototype._log.call(undefined, message, 'FileScanLoader');
      return message;
   }

   function LogError(error) {
      FileLoaderAbstract.prototype._logError.call(undefined, error, 'FileScanLoader');
      return error;
   }

   /**
    * Класс загрузки сканов
    * <b>Внимание</b>: Использование класса FileScanLoader признано устаревшим, используйте класс {@link Lib/File/Attach/SbisAttach}
    *
    * @class Lib/Control/FileScanLoader/FileScanLoader
    * @extends Lib/Control/FileLoaderAbstract/FileLoaderAbstract
    * @author Крайнов Д.О.
    * @category Loaders
    * @control
    * @public
    * @nonVisual
    * @deprecated
    */
   var FileScanLoader = FileLoaderAbstract.extend(/** @lends Lib/Control/FileScanLoader/FileScanLoader.prototype */{
      $protected : {
         _options : {
            /**
             * @cfg {Boolean} Сжимать несколько файлов в один zip архив
             *
             * Если true, то несколько отсканированных файлов будут помещены в zip-архив и загружен будет уже архив
             * Если false, то несколько отсканированных файлов будут загружены по отдельности
             */
            createZip: true,

            /**
             * @cfg {Boolean} Всегда ли показывать диалог установки СБИС Плагина
             *
             */
            installDlg: true
         }
      },
      $constructor: function(){
         this._container = $(); // контейнер не нужен!
         CommandDispatcher.declareCommand(this, 'scan', this.scan);
         // Показываем индикатор только когда раньше было - показывать с диалогом
         if (!this._options.showInterface)
            this._options.showIndicator = false;
      },

      /**
       * Инициализация плагина сканирования
       * @private
       */
      _getPlugin: function() {
         return this._getPlugins('Scanner', '1.0.0.4', {
            'createZip': this._options.createZip,
             mode: (this._options.installDlg ? 'always' : 'silent')
         });
      },

	   /**
        * @private
	    */
	   _notifyAppletReady: function(id, plugin, withAlert, error){
		   if (!this._notify('onAppletReady', id, plugin || null, error) &&
               withAlert &&
               !this._options.installDlg
           ) {
			   dFCHelpers.alert(error.message, {}, this);
		   }
       },

	   /**
       * Вызывает диалог сканера
       * @private
       */
      _scan: function(plugin) {
         var self = this;
         this._notifyAppletReady(self._randomId, plugin);
         scan(plugin).addCallback(function(result) {
            if (result) {
               upload(plugin, self._randomId).addCallback(self._post.bind(self));
            } else {
               Log(rk('Сканирование отменено'));
            }
         });
      },

      /**
       * Узнает каким сканером пользоваться
       * @private
       */
      _prepareScanner: function(plugin) {
         if (!this._pluginOpen) {
            this._pluginOpen = true;
            var self = this;
            prepareScanner(plugin).addCallbacks(
                self._scan.bind(self, plugin),
                self._notifyAppletReady.bind(self, self._randomId, null, true)
            ).addBoth(function() {
               self._pluginOpen = false;
            });
         }
      },

      /**
       * Сканировать изображение и залить его на сервер
       * @returns {Core/Deferred}
       */
      scan: function() {
         var self = this;
         return self._getPlugin().addCallbacks(
             self._prepareScanner.bind(self),
             self._notifyAppletReady.bind(self, self._randomId, null, false)
         );
      },

      //////////////////////////////////////////
      /// Setters
      //////////////////////////////////////////
      setCreateZip: function(createZip) {
         this._options.createZip = !!createZip;
      },

      //////////////////////////////////////////
      /// Getters
      //////////////////////////////////////////
      getCreateZip: function() {
         return this._options.createZip;
      }
   });

   return FileScanLoader;
});