define("Lib/File/Attach/Container/GetterContainer", ["require", "exports", "Core/Deferred"], function (require, exports, Deferred) {
    "use strict";
    /**
     * Контейнер для работы с различными реализациями IResourceGetter
     * @class Lib/File/Attach/Container/GetterContainer
     * @private
     * @see Lib/File/ResourceGetter/IResourceGetterBase
     * @author Заляев А.В.
     */
    var GetterContainer = /** @class */ (function () {
        function GetterContainer() {
            this._getters = Object.create(null);
        }
        /**
         * Регестрирует источник ресурсов
         * @param {IResourceGetter} getter
         * @return {String} Имя модуля
         * @see Lib/File/ResourceGetter/IResourceGetterBase
         */
        GetterContainer.prototype.push = function (getter) {
            this._getters[getter.getType()] = getter;
        };
        /**
         * Метод асинхронного получения экземпляра IResourceGetter
         * @param name
         * @return {Deferred<IResourceGetter>}
         * @see Lib/File/ResourceGetter/IResourceGetterBase
         */
        GetterContainer.prototype.get = function (name) {
            if (this._getters[name]) {
                return Deferred.success(this._getters[name]);
            }
            return Deferred.fail("ResourceGetter <" + name + "> is not registered");
        };
        /**
         * Зарегестрирован ли источник ресурсов в контейнере
         * @param {String} name Имя источника
         * @return {Boolean}
         */
        GetterContainer.prototype.has = function (name) {
            return !!this._getters[name];
        };
        return GetterContainer;
    }());
    return GetterContainer;
});
