/// <amd-module name="Lib/File/Attach/Container/GetterContainer" />
// dependency for types
import {HashMap} from "lib/File/Interface";
import IContainer = require("Lib/File/Attach/Container/IContainer");
import IResourceGetter = require("Lib/File/ResourceGetter/IResourceGetter");
// real dependency
import Deferred = require("Core/Deferred");

/**
 * Контейнер для работы с различными реализациями IResourceGetter
 * @class Lib/File/Attach/Container/GetterContainer
 * @private
 * @see Lib/File/ResourceGetter/IResourceGetterBase
 * @author Заляев А.В.
 */
class GetterContainer implements IContainer<IResourceGetter> /** @lends Lib/File/Attach/Container/GetterContainer.prototype*/{
    /**
     * Экземпляры IResourceGetterBase
     * @private
     */
    private _getters: HashMap<IResourceGetter>;
    
    constructor() {
        this._getters = Object.create(null);
    }
    
    /**
     * Регестрирует источник ресурсов
     * @param {IResourceGetter} getter
     * @return {String} Имя модуля
     * @see Lib/File/ResourceGetter/IResourceGetterBase
     */
    push(getter: IResourceGetter) {
        this._getters[getter.getType()] = getter;
    }
    
    /**
     * Метод асинхронного получения экземпляра IResourceGetter
     * @param name
     * @return {Deferred<IResourceGetter>}
     * @see Lib/File/ResourceGetter/IResourceGetterBase
     */
    get(name: string): Deferred<IResourceGetter> {
        if (this._getters[name]) {
            return Deferred.success(this._getters[name]);
        }
        return Deferred.fail(`ResourceGetter <${name}> is not registered`);
    }
    
    /**
     * Зарегестрирован ли источник ресурсов в контейнере
     * @param {String} name Имя источника
     * @return {Boolean}
     */
    has(name: string): boolean {
        return !!this._getters[name]
    }
}

export = GetterContainer;