define("Lib/File/Attach/Container/GetterContainerLazy", ["require", "exports", "tslib", "Core/Deferred", "Core/moduleStubs", "Lib/File/Attach/Container/GetterContainer"], function (require, exports, tslib_1, Deferred, moduleStubs, GetterContainer) {
    "use strict";
    /**
     * Контейнер для работы с различными реализациями IResourceGetter, позволяющий работать посредством ленивой подгрузки
     * @class Lib/File/Attach/Container/GetterContainerLazy
     * @private
     * @see Lib/File/ResourceGetter/IResourceGetterBase
     * @author Заляев А.В.
     */
    var GetterContainerLazy = /** @class */ (function (_super) {
        tslib_1.__extends(GetterContainerLazy, _super); /** @lends Lib/File/Attach/Container/GetterContainerLazy.prototype*/
        function GetterContainerLazy() {
            var _this = _super.call(this) || this;
            _this._links = Object.create(null);
            _this._options = Object.create(null);
            return _this;
        }
        /**
         * Метод асинхронного получения экземпляра IResourceGetter
         * @param name
         * @return {Deferred<IResourceGetter>}
         * @see Lib/File/ResourceGetter/IResourceGetterBase
         */
        GetterContainerLazy.prototype.get = function (name) {
            var _this = this;
            return _super.prototype.get.call(this, name).addErrback(function (error) {
                if (!_this._links[name]) {
                    return error;
                }
                /*
                 * Загружаем Модуль через optional! т.к. не все проекты включают в себя модули из репозитория СБИС Плагина
                 */
                return moduleStubs.require("optional!" + _this._links[name]).addCallback(function (modules) {
                    var ResourceGetter = modules[0];
                    delete _this._links[name];
                    if (!ResourceGetter) {
                        return Deferred.fail("ResourceGetter \"" + name + "\" is not supported in this project");
                    }
                    var getter = new ResourceGetter(_this._options[name]);
                    _this.push(getter);
                    return getter;
                });
            });
        };
        /**
         * Зарегестрирован ли источник ресурсов в контейнере
         * @param {String} name Имя источника
         * @return {Boolean}
         * @see Lib/File/ResourceGetter/IResourceGetterBase
         */
        GetterContainerLazy.prototype.has = function (name) {
            return _super.prototype.has.call(this, name) || !!this._links[name];
        };
        /**
         * Регистрация ссылки для последующей ленивой загрузки
         * @param {String} name Имя модуля
         * @param {String} link Ссылка на модуль
         * @param {} options Параметры для конструирования
         * @see Lib/File/ResourceGetter/IResourceGetterBase
         */
        GetterContainerLazy.prototype.register = function (name, link, options) {
            this._links[name] = link;
            this._options[name] = options;
        };
        return GetterContainerLazy;
    }(GetterContainer /** @lends Lib/File/Attach/Container/GetterContainerLazy.prototype*/));
    return GetterContainerLazy;
});
