define("Lib/File/Attach/LazyAttach", ["require", "exports", "Lib/File/Attach/BaseAttach", "Core/core-simpleExtend", "Lib/File/Attach/Container/GetterContainerLazy", "Lib/File/Attach/Container/SourceContainerLazy"], function (require, exports, BaseAttach, CoreExtend, GetterContainerLazy, SourceContainerLazy) {
    "use strict";
    /**
     * Класс, наследник BaseAttach, позволяющий регестрировать "ленивые" IResourceGetter и ISource
     * @public
     * @class Lib/File/Attach/LazyAttach
     * @extends Lib/File/Attach/BaseAttach
     * @author Заляев А.В.
     */
    var LazyAttach = CoreExtend.extend(BaseAttach, /** @lends Lib/File/Attach/LazyAttach.prototype*/ {
        constructor: function () {
            LazyAttach.superclass.constructor.apply(this, arguments);
            this._getterContainer = new GetterContainerLazy();
            this._sourceContainer = new SourceContainerLazy();
        },
        /**
         * Ленивая регестрация экземпляров IResourceGetterBase, для получения файлов
         * @param {String} name Имя модуля
         * @param {String} link Сылка на модуль
         * @param {*} [options] Параметры вызова конструктора
         * @void
         */
        registerLazyGetter: function (name, link, options) {
            return this._getterContainer.register(name, link, options);
        },
        /**
         * Ленивая регестрация ISource
         * @param {Function} fileType Конструктор обёртки над ресурсом
         * @param {String} link Ссылка на источник данных
         * @param {*} [options] Параметры вызова конструктора обёртки
         * @void
         */
        registerLazySource: function (fileType, link, options) {
            return this._sourceContainer.register(fileType, link, options);
        }
    });
    return LazyAttach;
});
