/// <amd-module name="Lib/File/Attach/LazyAttach" />
import BaseAttach = require("Lib/File/Attach/BaseAttach");
import CoreExtend = require('Core/core-simpleExtend');
import GetterContainerLazy = require("Lib/File/Attach/Container/GetterContainerLazy");
import SourceContainerLazy = require("Lib/File/Attach/Container/SourceContainerLazy");

/**
 * Класс, наследник BaseAttach, позволяющий регестрировать "ленивые" IResourceGetter и ISource
 * @public
 * @class Lib/File/Attach/LazyAttach
 * @extends Lib/File/Attach/BaseAttach
 * @author Заляев А.В.
 */
let LazyAttach  = CoreExtend.extend(BaseAttach, /** @lends Lib/File/Attach/LazyAttach.prototype*/ {
    constructor(){
        LazyAttach.superclass.constructor.apply(this, arguments);
        this._getterContainer = new GetterContainerLazy();
        this._sourceContainer = new SourceContainerLazy();
    },
    /**
     * Ленивая регестрация экземпляров IResourceGetterBase, для получения файлов
     * @param {String} name Имя модуля
     * @param {String} link Сылка на модуль
     * @param {*} [options] Параметры вызова конструктора
     * @void
     */
    registerLazyGetter(name: string, link: string, options?) {
        return this._getterContainer.register(name, link, options);
    },
    /**
     * Ленивая регестрация ISource
     * @param {Function} fileType Конструктор обёртки над ресурсом
     * @param {String} link Ссылка на источник данных
     * @param {*} [options] Параметры вызова конструктора обёртки
     * @void
     */
    registerLazySource(fileType: Function, link: string, options?: any) {
        return this._sourceContainer.register(fileType, link, options);
    }
});
export = LazyAttach;
