/// <amd-module name="Lib/File/Attach/Option/Source" />
define("Lib/File/Attach/Option/Source", ["require", "exports"], function (require, exports) {
    "use strict";
    /**
     * Класс конфигурации ISource, передаваемый в SbisAttach
     * @class
     * @name Lib/File/Attach/Option/Source
     * @public
     * @see Lib/File/Attach/SbisAttach
     * @author Заляев А.В.
     */
    var SourceOption /** @lends Lib/File/Attach/Option/Source.prototype*/ = /** @class */ (function () {
        /**
         * @cfg {String} source Ссылка на модуль источника данных ISource
         * @see WS.Data/Source/ISource
         */
        /**
         * @cfg {FunctionConstructor} resourceType Конструктор обёртки над ресурсом
         * @see Lib/File/LocalFile
         * @see Lib/File/LocalFileLink
         * @see Lib/File/HttpFileLink
         */
        /**
         * @cfg {Object} options Объект параметров для конструктора ISource
         * @see WS.Data/Source/ISource
         */
        function SourceOption(source, resourceType, options) {
            this.source = source;
            this.resourceType = resourceType;
            this.options = options;
        }
        /**
         * Возвращает ссылку на модуль с регестрируемым источником данных
         * @return {string}
         */
        SourceOption.prototype.getSourceName = function () {
            return this.source;
        };
        /**
         * Параметры вызова конструктора
         * @return {*}
         */
        SourceOption.prototype.getOptions = function () {
            return this.options;
        };
        /**
         * Возвращает конструктор ресурса, для которого будет зарегестрирован источник
         * @return {IFileDataConstructor}
         */
        SourceOption.prototype.getResourceType = function () {
            return this.resourceType;
        };
        return SourceOption;
    }());
    return SourceOption;
});
