/// <amd-module name="Lib/File/Attach/Option/Source" />

import {IFileDataConstructor}  from "Lib/File/Interface";
/**
 * Класс конфигурации ISource, передаваемый в SbisAttach
 * @class
 * @name Lib/File/Attach/Option/Source
 * @public
 * @see Lib/File/Attach/SbisAttach
 * @author Заляев А.В.
 */
class SourceOption /** @lends Lib/File/Attach/Option/Source.prototype*/{
    /**
     * @cfg {String} source Ссылка на модуль источника данных ISource
     * @see WS.Data/Source/ISource
     */
    /**
     * @cfg {FunctionConstructor} resourceType Конструктор обёртки над ресурсом
     * @see Lib/File/LocalFile
     * @see Lib/File/LocalFileLink
     * @see Lib/File/HttpFileLink
     */
    /**
     * @cfg {Object} options Объект параметров для конструктора ISource
     * @see WS.Data/Source/ISource
     */
    constructor(
        protected source: string,
        protected resourceType: IFileDataConstructor,
        protected options: any
    ){}
    /**
     * Возвращает ссылку на модуль с регестрируемым источником данных
     * @return {string}
     */
    getSourceName(): string {
        return this.source;
    }
    /**
     * Параметры вызова конструктора
     * @return {*}
     */
    getOptions(): any {
        return this.options;
    }
    /**
     * Возвращает конструктор ресурса, для которого будет зарегестрирован источник
     * @return {IFileDataConstructor}
     */
    getResourceType(): IFileDataConstructor {
        return this.resourceType;
    }
}
export = SourceOption;