/// <amd-module name="Lib/File/Attach/Option/Sources/BLPlugin" />

import SourceOption = require("Lib/File/Attach/Option/Source");
import LocalFileLink = require("Lib/File/LocalFileLink");

const LINK = "js!SBIS3.Plugin/components/Extensions/Integration/Source/BL_SbisPluginSource";

/**
 * @typedef {Object} Endpoint
 * @property {String} contract Контракт - определяет доступные операции
 * @property {String} [address] Адрес - указывает место расположения сервиса, к которому будет осуществлено подключение
 * @property {String} [moveContract=ПорядковыйНомер] Название объекта бл в которому принадлежат методы перемещения
 */

/**
 * @typedef {Object} Binding
 * @property {String} [create]
 * @property {String} [read]
 * @property {String} [update]
 * @property {String} [query]
 * @property {String} [move]
 * @property {String} [copy]
 * @property {String} [merge]
 * @property {String} [destroy]
 */

/**
 * Класс конфигурации ISource для загрузки на Бизнес-Логику через СБИС Плагин
 * @class
 * @name Lib/File/Attach/Option/Sources/BLPlugin
 * @public
 * @author Заляев А.В.
 */
class BLPlugin extends SourceOption /** @lends Lib/File/Attach/Option/Sources/BLPlugin.prototype*/{
    /**
     * @cfg {Endpoint|String} endpoint Конечная точка, обеспечивающая доступ клиента к функциональным возможностям источника данных.
     */
    /**
     * @cfg {Binding} binding Соответствие методов CRUD+ методам БЛ. Определяет, какой метод объекта БЛ соответствует каждому методу CRUD+.
     */
    /**
     * @param {*} [options] Параметры вызова конструктора
     * @constructor
     * @see Lib/File/LocalFileLink
     * @see WS.Data/Source/ISource
     */
    constructor (options?: any) {
        super (LINK, LocalFileLink, options || {});
    }
}
export = BLPlugin;