/// <amd-module name="Lib/File/Attach/Option/Sources/Fabric" />
define("Lib/File/Attach/Option/Sources/Fabric", ["require", "exports", "Lib/File/Attach/Option/Sources/FileTransferPlugin", "Lib/File/Attach/Option/Sources/SbisDiskPlugin", "Lib/File/Attach/Option/Sources/SbisDiskXHR", "Lib/File/Attach/Option/Sources/BLPlugin", "Lib/File/Attach/Option/Sources/BLXHR"], function (require, exports, FileTransferPlugin, SbisDiskPlugin, SbisDiskXHR, BLPlugin, BLXHR) {
    "use strict";
    return {
        /**
         * @typedef {Object} Endpoint
         * @property {String} contract Контракт - определяет доступные операции
         * @property {String} [address] Адрес - указывает место расположения сервиса, к которому будет осуществлено подключение
         * @property {String} [moveContract=ПорядковыйНомер] Название объекта бл в которому принадлежат методы перемещения
         */
        /**
         * @typedef {Object} Binding
         * @property {String} [create]
         * @property {String} [read]
         * @property {String} [update]
         * @property {String} [query]
         * @property {String} [move]
         * @property {String} [copy]
         * @property {String} [merge]
         * @property {String} [destroy]
         */
        /**
         * @typedef sourceOptions
         * @property {Binding} binding
         * @property {Endpoint} endpoint
         * @property {String} idProperty
         */
        /**
         * @typedef {Object} sbisDiskOptions
         * @extends sourceOptions
         * @property {String} [catalog] Каталог для загрузки на сервис
         * @property {String} [serviceUrl] Адресс сервиса СБИС Диск
         */
        /**
         * Получить набор опций для загрузки в СБИС Диск
         * @param {sbisDiskOptions} opt Параметр конфигурации ISource
         * @return {Array<Lib/File/Attach/Option/Source>}
         */
        getSbisDiskSourceOptions: function (opt) {
            return [
                new SbisDiskPlugin(opt),
                new SbisDiskXHR(opt),
            ];
        },
        /**
         * Получить набор опций для загрузки на БЛ
         * @param {sourceOptions} opt Параметр конфигурации ISource
         * @return {Array<Lib/File/Attach/Option/Source>}
         */
        getBLSourceOptions: function (opt) {
            return [
                new BLXHR(opt),
                new BLPlugin(opt)
            ];
        },
        /**
         * Получить набор опций для загрузки в FileTransfer
         * @param {sourceOptions} opt Параметр конфигурации ISource
         * @return {Array<Lib/File/Attach/Option/Source>}
         */
        getFileTransferSourceOptions: function (opt) {
            return [
                new FileTransferPlugin(opt),
            ];
        }
    };
});
