/// <amd-module name="Lib/File/Attach/Option/Sources/Fabric" />

import SourceOption = require("Lib/File/Attach/Option/Source");

import FileTransferPlugin = require("Lib/File/Attach/Option/Sources/FileTransferPlugin");
import SbisDiskPlugin = require("Lib/File/Attach/Option/Sources/SbisDiskPlugin");
import SbisDiskXHR = require("Lib/File/Attach/Option/Sources/SbisDiskXHR");
import BLPlugin = require("Lib/File/Attach/Option/Sources/BLPlugin");
import BLXHR = require("Lib/File/Attach/Option/Sources/BLXHR");

interface sourceOptions {
    binding: {
        create: string
    };
    endpoint: {
        address: string,
        contract: string
    },
    idProperty: string
}

export = {
    /**
     * @typedef {Object} Endpoint
     * @property {String} contract Контракт - определяет доступные операции
     * @property {String} [address] Адрес - указывает место расположения сервиса, к которому будет осуществлено подключение
     * @property {String} [moveContract=ПорядковыйНомер] Название объекта бл в которому принадлежат методы перемещения
     */
    /**
     * @typedef {Object} Binding
     * @property {String} [create]
     * @property {String} [read]
     * @property {String} [update]
     * @property {String} [query]
     * @property {String} [move]
     * @property {String} [copy]
     * @property {String} [merge]
     * @property {String} [destroy]
     */
    
    /**
     * @typedef sourceOptions
     * @property {Binding} binding
     * @property {Endpoint} endpoint
     * @property {String} idProperty
     */
    /**
     * @typedef {Object} sbisDiskOptions
     * @extends sourceOptions
     * @property {String} [catalog] Каталог для загрузки на сервис
     * @property {String} [serviceUrl] Адресс сервиса СБИС Диск
     */
    /**
     * Получить набор опций для загрузки в СБИС Диск
     * @param {sbisDiskOptions} opt Параметр конфигурации ISource
     * @return {Array<Lib/File/Attach/Option/Source>}
     */
    getSbisDiskSourceOptions(opt: sourceOptions): Array<SourceOption> {
        return [
            new SbisDiskPlugin(opt),
            new SbisDiskXHR(opt),
        ]
    },
    /**
     * Получить набор опций для загрузки на БЛ
     * @param {sourceOptions} opt Параметр конфигурации ISource
     * @return {Array<Lib/File/Attach/Option/Source>}
     */
    getBLSourceOptions(opt: sourceOptions): Array<SourceOption> {
        return [
            new BLXHR(opt),
            new BLPlugin(opt)
        ]
    },
    /**
     * Получить набор опций для загрузки в FileTransfer
     * @param {sourceOptions} opt Параметр конфигурации ISource
     * @return {Array<Lib/File/Attach/Option/Source>}
     */
    getFileTransferSourceOptions(opt: sourceOptions): Array<SourceOption> {
        return [
            new FileTransferPlugin(opt),
            // TODO Options for upload Blob/File  from xhr
        ]
    }
}