/// <amd-module name="Lib/File/Interface" />
import Deferred = require("Core/Deferred");
import IResourceGetter = require("Lib/File/ResourceGetter/IResourceGetter");
import ISource = require("WS.Data/Source/ISource");
import Record = require("WS.Data/Entity/Record");
import SourceContainer = require("Lib/File/Attach/Container/SourceContainer");

/**
 * @author Заляев А.В.
 */
declare module Interface {
    type HashMap<T> = {
        [propName: string]: T
    };

    ///region IFile*
    /*
     * Обёртки над данными
     */
    interface IFileData {
        getMeta(): any;
    }
    interface IFileDataConstructor {
        new(...args): IFileData;
    }
    
    interface ILocalFile extends IFileData {
        getData(): File | Blob;
    }
    interface ILocalFileConstructor {
        new(file: Blob | File, meta?: any): ILocalFile;
    }
    
    interface ILocalFileLink extends IFileData {
        getLink(): string;
    }
    interface ILocalFileLinkConstructor {
        new(fileLink: string, meta?: any): ILocalFileLink;
    }
    
    interface IHttpFileLink extends IFileData {
        getLink(): string;
    }
    interface IHttpFileLinkConstructor {
        new(fileLink: string, meta?: any): IHttpFileLink;
    }
    ///endregion IFile*
    
    ///region File Getter
    interface IResourceGetterConstructor {
        new(...args: any[]): IResourceGetter
    }
    ///endregion File Getter

    ///region FileSource
    type SourceMeta = {
    }
    type CreateHandler = (data: any, file: IFileData) => void;
    type CreateHandlerName = "onLoaded" | "onProgress" | "onWarning" | "onLoadedFolder";
    type CreateHandlers = {
        [propName in CreateHandlerName]?: CreateHandler;
    }
    
    interface IFileSource extends ISource {
        create(meta: SourceMeta): Deferred<Record>
    }
    interface ISbisFileSource extends IFileSource {
        create(meta: SourceMeta, handlers?: CreateHandlers): Deferred<Record>
    }
    ///endregion FileSource

    ///region Uploader
    interface IUploader {
        upload(files: IFileData[], meta?: any, handlers?: CreateHandlers): Deferred<Array<Record | Error>>
    }
    interface IUploaderConstructor {
        new (container: SourceContainer, fileProperty: string): IUpload;
    }
    ///endregion Uploader

    interface IAttacher {
        registerGetter(ResourceGetter: IResourceGetter): void;
        registerSource(type: IFileDataConstructor, source: ISource, fileProperty?: string): void;
    }
    interface IDirectInsertFile {
        setSelectedFiles(files: IFileData | IFileData[]): void;
        clearSelectedFiles(): void;
        getSelectedFiles(): IFileData[];
    }
    
    interface IUpload {
        upload(meta?: Object): Deferred<Array<Record | Error>>
    }
    interface BaseAttach extends IAttacher, IDirectInsertFile, IUpload {
        choose(...args): Deferred<IFileData[]>;
    }
    interface LazyAttach extends BaseAttach {
        registerLazyGetter(name: string, link: string, options?);
        registerLazySource(fileType: IResourceGetterConstructor, link: string, options?: any);
    }

    interface SourceOption {
        getSourceName(): string;
        getOptions(): any;
        getResourceType(): IFileDataConstructor;
    }

    interface ResourceGetterOption {
        getName(): string;
        getOptions(): any;
        getGetter(): string | IResourceGetter;
    }

    type Options = {
        sourceOptions: SourceOption[];
        getterOptions: ResourceGetterOption[];
        attachOptions: {
            fileProperty: string;
            multiselect: boolean;
        }
        [propName: string]: any;
    }
    interface SbisAttach extends IUpload, IDirectInsertFile {
        constructor(options: Options);
        choose(getterName: string): Deferred<IFileData[]>;
    }
}
export = Interface;