/// <amd-module name="Lib/File/utils/ExtensionsError" />

const DEFAULT_MESSAGE = rk('Выбранный файл неверного типа');
/**
 * @cfg {String} Имя файла, вызвавшего ошибку
 * @name Lib/File/utils/ExtensionsError#fileName
 */

/**
 * Ошибка несоответствия типа выбранного файла
 * @class
 * @name Lib/File/utils/ExtensionsError
 * @public
 * @extends Error
 */
class ExtensionsError extends Error {
    public message: string;
    public fileName: string;
    constructor(message: string, fileName?: string) {
        super();
        /*
         * Error реализован не как нормальный конструктор
         *
         * через наследование core/classic-extend
         * теряем .stack
         *
         *
         * без Object.setPrototypeOf(this, Class.prototype) получим поведение:
         * let e = new ExtensionsError("1.png");
         * e instanceof Error; // true
         * e instanceof ExtensionsError; // false
         */
        Object.setPrototypeOf(this, ExtensionsError.prototype);
        if (!fileName) {
            fileName = message;
            message = DEFAULT_MESSAGE;
        }
        this.message = message;
        this.fileName = fileName;
    }
}

export = ExtensionsError;
