/// <amd-module name="Lib/ServerEvent/DebugUtils" />
"use strict";
import moduleStubs = require('Core/moduleStubs');

function addSharedBusLog() {
    if (typeof Promise !== "function") {
        return;
    }
    let beautyLog = (item) => {
        let channel = '';
        if (item["headers"] && item["headers"]["event-type"]) {
            channel = item["headers"]["event-type"];
        }
        console.log(
            '[EventLog]',
            new Date(item.timestamp).toLocaleString(),
            channel,
            item);
    };
    let show = function (reader, filter) {
        reader.query(filter).then((d) => {
            d.forEach(beautyLog);
        });
    };

    /**
     * @type {Lib/ServerEvent/native/EventStorage/Reader}
     */
    let reader;
    /**
     * Функция отображения собранны серверных событий
     * TODO использовать COnnectorIE и обрабатывать ошибки прилетающие от Connector::connect()
     * @param {string | RegExp} filter
     */
    window["sharedBusLog"] = function (filter: string | RegExp) {
        if (reader) {
            show(reader, filter);
            return;
        }
        moduleStubs.require(["Lib/ServerEvent/native/EventStorage/Connector"])
            .addCallback(([Connector]) => {
                Connector.connect().then((connect) => {
                    reader = connect.getReader();
                    show(reader, filter);
                });
            });
    };
}

let watcherEnable = false;
function addSharedBusWatcher() {
    if (watcherEnable) {
        return;
    }
    watcherEnable = true;
    moduleStubs.require(["Lib/ServerEvent/Bus", "Lib/ServerEvent/class/logger/ConsoleWatchDog"])
        .addCallback(([seb, wd]) => {
            seb["addWatchDog"](new wd());
        });
}

export = {
    attachDebugFn() {
        if (!window) {
            return;
        }
        addSharedBusLog();
        window["sharedBusWatch"] = addSharedBusWatcher;
    }
};

/**
 * Функция вывода последних 100 событий полученных браузером
 * @function window.sharedBusLog
 * @param filter {string|RegExp|null} Фильтр по названию канала
 */
