define("Lib/ServerEvent/class/Notifier", ["require", "exports", "Core/EventBus", "Lib/ServerEvent/ResponseConverter", "Lib/ServerEvent/class/logger/WatchDogAggregator"], function (require, exports, EventBus, Converter, Watcher) {
    "use strict";
    var EVENTS_CONNECT;
    (function (EVENTS_CONNECT) {
        EVENTS_CONNECT["READY"] = "onready";
        EVENTS_CONNECT["DISCONNECT"] = "ondisconnect";
    })(EVENTS_CONNECT || (EVENTS_CONNECT = {}));
    var Notifier = /** @class */ (function () {
        function Notifier(watcher) {
            this.watcher = watcher;
            this.readyStatuses = {};
            if (!watcher) {
                this.watcher = new Watcher();
            }
        }
        Notifier.prototype.setWatcher = function (watcher) {
            this.watcher = watcher;
        };
        /**
         * Уведомляем о событии
         * @param {string} channelName
         * @param {string} eventName
         * @param {string} rawData
         */
        Notifier.prototype.fire = function (channelName, eventName, rawData) {
            if (this.isSkip(channelName, eventName)) {
                return;
            }
            var data;
            if (rawData) {
                data = Converter(rawData);
            }
            this.watcher.logEvent(channelName, eventName, data);
            EventBus.channel(channelName).notify(eventName, data, rawData);
        };
        /**
         * Событие onready должно по каналу приходить только один раз. Либо после ondisconnect.
         * Транспортами это не возможно в случае с главной вкладкой.
         * @param {string} channelName
         * @param {string} eventName
         * @return {boolean} Нужно ли пропустить текущее событие onready
         */
        Notifier.prototype.isSkip = function (channelName, eventName) {
            if (eventName !== EVENTS_CONNECT.READY && eventName !== EVENTS_CONNECT.DISCONNECT) {
                return false;
            }
            if (eventName === EVENTS_CONNECT.READY && this.readyStatuses[channelName]) {
                return true;
            }
            if (eventName === EVENTS_CONNECT.READY) {
                this.readyStatuses[channelName] = true;
            }
            if (eventName === EVENTS_CONNECT.DISCONNECT) {
                this.readyStatuses[channelName] = false;
            }
            return false;
        };
        return Notifier;
    }());
    return Notifier;
});
