/// <amd-module name="Lib/ServerEvent/class/Notifier" />
import {SEB} from "../interfaces";
import EventBus = require('Core/EventBus');
import Converter = require('Lib/ServerEvent/ResponseConverter');
import Watcher = require('Lib/ServerEvent/class/logger/WatchDogAggregator');

enum EVENTS_CONNECT {
    READY = "onready",
    DISCONNECT = "ondisconnect"
}

class Notifier {
    private readyStatuses = {};
    constructor(private watcher?: SEB.IWatchDog) {
        if (!watcher) {
            this.watcher = new Watcher();
        }
    }

    setWatcher(watcher: SEB.IWatchDog) {
        this.watcher = watcher;
    }

    /**
     * Уведомляем о событии
     * @param {string} channelName
     * @param {string} eventName
     * @param {string} rawData
     */
    fire(channelName: string, eventName: string, rawData?: string) {
        if (this.isSkip(channelName, eventName) ) {
            return;
        }
        let data;
        if (rawData) {
            data = Converter(rawData);
        }

        this.watcher.logEvent(channelName, eventName, data);
        EventBus.channel(channelName).notify(eventName, data, rawData);
    }

    /**
     * Событие onready должно по каналу приходить только один раз. Либо после ondisconnect.
     * Транспортами это не возможно в случае с главной вкладкой.
     * @param {string} channelName
     * @param {string} eventName
     * @return {boolean} Нужно ли пропустить текущее событие onready
     */
    private isSkip(channelName: string, eventName: string): boolean {
        if (eventName !== EVENTS_CONNECT.READY && eventName !== EVENTS_CONNECT.DISCONNECT) {
            return false;
        }
        if (eventName === EVENTS_CONNECT.READY && this.readyStatuses[channelName]) {
            return true;
        }
        if (eventName === EVENTS_CONNECT.READY) {
            this.readyStatuses[channelName] = true;
        }
        if (eventName === EVENTS_CONNECT.DISCONNECT) {
            this.readyStatuses[channelName] = false;
        }
        return false;
    }
}

export = Notifier;