define("Lib/ServerEvent/class/creator/CreatorExclusive", ["require", "exports", "Core/Deferred", "Core/detection", "Core/UserInfo", "Core/moduleStubs"], function (require, exports, Deferred, detection, UserInfo, moduleStubs) {
    "use strict";
    /**
     * @class Lib/ServerEvent/class/creator/CreatorExclusive
     * @memberOf module:ServerEvent.class.creator
     */
    var CreatorExclusive = /** @class */ (function () {
        /**
         * @param {Lib/ServerEvent/class/ConnectOptions} connectOptions
         */
        function CreatorExclusive(connectOptions) {
            this.connectOptions = connectOptions;
            var userName = UserInfo.get('ЛогинПользователя') || '';
            this.isGuestAccess = userName.indexOf('__сбис__гость__') > -1;
        }
        CreatorExclusive.prototype.getWebSocket = function (url) {
            return moduleStubs.require("Lib/ServerEvent/native/SockjsEmulator").addCallback(function (_a) {
                var SockJSTransport = _a[0];
                return new SockJSTransport(url, undefined, true);
            }).createDependent();
        };
        CreatorExclusive.prototype.getResources = function () {
            return moduleStubs.require("Lib/ServerEvent/resources/stomp").createDependent();
        };
        CreatorExclusive.prototype.isAvailableInEnv = function (isExclusive) {
            var isMobile = detection.isMobileAndroid || detection.isMobileSafari;
            return isExclusive || isMobile || detection.safari11 || this.connectOptions.isDesktop();
        };
        CreatorExclusive.prototype.build = function (hash) {
            var _this = this;
            return moduleStubs.require('Lib/ServerEvent/class/transport/LocalPageTransport')
                .createDependent().addCallback(function (_a) {
                var transportClass = _a[0];
                return _this.createTransport(transportClass, hash);
            }).createDependent();
        };
        CreatorExclusive.prototype.createTransport = function (ctor, hash) {
            var _this = this;
            var def = new Deferred();
            this.getResources().addCallback(function () {
                _this.connect().addCallbacks(function (connect) {
                    def.callback(new ctor(connect, hash, false, _this.connectOptions.exchange));
                }, function (err) {
                    def.errback(err);
                });
            });
            return def;
        };
        CreatorExclusive.prototype.connect = function () {
            var def = new Deferred();
            try {
                this.getWebSocket(this.connectOptions.getWebSocketUrl()).addCallback(function (webSocket) {
                    var stomp = Stomp.over(webSocket);
                    stomp.heartbeat.outgoing = CreatorExclusive.STOMP_HEARTBEAT_OUT_TIME;
                    stomp.heartbeat.incoming = CreatorExclusive.STOMP_HEARTBEAT_IN_TIME;
                    stomp.debug = function () { };
                    stomp.connect(CreatorExclusive.LOGIN, CreatorExclusive.PASSWORD, function () {
                        def.callback(stomp);
                    }, function (data) {
                        if (def.isReady()) {
                            return console.warn(data);
                        }
                        def.errback(data);
                    });
                }).addErrback(function (e) {
                    def.errback(e);
                });
            }
            catch (e) {
                def.errback(e);
            }
            return def;
        };
        CreatorExclusive.STOMP_HEARTBEAT_OUT_TIME = 60000;
        CreatorExclusive.STOMP_HEARTBEAT_IN_TIME = 0;
        CreatorExclusive.LOGIN = 'stomp_user';
        CreatorExclusive.PASSWORD = 'stomp_user';
        return CreatorExclusive;
    }());
    return CreatorExclusive;
});
