/// <amd-module name="Lib/ServerEvent/class/creator/CreatorExclusive" />
import {SEB} from "../../interfaces";
import Deferred = require('Core/Deferred');
import detection = require('Core/detection');
import UserInfo = require('Core/UserInfo');
import moduleStubs = require('Core/moduleStubs');
import ConnectOptions = require('Lib/ServerEvent/class/ConnectOptions');

/**
 * @class Lib/ServerEvent/class/creator/CreatorExclusive
 * @memberOf module:ServerEvent.class.creator
 */
class CreatorExclusive implements SEB.ITransportCreator {
    static STOMP_HEARTBEAT_OUT_TIME = 60000;
    static STOMP_HEARTBEAT_IN_TIME = 0;
    static LOGIN = 'stomp_user';
    static PASSWORD = 'stomp_user';
    private isGuestAccess: boolean;

    /**
     * @param {Lib/ServerEvent/class/ConnectOptions} connectOptions
     */
    constructor(private connectOptions: ConnectOptions) {
        let userName = UserInfo.get('ЛогинПользователя') || '';
        this.isGuestAccess = userName.indexOf('__сбис__гость__') > -1;
    }

    protected getWebSocket(url): Deferred<WebSocket> {
        return moduleStubs.require("Lib/ServerEvent/native/SockjsEmulator").addCallback(([SockJSTransport]) => {
            return new SockJSTransport(url, undefined, true);
        }).createDependent();
    }

    protected getResources(): Deferred<any[]> {
        return moduleStubs.require("Lib/ServerEvent/resources/stomp").createDependent();
    }

    isAvailableInEnv(isExclusive: boolean) {
        let isMobile = detection.isMobileAndroid || detection.isMobileSafari;
        return isExclusive || isMobile || detection.safari11 || this.connectOptions.isDesktop();
    }

    build(hash) {
        return moduleStubs.require('Lib/ServerEvent/class/transport/LocalPageTransport')
            .createDependent().addCallback(([transportClass]) => {
            return this.createTransport(transportClass, hash);
        }).createDependent();
    }

    private createTransport(ctor: SEB.ITransportConstructor, hash): Deferred<SEB.ITrackedTransport> {
        let def = new Deferred<SEB.ITrackedTransport>();
        this.getResources().addCallback(() => {
            this.connect().addCallbacks((connect: Stomp.Client) => {
                def.callback(new ctor(
                    connect, hash,
                    false,
                    this.connectOptions.exchange
                ));
            }, (err) => {
                def.errback(err);
            });
        });
        return def;
    }

    private connect(): Deferred<Stomp.Client> {
        let def = new Deferred<Stomp.Client>();
        try {
            this.getWebSocket(this.connectOptions.getWebSocketUrl()).addCallback((webSocket) => {
                let stomp = Stomp.over(webSocket);
                stomp.heartbeat.outgoing = CreatorExclusive.STOMP_HEARTBEAT_OUT_TIME;
                stomp.heartbeat.incoming = CreatorExclusive.STOMP_HEARTBEAT_IN_TIME;
                stomp.debug = function () {};
                stomp.connect(
                    CreatorExclusive.LOGIN,
                    CreatorExclusive.PASSWORD,
                    () => {
                        def.callback(stomp);
                    },
                    (data) => {
                        if (def.isReady()) {
                            return console.warn(data);
                        }
                        def.errback(data);
                    }
                );
            }).addErrback((e) => {
                def.errback(e);
            });
        } catch (e) {
            def.errback(e);
        }

        return def;
    }
}

export = CreatorExclusive;