define("Lib/ServerEvent/class/creator/CreatorLocalWebSocket", ["require", "exports", "Core/Deferred", "Core/detection", "Core/UserInfo", "Core/moduleStubs", "Lib/Storage/LocalStorage", "Lib/ServerEvent/class/transport/Constants"], function (require, exports, Deferred, detection, UserInfo, moduleStubs, LocalStorage, CONST) {
    "use strict";
    /**
     * @class Lib/ServerEvent/class/creator/CreatorLocalWebSocket
     * @memberOf module:ServerEvent.class.creator
     */
    var CreatorLocalWebSocket = /** @class */ (function () {
        function CreatorLocalWebSocket(connectOptions) {
            this.connectOptions = connectOptions;
            this.ls = new LocalStorage('SEB');
            var userName = UserInfo.get('ЛогинПользователя') || '';
            this.isGuestAccess = userName.indexOf('__сбис__гость__') > -1;
        }
        CreatorLocalWebSocket.prototype.isAvailableInEnv = function (isExclusive) {
            if (isExclusive) {
                return false;
            }
            var isMobile = detection.isMobileAndroid || detection.isMobileIOS;
            return !isMobile && !detection.safari11 && !this.connectOptions.isDesktop();
        };
        CreatorLocalWebSocket.prototype.build = function (hash) {
            var _this = this;
            return this.chooseTransportConstructor()
                .addCallback(function (ctor) {
                return _this.createTransport(ctor, hash);
            });
        };
        CreatorLocalWebSocket.prototype.createTransport = function (ctor, hash) {
            var _this = this;
            var def = new Deferred();
            this.getResources().addCallback(function () {
                if (ctor.getLocalName() === 'LocalSlaveTransport') {
                    def.callback(new ctor());
                    return;
                }
                _this.connect().addCallbacks(function (connect) {
                    if (ctor.getLocalName() != 'LocalPageTransport' &&
                        ctor.getLocalName() != 'LocalMainTransport') {
                        return;
                    }
                    def.callback(new ctor(connect, hash, !_this.isGuestAccess, _this.connectOptions.exchange));
                }, function (err) {
                    def.errback(err);
                });
            });
            return def;
        };
        CreatorLocalWebSocket.prototype.chooseTransportConstructor = function () {
            var timestamp = this.ls.getItem(CONST.KEY_ACTUAL_DATE);
            if (!timestamp) {
                return moduleStubs.require('Lib/ServerEvent/class/transport/LocalMainTransport')
                    .createDependent().addCallback(function (_a) {
                    var transportClass = _a[0];
                    return transportClass;
                });
            }
            var last = new Date(timestamp + CONST.CHECK_MIN_INTERVAL * 2);
            if (last < new Date() || isNaN(last.valueOf())) {
                return moduleStubs.require('Lib/ServerEvent/class/transport/LocalMainTransport')
                    .createDependent().addCallback(function (_a) {
                    var transportClass = _a[0];
                    return transportClass;
                });
            }
            return moduleStubs.require('Lib/ServerEvent/class/transport/LocalSlaveTransport')
                .createDependent().addCallback(function (_a) {
                var transportClass = _a[0];
                return transportClass;
            });
        };
        CreatorLocalWebSocket.prototype.connect = function () {
            var def = new Deferred();
            try {
                this.getWebSocket(this.connectOptions.getWebSocketUrl()).addCallback(function (webSocket) {
                    var stomp = Stomp.over(webSocket);
                    stomp.heartbeat.outgoing = CreatorLocalWebSocket.STOMP_HEARTBEAT_OUT_TIME;
                    stomp.heartbeat.incoming = CreatorLocalWebSocket.STOMP_HEARTBEAT_IN_TIME;
                    stomp.debug = function () { };
                    stomp.connect(CreatorLocalWebSocket.LOGIN, CreatorLocalWebSocket.PASSWORD, function () {
                        def.callback(stomp);
                    }, function (data) {
                        if (def.isReady()) {
                            return console.warn(data);
                        }
                        def.errback(data);
                    });
                }).addErrback(function (e) {
                    def.errback(e);
                });
            }
            catch (e) {
                def.errback(e);
            }
            return def;
        };
        CreatorLocalWebSocket.STOMP_HEARTBEAT_OUT_TIME = 60000;
        CreatorLocalWebSocket.STOMP_HEARTBEAT_IN_TIME = 0;
        CreatorLocalWebSocket.LOGIN = 'stomp_user';
        CreatorLocalWebSocket.PASSWORD = 'stomp_user';
        return CreatorLocalWebSocket;
    }());
    return CreatorLocalWebSocket;
});
