/// <amd-module name="Lib/ServerEvent/class/creator/CreatorWorker" />
import {SEB} from "../../interfaces";
import Deferred = require('Core/Deferred');
import detection = require('Core/detection');
import UserInfo = require('Core/UserInfo');
import moduleStubs = require('Core/moduleStubs');
import ConnectOptions = require('Lib/ServerEvent/class/ConnectOptions');
import LocalStorageNative = require('Core/LocalStorageNative');

abstract class CreatorWorker implements SEB.ITransportCreator {
    private isGuestAccess: boolean;
    private sid: string;

    constructor(private connectOptions: ConnectOptions) {
        this.sid = connectOptions.sid;
        let userName = UserInfo.get('ЛогинПользователя') || '';
        this.isGuestAccess = userName.indexOf('__сбис__гость__') > -1;
    }

    isAvailableInEnv(isExclusive: boolean) {
        if (isExclusive) {
            return false;
        }
        let isMobile = detection.isMobileAndroid || detection.isMobileIOS || detection.isIE12;
        return !isMobile && typeof Promise != 'undefined';
    }

    build(hash) {
        let def = new Deferred<SEB.ITrackedTransport>();
        moduleStubs.require([
            'Lib/ServerEvent/worker-connect/WorkerBuilder',
            'Lib/ServerEvent/class/transport/WorkerTransport'])
            .addCallback(([Seb, WorkerTransport]) => {
            this.realBuild(hash, Seb, def, WorkerTransport);
            return Seb;
        });
        return def;
    }

    /**
     * @param hash {string}
     * @param Seb {Object}
     * @param def {Deferred}
     * @param WorkerTransport {SEB.ITrackedTransport}
     */
    private realBuild(hash: string, Seb, def: Deferred<any>, WorkerTransport) {
        let isPersist = !this.isGuestAccess;
        let isAck = false;
        if ('true' == LocalStorageNative.getItem('shared-bus-ack')) {
            isAck = true;
        }

        Seb.createConnection(
            this.getWorkerType(),
            this.connectOptions.getWebSocketUrl(),
            this.connectOptions.exchange,
            this.sid,
            hash,
            isPersist, isAck).addCallback((connect) => {
            def.callback(new WorkerTransport(connect));
        }).addErrback((e) => {
            def.errback(e);
            console.warn(e);
        });
    }

    protected abstract getWorkerType(): string;
}

export = CreatorWorker;