/// <amd-module name="Lib/ServerEvent/class/deliver/DeliveryChooser" />
import {SEB} from "../../interfaces";
import Deferred = require('Core/Deferred');
import Notifier = require('Lib/ServerEvent/class/Notifier');
import LocalPageTransport = require('Lib/ServerEvent/class/transport/LocalPageTransport');
import WorkerTransport = require('Lib/ServerEvent/class/transport/WorkerTransport');
import LSEventDeliver = require('Lib/ServerEvent/class/deliver/LSEventDeliver');
import PageEventDeliver = require('Lib/ServerEvent/class/deliver/PageEventDeliver');

/**
 * Создаю один доставщик на страницу.
 * Как бы не менялись транспорты, канал остаётся один.
 * @type {Notifier}
 */
const notifier = new Notifier();

class DeliveryChooser {
    constructor(private watcher: SEB.IWatchDog) {
        notifier.setWatcher(watcher);
    }

    choose(transport: SEB.ITransport): Deferred<SEB.IEventDeliver> {
        if (transport instanceof WorkerTransport || transport instanceof LocalPageTransport) {
            return Deferred.success(new PageEventDeliver(notifier));
        }

        return Deferred.success(new LSEventDeliver(notifier));
    }
}

export = DeliveryChooser;