/// <amd-module name="Lib/ServerEvent/class/deliver/LSEventDeliver" />
import Notifier = require('Lib/ServerEvent/class/Notifier');
import TabMessage = require('Lib/TabMessage/TabMessage');
import {SEB} from "../../interfaces";

class LSEventDeliver implements SEB.IEventDeliver {
    private static ON_MESSAGE = 'servereventbus_onmessage';
    private taber: TabMessage;

    constructor(private notifier: Notifier) {
        this.onmessageHandler = this.onmessageHandler.bind(this);
        this.taber = new TabMessage();
        this.taber.subscribe(LSEventDeliver.ON_MESSAGE, this.onmessageHandler);
    }

    private onmessageHandler(event, data: { channelName: string, eventName: string, rawData: any }) {
        this.notifier.fire(data.channelName, data.eventName, data.rawData);
    }

    deliver(channelName: string, eventName: string, rawData?: string) {
        this.notifier.fire(channelName, eventName, rawData);
        this.taber.notify(LSEventDeliver.ON_MESSAGE, {
            channelName,
            eventName,
            rawData
        });
    }

    destroy() {
        this.taber.unsubscribe(LSEventDeliver.ON_MESSAGE, this.onmessageHandler)
    }
}

export = LSEventDeliver;