define("Lib/ServerEvent/class/transport/WorkerTransport", ["require", "exports", "tslib", "Lib/ServerEvent/class/transport/Transport", "Lib/ServerEvent/class/SubscribeContainer"], function (require, exports, tslib_1, Transport, SubscribeContainer) {
    "use strict";
    var WorkerTransport = /** @class */ (function (_super) {
        tslib_1.__extends(WorkerTransport, _super);
        /**
         * @param connect {Connect}
         */
        function WorkerTransport(connect) {
            var _this = _super.call(this) || this;
            _this.connect = connect;
            _this.subscribes = new SubscribeContainer();
            connect.onmessage = _this.messageHandler;
            _this.unloadWindow = _this.unloadWindow.bind(_this);
            window.addEventListener('unload', _this.unloadWindow);
            return _this;
        }
        WorkerTransport.getLocalName = function () {
            return 'WorkerTransport';
        };
        /**
         * TODO Нужно проверять, на то что подписка уже зарегестрирована
         * @param {SEB.ISubscribe} subscribe
         */
        WorkerTransport.prototype.subscribe = function (subscribe) {
            this.subscribes.add(subscribe);
            if (!subscribe.isChanneled()) {
                this.connect.subscribe(subscribe.getChannelName());
            }
            else {
                this.connect.subscribeChanneled(subscribe.getChannelName(), subscribe.getTarget());
            }
            this.delivery.deliver(subscribe.getChannelName(), 'onready');
        };
        /**
         * @param {SEB.ISubscribe} subscribe
         */
        WorkerTransport.prototype.unsubscribe = function (subscribe) {
            this.subscribes.remove(subscribe);
            //TODO закладываюсь, что нет канализированных и не канализированных с одним именем
            if (this.subscribes.has(subscribe)) {
                return;
            }
            this.delivery.deliver(subscribe.getChannelName(), 'ondisconnect');
            if (!subscribe.isChanneled()) {
                return;
            }
            this.connect.unsubscribeChanneled(subscribe.getChannelName(), subscribe.getTarget());
        };
        WorkerTransport.prototype.destructor = function (isFireCallback) {
            if (isFireCallback === void 0) { isFireCallback = true; }
            window.removeEventListener('unload', this.unloadWindow);
            this.callDisconnect();
            this.connect.destructor();
            var callback = this.disconnectCallback;
            this.disconnectCallback = function () { };
            _super.prototype.destructor.call(this);
            if (isFireCallback) {
                callback();
            }
        };
        WorkerTransport.prototype.setDisconnectHandler = function (fn) {
            var _this = this;
            this.disconnectCallback = fn;
            this.connect.onwebsocketclose = function (e) {
                _this.destructor();
            };
        };
        WorkerTransport.prototype.unloadWindow = function () {
            this.close(false);
        };
        WorkerTransport.prototype.close = function (isFireCallback) {
            if (isFireCallback === void 0) { isFireCallback = true; }
            for (var _i = 0, _a = this.subscribes.all(); _i < _a.length; _i++) {
                var subscribe = _a[_i];
                this.unsubscribe(subscribe);
            }
            this.destructor(isFireCallback);
            this.connect.onwebsocketclose = function () { };
        };
        WorkerTransport.prototype.callDisconnect = function () {
            var _this = this;
            var deliver = function (subscribe) {
                _this.delivery.deliver(subscribe.getChannelName(), 'ondisconnect');
            };
            this.subscribes.all().forEach(deliver);
        };
        return WorkerTransport;
    }(Transport));
    return WorkerTransport;
});
