/// <amd-module name="Lib/ServerEvent/class/transport/WorkerTransport" />
import Transport = require('Lib/ServerEvent/class/transport/Transport');
import SubscribeContainer = require('Lib/ServerEvent/class/SubscribeContainer');
import {SEB} from "../../interfaces";

class WorkerTransport extends Transport {
    private subscribes = new SubscribeContainer();

    /**
     * Необходима, что бы при обаружении смены мастера, вызвать пересоздание
     */
    private disconnectCallback: Function;

    /**
     * @param connect {Connect}
     */
    constructor(public connect) {
        super();
        connect.onmessage = this.messageHandler;
        this.unloadWindow = this.unloadWindow.bind(this);
        window.addEventListener('unload', this.unloadWindow);
    }

    static getLocalName() {
        return 'WorkerTransport';
    }

    /**
     * TODO Нужно проверять, на то что подписка уже зарегестрирована
     * @param {SEB.ISubscribe} subscribe
     */
    subscribe(subscribe: SEB.ISubscribe) {
        this.subscribes.add(subscribe);

        if (!subscribe.isChanneled()) {
            this.connect.subscribe(subscribe.getChannelName());
        } else {
            this.connect.subscribeChanneled(
                subscribe.getChannelName(),
                (subscribe as SEB.IChanneledSubscribe).getTarget()
            );
        }
        this.delivery.deliver(subscribe.getChannelName(), 'onready');
    }

    /**
     * @param {SEB.ISubscribe} subscribe
     */
    unsubscribe(subscribe: SEB.ISubscribe) {
        this.subscribes.remove(subscribe);
        //TODO закладываюсь, что нет канализированных и не канализированных с одним именем
        if (this.subscribes.has(subscribe)) {
            return;
        }
        this.delivery.deliver(subscribe.getChannelName(), 'ondisconnect');
        if (!subscribe.isChanneled()) {
            return;
        }
        this.connect.unsubscribeChanneled(subscribe.getChannelName(),
            (subscribe as SEB.IChanneledSubscribe).getTarget());
    }

    protected destructor(isFireCallback:boolean = true) {
        window.removeEventListener('unload', this.unloadWindow);
        this.callDisconnect();
        this.connect.destructor();
        let callback = this.disconnectCallback;
        this.disconnectCallback = () => {};
        super.destructor();
        if (isFireCallback) {
            callback();
        }
    }

    setDisconnectHandler(fn) {
        this.disconnectCallback = fn;
        this.connect.onwebsocketclose = (e) => {
            this.destructor();
        };
    }

    private unloadWindow() {
        this.close(false);
    }

    close(isFireCallback: boolean=true) {
        for (let subscribe of this.subscribes.all()) {
            this.unsubscribe(subscribe);
        }
        this.destructor(isFireCallback);
        this.connect.onwebsocketclose = () => {};
    }

    private callDisconnect() {
        let deliver = (subscribe: SEB.ISubscribe) => {
            this.delivery.deliver(subscribe.getChannelName(), 'ondisconnect');
        };
        this.subscribes.all().forEach(deliver);
    }
}

export = WorkerTransport;