define("Lib/ServerEvent/native/EventStorage/Connector", ["require", "exports", "Lib/ServerEvent/native/EventStorage/Writer", "Lib/ServerEvent/native/EventStorage/Reader"], function (require, exports, Writer, Reader) {
    "use strict";
    /**
     * @class Lib/ServerEvent/native/EventStorage/Connector
     * @memberOf module:ServerEvent.native.EventStorage
     */
    var Connector = /** @class */ (function () {
        function Connector(db) {
            this.db = db;
        }
        Connector.connect = function () {
            return new Promise(function (resolve, reject) {
                var request;
                try {
                    request = indexedDB.open(Connector.DB_NAME, 1);
                }
                catch (e) {
                    console.warn(e);
                    reject(e);
                    return;
                }
                request.onupgradeneeded = function (event) {
                    var db = this.result;
                    db.onerror = function (event) {
                        console.error("Database error: " + event.target.errorCode);
                    };
                    new Writer(db, Connector.STORE_NAME).createStore();
                };
                request.onsuccess = function (event) {
                    var db = this.result;
                    resolve(new Connector(db));
                };
                request.onerror = function (event) {
                    console.warn("Почему Вы не позволяете online.sbis.ru использовать IndexedDB?!");
                    reject("You haven't rights to use IndexDB.");
                    /**
                     * https://github.com/localForage/localForage/issues/195#issuecomment-185135074
                     */
                    return true;
                };
            });
        };
        /**
         * @return {Lib/ServerEvent/native/EventStorage/Writer}
         */
        Connector.prototype.getWriter = function () {
            return new Writer(this.db, Connector.STORE_NAME);
        };
        /**
         * @return {Lib/ServerEvent/native/EventStorage/Reader}
         */
        Connector.prototype.getReader = function () {
            return new Reader(this.db, Connector.STORE_NAME);
        };
        Connector.DB_NAME = "sbis.seb";
        Connector.STORE_NAME = "events";
        return Connector;
    }());
    return Connector;
});
