/// <amd-module name="Lib/ServerEvent/native/EventStorage/Connector" />
import Writer = require('Lib/ServerEvent/native/EventStorage/Writer');
import Reader = require('Lib/ServerEvent/native/EventStorage/Reader');

/**
 * @class Lib/ServerEvent/native/EventStorage/Connector
 * @memberOf module:ServerEvent.native.EventStorage
 */
class Connector {
    static DB_NAME = "sbis.seb";
    static STORE_NAME = "events";

    static connect(): Promise<Connector> {
        return new Promise((resolve, reject) => {
            let request;
            try {
                request = indexedDB.open(Connector.DB_NAME, 1);
            } catch (e) {
                console.warn(e);
                reject(e);
                return;
            }
            request.onupgradeneeded = function (event) {
                let db = this.result;
                db.onerror = (event) => {
                    console.error("Database error: " + event.target.errorCode);
                };
                new Writer(db, Connector.STORE_NAME).createStore();
            };
            request.onsuccess = function (event) {
                let db = this.result;
                resolve(new Connector(db));
            };
            request.onerror = function (event) {
                console.warn("Почему Вы не позволяете online.sbis.ru использовать IndexedDB?!");
                reject("You haven't rights to use IndexDB.");
                /**
                 * https://github.com/localForage/localForage/issues/195#issuecomment-185135074
                 */
                return true;
            };
        });
    }

    constructor(private db: IDBDatabase) {}

    /**
     * @return {Lib/ServerEvent/native/EventStorage/Writer}
     */
    getWriter(): Writer {
        return new Writer(this.db, Connector.STORE_NAME);
    }

    /**
     * @return {Lib/ServerEvent/native/EventStorage/Reader}
     */
    getReader(): Reader {
        return new Reader(this.db, Connector.STORE_NAME);
    }
}

export = Connector;