define("Lib/ServerEvent/native/EventStorage/ConnectorIE", ["require", "exports", "Lib/ServerEvent/native/EventStorage/Writer", "Lib/ServerEvent/native/EventStorage/Reader", "Core/Deferred"], function (require, exports, Writer, Reader, Deferred) {
    "use strict";
    /**
     * Создан для соединения по IE
     * @class Lib/ServerEvent/native/EventStorage/ConnectorIE
     */
    var ConnectorIE = /** @class */ (function () {
        function ConnectorIE(db) {
            this.db = db;
        }
        /**
         * @param {Core/Deferred} def
         * @return {Core/Deferred<ConnectorIE>}
         */
        ConnectorIE.connect = function () {
            var def = new Deferred();
            if (typeof indexedDB !== 'object') {
                return Deferred.fail('В окружении нет indexedDB');
            }
            var request;
            try {
                request = indexedDB.open(ConnectorIE.DB_NAME, 1);
            }
            catch (e) {
                console.warn(e);
                def.errback(e);
                return def;
            }
            request.onupgradeneeded = function (event) {
                var db = this.result;
                db.onerror = function (event) {
                    console.error("Database error: " + event.target.errorCode);
                };
                new Writer(db, ConnectorIE.STORE_NAME).createStore();
            };
            request.onsuccess = function (event) {
                var db = this.result;
                def.callback(new ConnectorIE(db));
            };
            request.onerror = function (event) {
                console.warn("Почему Вы не позволяете online.sbis.ru использовать IndexedDB?!");
                def.errback("You haven't rights to use IndexDB.");
                /**
                 * https://github.com/localForage/localForage/issues/195#issuecomment-185135074
                 */
                return true;
            };
            return def;
        };
        /**
         * @return {Lib/ServerEvent/native/EventStorage/Writer}
         */
        ConnectorIE.prototype.getWriter = function () {
            return new Writer(this.db, ConnectorIE.STORE_NAME);
        };
        /**
         * @return {Lib/ServerEvent/native/EventStorage/Reader}
         */
        ConnectorIE.prototype.getReader = function () {
            return new Reader(this.db, ConnectorIE.STORE_NAME);
        };
        ConnectorIE.DB_NAME = "sbis.seb";
        ConnectorIE.STORE_NAME = "events";
        return ConnectorIE;
    }());
    return ConnectorIE;
});
