/// <amd-module name="Lib/ServerEvent/native/EventStorage/ConnectorIE" />
import Writer = require('Lib/ServerEvent/native/EventStorage/Writer');
import Reader = require('Lib/ServerEvent/native/EventStorage/Reader');
import Deferred = require('Core/Deferred');

/**
 * Создан для соединения по IE
 * @class Lib/ServerEvent/native/EventStorage/ConnectorIE
 */
class ConnectorIE {
    static DB_NAME = "sbis.seb";
    static STORE_NAME = "events";

    /**
     * @param {Core/Deferred} def
     * @return {Core/Deferred<ConnectorIE>}
     */
    static connect(): Deferred<ConnectorIE> {
        let def = new Deferred<ConnectorIE>();
        if (typeof indexedDB !== 'object') {
            return Deferred.fail('В окружении нет indexedDB');
        }
        let request;
        try {
            request = indexedDB.open(ConnectorIE.DB_NAME, 1);
        } catch (e) {
            console.warn(e);
            def.errback(e);
            return def;
        }
        request.onupgradeneeded = function (event) {
            let db = this.result;
            db.onerror = (event) => {
                console.error("Database error: " + event.target.errorCode);
            };
            new Writer(db, ConnectorIE.STORE_NAME).createStore();
        };
        request.onsuccess = function (event) {
            let db = this.result;
            def.callback(new ConnectorIE(db));
        };
        request.onerror = function (event) {
            console.warn("Почему Вы не позволяете online.sbis.ru использовать IndexedDB?!");
            def.errback("You haven't rights to use IndexDB.");
            /**
             * https://github.com/localForage/localForage/issues/195#issuecomment-185135074
             */
            return true;
        };

        return def;
    }

    constructor(private db: IDBDatabase) {
    }

    /**
     * @return {Lib/ServerEvent/native/EventStorage/Writer}
     */
    getWriter(): Writer {
        return new Writer(this.db, ConnectorIE.STORE_NAME);
    }

    /**
     * @return {Lib/ServerEvent/native/EventStorage/Reader}
     */
    getReader(): Reader {
        return new Reader(this.db, ConnectorIE.STORE_NAME);
    }
}

export = ConnectorIE;