/// <reference path="../../resources/stomp.d.ts" />
/// <amd-module name="Lib/ServerEvent/native/EventStorage/Reader" />
define("Lib/ServerEvent/native/EventStorage/Reader", ["require", "exports"], function (require, exports) {
    "use strict";
    /**
     * @class Lib/ServerEvent/native/EventStorage/Reader
     * @memberOf module:ServerEvent.native.EventStorage
     */
    var Reader = /** @class */ (function () {
        function Reader(db, storeName) {
            this.db = db;
            this.storeName = storeName;
        }
        Reader.prototype.query = function (channelFilter) {
            var _this = this;
            return new Promise(function (resolve, reject) {
                var request = _this.db.transaction(_this.storeName, "readonly")
                    .objectStore(_this.storeName)
                    .openCursor();
                var results = [];
                request.onsuccess = function (event) {
                    var cursor = request.result;
                    if (!cursor) {
                        return resolve(results);
                    }
                    var value = cursor.value;
                    if (value) {
                        results.push(cursor.value);
                    }
                    cursor.continue();
                };
                request.onerror = reject;
            }).then(function (data) {
                if (!channelFilter) {
                    return data;
                }
                return data.filter(function (item) {
                    var channel;
                    if (item["headers"] && item["headers"]["event-type"]) {
                        channel = item["headers"]["event-type"];
                    }
                    return channel.match(channelFilter);
                });
            });
        };
        return Reader;
    }());
    return Reader;
});
