/// <reference path="../../resources/stomp.d.ts" />
/// <amd-module name="Lib/ServerEvent/native/EventStorage/Reader" />

/**
 * @class Lib/ServerEvent/native/EventStorage/Reader
 * @memberOf module:ServerEvent.native.EventStorage
 */
class Reader
{
    constructor(private db: IDBDatabase, private storeName: string) {
    }

    query(channelFilter?: string|RegExp): Promise<Object[]> {
        return new Promise<Object[]>((resolve, reject) => {
            let request = this.db.transaction(this.storeName, "readonly")
                .objectStore(this.storeName)
                .openCursor();

            let results = [];
            request.onsuccess = function (event) {
                let cursor = request.result;
                if (!cursor) {
                    return resolve(results);
                }
                let value = cursor.value;
                if (value) {
                    results.push(cursor.value);
                }
                cursor.continue();
            };

            request.onerror = reject;
        }).then((data) => {
            if (!channelFilter) {
                return data;
            }
            return data.filter((item) => {
                let channel;
                if (item["headers"] && item["headers"]["event-type"]) {
                    channel = item["headers"]["event-type"];
                }
                return channel.match(channelFilter);
            });
        });
    }
}

export = Reader;