/// <reference path="../../resources/stomp.d.ts" />
/// <amd-module name="Lib/ServerEvent/native/EventStorage/Writer" />
define("Lib/ServerEvent/native/EventStorage/Writer", ["require", "exports"], function (require, exports) {
    "use strict";
    /**
     * Класс, который записывает Stomp данные в IndexDB
     * @class Lib/ServerEvent/native/EventStorage/Writer
     * @memberOf module:ServerEvent.native.EventStorage
     */
    var Writer = /** @class */ (function () {
        /** @constructor */
        function Writer(db, storeName) {
            this.db = db;
            this.storeName = storeName;
            this.garbageCollection = this.garbageCollection.bind(this);
            setInterval(this.garbageCollection, Writer.GC_INTERVAL);
        }
        Writer.prototype.write = function (message) {
            var key = Date.now();
            var data = {
                headers: message.headers,
                body: message.body,
                command: message.command
            };
            data[Writer.STORE_KEY] = key;
            try {
                this.db.transaction(this.storeName, "readwrite")
                    .objectStore(this.storeName)
                    .put(data);
            }
            catch (e) {
                /* Fx bug: https://bugzilla.mozilla.org/show_bug.cgi?id=781982
                   ничего сделать не можем */
            }
        };
        Writer.prototype.createStore = function () {
            try {
                this.db.createObjectStore(this.storeName, {
                    keyPath: Writer.STORE_KEY,
                    autoIncrement: false
                });
            }
            catch (e) {
                /* Fx bug: https://bugzilla.mozilla.org/show_bug.cgi?id=781982
                 ничего сделать не можем */
            }
        };
        Writer.prototype.garbageCollection = function () {
            try {
                var store_1 = this.db.transaction(this.storeName, "readwrite")
                    .objectStore(this.storeName);
                var request_1 = store_1.count();
                request_1.onsuccess = function () {
                    if (request_1.result <= 100) {
                        return;
                    }
                    store_1.delete(IDBKeyRange.upperBound(Date.now() - Writer.GC_EVENT_LIMIT));
                };
                request_1.onerror = function () {
                };
            }
            catch (e) {
                /* Fx bug: https://bugzilla.mozilla.org/show_bug.cgi?id=781982
                 ничего сделать не можем */
            }
        };
        Writer.GC_INTERVAL = 5000;
        Writer.STORE_KEY = "timestamp";
        Writer.GC_EVENT_LIMIT = 5 * 60 * 1000;
        return Writer;
    }());
    return Writer;
});
