/// <reference path="../../resources/stomp.d.ts" />
/// <amd-module name="Lib/ServerEvent/native/EventStorage/Writer" />

/**
 * Класс, который записывает Stomp данные в IndexDB
 * @class Lib/ServerEvent/native/EventStorage/Writer
 * @memberOf module:ServerEvent.native.EventStorage
 */
class Writer
{
    static GC_INTERVAL = 5000;
    static STORE_KEY = "timestamp";
    static GC_EVENT_LIMIT = 5 * 60 * 1000;

    /** @constructor */
    constructor(private db: IDBDatabase, private storeName: string) {
        this.garbageCollection = this.garbageCollection.bind(this);
        setInterval(this.garbageCollection, Writer.GC_INTERVAL);
    }

    write(message: Stomp.Message) {
        let key = Date.now();
        let data = {
            headers: message.headers,
            body: message.body,
            command: message.command
        };
        data[Writer.STORE_KEY] = key;
        try {
            this.db.transaction(this.storeName, "readwrite")
                .objectStore(this.storeName)
                .put(data);
        } catch (e) {
          /* Fx bug: https://bugzilla.mozilla.org/show_bug.cgi?id=781982
             ничего сделать не можем */
        }
    }

    createStore() {
        try {
            this.db.createObjectStore(this.storeName, {
                keyPath: Writer.STORE_KEY,
                autoIncrement: false
            });
        } catch (e) {
          /* Fx bug: https://bugzilla.mozilla.org/show_bug.cgi?id=781982
           ничего сделать не можем */
        }
    }

    private garbageCollection() {
        try {
            let store = this.db.transaction(this.storeName, "readwrite")
                .objectStore(this.storeName);
            let request = store.count();
            request.onsuccess = () => {
                if (request.result <= 100) {
                    return;
                }
                store.delete(IDBKeyRange.upperBound(Date.now() - Writer.GC_EVENT_LIMIT));
            };
            request.onerror = () => {
            };
        } catch (e) {
          /* Fx bug: https://bugzilla.mozilla.org/show_bug.cgi?id=781982
           ничего сделать не можем */
        }
    }
}

export = Writer;