/* eslint-disable no-console */

var path = require('path');
var cwd = process.cwd();
var async = require(path.join(cwd, 'node_modules', 'async'));
var modPacker = require(path.join(cwd, 'node_modules', 'grunt-wsmod-packer'));
var common = require(path.join(cwd, 'lib', 'common'));
var cm = require(path.join(cwd, 'lib', 'content-manager'));
var dblSlashes = /\\/g;

//cache
var packedFilesCache = {};

module.exports = function (Component, Service) {
   var root = Service.getConfigRoot();
   var appRoot = Service.getRoot();
   var createLinks = function (name) {
      return path.join(Service.getResourcesUrl(), common.cacheFolderName, name).replace(dblSlashes, '/');
   };

   return {
      '/get_package/': function (req, res) {
         var config = {};

         try {
            config.modules = (req.query.modules ? JSON.parse(req.query.modules) : []).sort();
            config.exclude = (req.query.exclude ? JSON.parse(req.query.exclude) : []).sort();
            config.include = (req.query.include ? JSON.parse(req.query.include) : []).sort();
            config.includeCore = (req.query.includeCore ? req.query.includeCore : undefined);
         } catch (err) {
            console.log(err.stack);
            return res.status(500).send(err.stack);
         }

         packedFilesCache[appRoot] = packedFilesCache[appRoot] || {};
         var key = JSON.stringify(config);
         var names = packedFilesCache[appRoot][key];

         if (names) {
            res.json(names);
         } else {
            modPacker.customPack(config, root, appRoot, function (err, files) {
               var
                  jsName = '',
                  filesToSave = [],
                  packedCSSBatches;

               if (err) {
                  console.log(err.stack);
                  res.status(500).send(err.stack);
               } else {
                  if (files.js) {
                     jsName = common.uniqFileName(files.js, "js");

                     filesToSave.push({
                        data: files.js,
                        name: jsName
                     });
                  }

                  if (files.css) {
                     packedCSSBatches = files.css.map(function (item) {
                        var cssName = common.uniqFileName(item, "css");

                        filesToSave.push({
                           data: item,
                           name: cssName
                        });

                        return createLinks(cssName);
                     });
                  }

                  async.each(filesToSave, function (file, cb) {
                     cm.saveResource(file, function () {
                        cb();
                     });
                  }, function () {
                     // вот теперь можно добавить поле в кэш
                     packedFilesCache[appRoot][key] = {};

                     if (files.js) {
                        packedFilesCache[appRoot][key].js = createLinks(jsName);
                     }
                     if (files.css) {
                        packedFilesCache[appRoot][key].css = packedCSSBatches;
                     }
                     res.json(packedFilesCache[appRoot][key]);
                  });
               }
            });
         }
      }
   }
};