'use strict';

var path = require('path');
var DepGraph = require('sbis3-dependency-graph');
var cache = {};
var smartCache = {};

module.exports = function (Component, Service) {
    var fn;
    try {
        var
            resourceRoot = Service.getResourcesRoot(),
            modDeps = global.requirejs('json!Resources/module-dependencies.json'),
            dg = new DepGraph();

        dg.fromJSON(modDeps);

        fn = function (req, res) {
            try {
                var
                    modules = JSON.parse(req.query.modules).sort(),
                    lightload = req.query.lightload,
                    smartCollect = !!req.query.smartcollect || !!req.query.smartCollect,
                    key = modules.join('') + ' | ' + !!lightload,
                    value;


                value = smartCollect ? smartCache[key] : cache[key];
                if (value) {
                    res.json(value);
                } else {


                    modules = modules.map(function (module) {
                        if (module.indexOf('SBIS3') == 0) {
                            return 'js!' + module;
                        }

                        return module;
                    });

                    if (lightload && !smartCollect) {
                        dg.getNodesToLoad(modules, ["html!SBIS3.*", "css!SBIS3.*", "Core/*", "Transport/*"]);
                    }
                    if (smartCollect) {
                        dg.firstLevelReached = false;
                        res.json(smartCache[key] = dg.getLoadOrder(modules, null, smartCollect));
                        if (dg.smartNodes && dg.smartNodes.length) {
                            dg.smartNodes.forEach(function(smartNode) {
                                delete(dg._nodes[smartNode]);
                            });

                        }

                    }
                    else {
                         if (dg.smartNodes && dg.smartNodes.length) {
                            dg.smartNodes.forEach(function(smartNode) {
                                delete(dg._nodes[smartNode]);
                            });
                        }
                        res.json(cache[key] = dg.getLoadOrder(modules, null, smartCollect));
                    }
                }
            } catch (err) {
                res.status(500).send(err.message);
            }
        };
    } catch (err) {
        var ioc = global.requirejs('Core/IoC');
        ioc.resolve('ILogger').error('DependenciesResolver.routes.js', err.message, err );
        fn = function (req, res) {
            res.status(500).send(err.message);
        };
    }

    return {
        '/depresolver/': fn
    }
};
