define('Transport/HTTPError', ['Core/core-extend'], function(extend) {
  /**
   * @class Transport/HTTPError
   * @author Бегунов А.В.
   * @public
   *
   */

  /**
   * @alias Transport/HTTPError
   * HTTP-ошибка
   * @param message Человекопонятное сообщение об ошибке.
   * @param httpError HTTP-код ошибки.
   * @param url Адрес.
   * @param payload.
   * @param [details]
   */
   function HTTPError(message, httpError, url, payload, details) {
      this.message = message || '';
      this.details = details || 'Мы знаем об этом и уже исправляем. Попробуйте заглянуть сюда через 15 минут.';
      this.name = 'HTTP error';
      this.httpError = httpError === undefined ? '' : httpError;
      this.url = url || '';
      this.payload = payload || '';
      this.processed = false;
   }

    /**
     * Ошибка офлайна
     * @param message Сообщение об ошибке.
     * @param url Адрес.
     */
   function OfflineError(message, url){
       this.url = url || '';
       this.message = message || '';
       this.processed = false;
       this._isOfflineMode = true;
   }
   /**
    * Наследуем HTTPError от Error
    */
   extend.classicExtend(HTTPError, Error);
   /*
        Не наследуем от HTTPError т.к. могут перехватить ошибку по признаку наследования
        и вернуть другую ошибку
        прим: RPCJSON обрабатывает hht-ошибки, возвращая TransportError,
        конечный пользователь не поймает флаг офлайна
     */
   extend.classicExtend(OfflineError, Error);

   HTTPError.prototype.toString = function() {
      return [this.name, ': ', this.message, '; httpError: ', this.httpError, '; url: ', this.url].join('');
   };

   HTTPError.offlineModeError = function (message, url) {
      return new OfflineError(message, url);
   };

   return HTTPError;
});