define('Transport/TransportError', ['Core/core-extend', 'Transport/HTTPError'], function(extend, HTTPError) {
   /**
    * @class Transport/TransportError
    * @extends Transport/HTTPError
    * @author Бегунов А.В.
    * @public
    *
    */

   /**
    * @alias Transport/TransportError
    * Конструктор класса Transport/TransportError
    * @param {String} message - текст ошибки.
    * @param {String} [httpError] - код HTTP ошибки.
    * @param {Number} [code] - код ошибки бизнес-логики.
    * @param {String} [methodName] - имя вызванного метода бизнес-логики.
    * @param {String} [details] - детальное описание ошибки.
    * @param {String} [url] - адрес, который загружался.
    * @param {String} [classid]
    * @param {String} [errType] - Тип ошибки.
    * @param {String} [addinfo] - Доп информация.
    * @param {String} [error_code] - код ошибки прикладников
    */
   function TransportError(message, httpError, code, methodName, details, url, classid, errType, addinfo, error_code) {
      this.message = message;
      this.httpError = httpError === undefined ? "" : httpError;
      this.code = code || 0;
      this.name = 'Transport error';
      this.methodName = methodName || "";
      this.details = details || "";
      this.url = url || '';
      this.classid = classid || '';
      this.errType = errType || 'error';
      this.addinfo = addinfo || '';
      this.error_code = error_code;
   }

   /**
    * Наследуем TransportError от HTTPError
    */
   extend.classicExtend(TransportError, HTTPError);
   /**
    * Переопределяем метод toString
    */
   TransportError.prototype.toString = function() {
      return [
         this.name, ': ', this.message, '; method: ', this.methodName, '; code: ', this.code, '; httpError: ',
         this.httpError, '; details: ', this.details
      ].join('');
   };

   var global = (function(){ return this || (0,eval)('this'); }());
   global.TransportError = TransportError;

   return TransportError;
});